unit RRPGBaseDataSet;

interface

uses
  Classes, SysUtils, DB;
                                           //IBCustomDataSet
type
  TVariantArray = array of Variant;

  TRMemoryStream = class(TMemoryStream)
  public
    constructor Create;
    destructor Destroy; override;

    function ObterString: AnsiString;
  end;

  TMStreamArray = array of TRMemoryStream;

  TBufferRecord = packed record
    RecNo: Integer;
    Campos: TVariantArray;
    Calculados: TVariantArray;
    Blobs: TMStreamArray;
  end;

  PBufferRecord = ^TBufferRecord;

  TInnerDataSetRecord = packed record
    // Indice do Buffer do Registro. ndices negativos possuem valores especiais.
    // Veja a sesso de constantes BUF_REG_
    BufferRegistro: Integer;
  end;

  PInnerDataSetRecord = ^TInnerDataSetRecord;
  TBufferDataSet = array of TBufferRecord;

  {$IFDEF VER150}
    PDataSetBufferPoint = PAnsiChar;
    TTipoBookmark = Pointer;
  {$ELSE}
    PDataSetBufferPoint = TRecordBuffer;
    TTipoBookmark = TBytes;
  {$ENDIF}

  TRCustomDataSet = class;

  TRBlobStream = class(TStream)
  private
    FMStream: TStream;
    FDataSet: TRCustomDataSet;
    FPosicao: Int64;
    FAntigaPosicao: Int64;
    procedure Posicionar;
    procedure VoltarPosicao;
  protected
  public
    constructor Create(Stream: TStream; DS: TRCustomDataSet);

    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; override;
    function GetSize: Int64; override;
    procedure SetSize(NewSize: Longint); override;
    procedure SetSize(const NewSize: Int64); override;        
  end;

  TRCustomDataSet = class(TDataSet)
  private
    FBuffer: TBufferDataSet;  // Lista de registros originais
    FTamanhoBuffer: Integer;
    FBufferEdicao: TBufferRecord; // Buffer usado ao inserir novo/editar registro

    FRegistroAtual: Integer;
    FAberto: Boolean;

    FAntigoBufferRegistro: Integer;
    FRegistrosInseridos: Integer;
    FRegistrosDeletados: Integer;
    FCountCamposFisicos: Integer;

  protected
    // Suporte ao TDataSet
    procedure ClearCalcFields(Buffer: PDataSetBufferPoint); override;
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
    function GetRecordSize: Word; override;
    procedure InternalAddRecord(Buffer: Pointer; Append: Boolean); override;
    function GetRecordCount: Integer; override;
    function GetRecord(Buffer: PDataSetBufferPoint; GetMode: TGetMode;
      DoCheck: Boolean): TGetResult; override;

    // TRCustomDataSet
    procedure InicializarRegistro(Buf: PBufferRecord); virtual;
    procedure LiberarRegistro(Buf: PBufferRecord); virtual;

    function InternoAlimentarProximoRecordBuffer(var Buf: TBufferRecord): Boolean; virtual;
    function InternoGetFieldCountForBuffer: Integer; virtual;
    function GarantirRegistro(const Indice: Integer): PBufferRecord; virtual;
    function GetRecNo: Integer; override;
    procedure SetRecNo(Value: Integer); override;
    procedure InternalSetToRecord(Buffer: PDataSetBufferPoint); override;
    function AllocRecordBuffer: PDataSetBufferPoint; override;
    procedure FreeRecordBuffer(var Buffer: PDataSetBufferPoint); override;
    procedure InternalInitRecord(Buffer: PDataSetBufferPoint); override;
    procedure InternalInsert; override;
    procedure SetFieldData(Field: TField; Buffer: Pointer); override;
    procedure InternalFirst; override;
    procedure InternalLast; override;
    procedure InternalEdit; override;    

    procedure CopiarRecord(Source, Dest: PBufferRecord); virtual;
    procedure InternalPost; override;
    procedure CopiarRegistroEmEdicaoParaBuffer; virtual;
    procedure CopiarRegistroEmInsercaoParaBuffer; virtual;

    procedure ExecutarRefreshInterno(Buf: PBufferRecord); virtual;
    procedure InternalRefresh; override;
    procedure DataConvert(Field: TField; Source: Pointer; Dest: Pointer;
      ToNative: Boolean); override;

    procedure AdicionarBufferRecord(Registro: TBufferRecord); virtual;
    procedure SetBookmarkFlag(Buffer: PDataSetBufferPoint; Value: TBookmarkFlag); override;
    function GetBookmarkFlag(Buffer: PDataSetBufferPoint): TBookmarkFlag; override;
    procedure GetBookmarkData(Buffer: PDataSetBufferPoint; Data: Pointer); override;
    procedure SetBookmarkData(Buffer: PDataSetBufferPoint; Data: Pointer); override;
    procedure InternalDelete; override;
    procedure InternalGotoBookmark(Bookmark: Pointer); override;

    procedure RemoverRegistroDataSet; virtual;

    property RegistrosInseridos: Integer read FRegistrosInseridos;
    property RegistrosDeletados: Integer read FRegistrosDeletados;
    property Buffer: TBufferDataSet read FBuffer write FBuffer;
    property CountCamposFisicos: Integer read FCountCamposFisicos write FCountCamposFisicos;
    property TamanhoBuffer: Integer read FTamanhoBuffer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream; override;

    function GetFieldData(Field: TField; Buffer: Pointer): Boolean; override;
    function GetCurrentRecord(Buffer: PDataSetBufferPoint): Boolean; override;
    function BookmarkValid(Bookmark: TTipoBookmark): Boolean; override;
  published
    property AutoCalcFields;
  end;

const
  BUF_REG_BOF = -1;
  BUF_REG_INEXISTENTE = -2;
  BUF_REG_NOVO = -3;
  BUF_REG_EM_EDICAO = -4;

  FETCH_ALL = MaxInt - 1;
  
implementation

uses
  Variants, RRPGDataSetUtil;

var
  _CountMStream: Integer = 0;

{ TMCustomDataSet }

procedure TRCustomDataSet.AdicionarBufferRecord(Registro: TBufferRecord);
var
  TamRealBuffer: Integer;
begin
  TamRealBuffer := Length(FBuffer);

  if FTamanhoBuffer + 1 > TamRealBuffer then
    SetLength(FBuffer, TamRealBuffer + (TamRealBuffer div 2) + 1);

  Inc(FTamanhoBuffer);    
  FBuffer[FTamanhoBuffer - 1] := Registro;
end;

function TRCustomDataSet.AllocRecordBuffer: PDataSetBufferPoint;
var
  P: PInnerDataSetRecord;
begin
  GetMem(P, Self.RecordSize);
  P^.BufferRegistro := BUF_REG_INEXISTENTE;
  Pointer(Result) := P;
end;

function TRCustomDataSet.BookmarkValid(Bookmark: TTipoBookmark): Boolean;
begin
  {$IFDEF VER150}
    Result := (Bookmark <> nil);
  {$ELSE}
    Result := Length(Bookmark) > 0;
  {$ENDIF}
end;

procedure TRCustomDataSet.ClearCalcFields(Buffer: PDataSetBufferPoint);
var
  Inner: PInnerDataSetRecord;
  Rec: PBufferRecord;
  i: Integer;
begin
  inherited;
  Inner := Pointer(Buffer);

  if Inner = nil then
    Exit;

  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);

  if Rec = nil then
    Exit;

  for i := 0 to Length(Rec^.Calculados) - 1 do
    Rec^.Calculados[i] := Null;
end;

procedure TRCustomDataSet.CopiarRecord(Source, Dest: PBufferRecord);
var
  i: Integer;
begin
  if Source <> nil then
  begin
    Dest^.RecNo := Source^.RecNo;
    SetLength(Dest^.Campos, Length(Source^.Campos));
    SetLength(Dest^.Calculados, Length(Source^.Calculados));
    SetLength(Dest^.Blobs, Length(Source^.Blobs));

    for i := 0 to Length(Source^.Campos) - 1 do
      Dest^.Campos[i] := Source^.Campos[i];

    for i := 0 to Length(Source^.Calculados) - 1 do
      Dest^.Calculados[i] := Source^.Calculados[i];

    for i := 0 to Length(Source^.Blobs) - 1 do
    begin
      if Source^.Blobs[i] = nil then
      begin
        if Dest^.Blobs[i] <> nil then
          FreeAndNil(Dest^.Blobs[i]);
      end else
      begin
        if Dest^.Blobs[i] = nil then
          Dest^.Blobs[i] := TRMemoryStream.Create;

        Source^.Blobs[i].Position := 0;
        Dest^.Blobs[i].Clear;
        Dest^.Blobs[i].CopyFrom(Source^.Blobs[i], Source^.Blobs[i].Size);
      end;
    end;
  end else
    Self.InicializarRegistro(Dest);
end;

procedure TRCustomDataSet.CopiarRegistroEmEdicaoParaBuffer;
var
  Inner: PInnerDataSetRecord;
  Rec1, Rec2: PBufferRecord;
begin
  Inner := Pointer(Self.ActiveBuffer);
  Rec1 := Self.GarantirRegistro(Inner^.BufferRegistro);
  Rec2 := Self.GarantirRegistro(FAntigoBufferRegistro);
  Self.CopiarRecord(Rec1, Rec2);
end;

procedure TRCustomDataSet.CopiarRegistroEmInsercaoParaBuffer;
var
  Inner: PInnerDataSetRecord;
  Rec1: PBufferRecord;
  NovoRec: TBufferRecord;
begin
  Inner := Pointer(Self.ActiveBuffer);
  Rec1 := Self.GarantirRegistro(Inner^.BufferRegistro);

  Self.GarantirRegistro(FETCH_ALL); // Executar um FetchAll Interno

  Self.InicializarRegistro(@NovoRec);
  Self.CopiarRecord(Rec1, @NovoRec);

  NovoRec.RecNo := FTamanhoBuffer;
  
  Self.AdicionarBufferRecord(NovoRec);

  FRegistroAtual := NovoRec.RecNo;   
  Inc(FRegistrosInseridos);
end;

constructor TRCustomDataSet.Create(AOwner: TComponent);
begin
  inherited;
  FRegistroAtual := -1;
  Self.BookmarkSize := SizeOf(Integer);
end;

function TRCustomDataSet.CreateBlobStream(Field: TField;
  Mode: TBlobStreamMode): TStream;
var
  Rec: PBufferRecord;
  M: TRMemoryStream;
begin
  Result := nil;

  if Self.ActiveRecord < 0 then
    Exit;

  if not Self.Active then
    Exit;

  Rec := Self.GarantirRegistro(PInnerDataSetRecord(Self.ActiveBuffer)^.BufferRegistro);

  if (Rec <> nil) and (Field.IsBlob) then
  begin
    M := Rec^.Blobs[Field.Offset];

    if Mode in [bmReadWrite, bmWrite] then
      if M = nil then
      begin
        M := TRMemoryStream.Create;
        Rec^.Blobs[Field.Offset] := M;
      end;

    if Mode = bmWrite then
      if M <> nil then
        M.Clear;

    Result := TRBlobStream.Create(M, Self);
  end else
    Result := TRBlobStream.Create(nil, Self);
end;

procedure TRCustomDataSet.DataConvert(Field: TField; Source, Dest: Pointer;
  ToNative: Boolean);
begin
  case Field.DataType of
    ftBCD: Currency(Dest^) := Currency(Source^);
    ftDateTime: TDateTime(Dest^) := TDateTime(Source^);

    else
      inherited;
  end    
end;

destructor TRCustomDataSet.Destroy;
begin

  inherited;
end;

procedure TRCustomDataSet.ExecutarRefreshInterno(Buf: PBufferRecord);
begin

end;

procedure TRCustomDataSet.FreeRecordBuffer(var Buffer: PDataSetBufferPoint);
begin
  inherited;
  FreeMem(Buffer, SizeOf(TInnerDataSetRecord));
end;

function TRCustomDataSet.GarantirRegistro(const Indice: Integer): PBufferRecord;
var
  Fetches: Integer;
  i: Integer;
  R: TBufferRecord;
begin
  Result := nil;

  if Indice < 0 then
  begin;
    case Indice of
      BUF_REG_NOVO: Result := @FBufferEdicao;

      BUF_REG_EM_EDICAO:
        if Self.State = dsOldValue then
          Result := Self.GarantirRegistro(FAntigoBufferRegistro)
        else
          Result := @FBufferEdicao;
    end;

    Exit;
  end;

  if Indice <= FTamanhoBuffer - 1 then
  begin
    Result := @FBuffer[Indice];
    Exit;
  end;

  Fetches := Indice - FTamanhoBuffer + 1;

  for i := 1 to Fetches do
  begin
    Self.InicializarRegistro(@R);
    R.RecNo := FTamanhoBuffer;

    if Self.InternoAlimentarProximoRecordBuffer(R) then
    begin
      Self.AdicionarBufferRecord(R);
    end else
    begin
      Self.LiberarRegistro(@R);
      Exit;
    end;
  end;

  Result := @FBuffer[Indice];
end;

procedure TRCustomDataSet.GetBookmarkData(Buffer: PDataSetBufferPoint; Data: Pointer);
var
  Inner: PInnerDataSetRecord;
begin
  inherited;
  Inner := Pointer(Buffer);
  PInteger(Data)^ := Inner.BufferRegistro;
end;

function TRCustomDataSet.GetBookmarkFlag(Buffer: PDataSetBufferPoint): TBookmarkFlag;
var
  Inner: PInnerDataSetRecord;
begin
  inherited GetBookmarkFlag(Buffer);
  Inner := Pointer(Buffer);

  case Inner^.BufferRegistro of
    BUF_REG_INEXISTENTE, BUF_REG_BOF: Result := bfBOF;
    BUF_REG_NOVO: Result := bfInserted;

    else
    begin
      if Inner^.BufferRegistro = FTamanhoBuffer then
        Result := bfEOF
      else
        Result := bfCurrent;  
    end;
  end;
end;

function TRCustomDataSet.GetCurrentRecord(Buffer: PDataSetBufferPoint): Boolean;
var
  Inner: PInnerDataSetRecord;
begin
  if not IsEmpty then
  begin
    Self.UpdateCursorPos;
    Inner := Pointer(Buffer);
    Inner.BufferRegistro := PInnerDataSetRecord(Self.ActiveBuffer)^.BufferRegistro;
    Result := True;
  end else
    Result := False;
end;

function TRCustomDataSet.GetFieldData(Field: TField;
  Buffer: Pointer): Boolean;
var
  Rec: PBufferRecord;
  Valor, ValorDest: Variant;
  Tam: Integer;
begin
  Result := False;

  if Self.ActiveRecord < 0 then
    Exit;

  if not Self.Active then
    Exit;

  Rec := Self.GarantirRegistro(PInnerDataSetRecord(Self.ActiveBuffer)^.BufferRegistro);

  if Rec <> nil then
  begin
    if not Field.IsBlob then
    begin
      if Field.FieldNo > 0 then
        Valor := Rec^.Campos[Field.FieldNo - 1]
      else
        Valor := Rec^.Calculados[Field.Offset]
    end else
    begin
      if Rec^.Blobs[Field.Offset] = nil then
        Result := False
      else
        Result := True;

      if Buffer = nil then
        Exit;

      if Result then
        Valor := Rec^.Blobs[Field.Offset].ObterString();
    end;

    ValorDest := ConverterVariant(Valor, FieldTypeToVariantType(Field.DataType));

    Tam := Field.DataSize;
    Result := EscreverVariantParaBuffer(ValorDest, Buffer^, Tam, Field);
  end;
end;

function TRCustomDataSet.GetRecNo: Integer;
var
  Inner: PInnerDataSetRecord;
  Rec: PBufferRecord;
begin
  Inner := Pointer(Self.ActiveBuffer);
  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);

  if Rec <> nil then
    Result := Rec.RecNo
  else
    Result := 0;
end;

function TRCustomDataSet.GetRecord(Buffer: PDataSetBufferPoint; GetMode: TGetMode;
  DoCheck: Boolean): TGetResult;
var
  Ptr: PBufferRecord;
  Inner: PInnerDataSetRecord;
begin
  Inner := Pointer(Buffer);
  Result := grOK;

  case GetMode of
    gmCurrent:;
    gmNext:
    begin
      if FRegistroAtual = BUF_REG_NOVO then
        FRegistroAtual := FAntigoBufferRegistro;
    
      Inc(FRegistroAtual);
    end;
    
    gmPrior: 
    begin
      if FRegistroAtual = BUF_REG_NOVO then
        FRegistroAtual := FAntigoBufferRegistro;
        
      Dec(FRegistroAtual);
    end;
  end;

  if FRegistroAtual < -1 then
    FRegistroAtual := -1;

  Ptr := Self.GarantirRegistro(FRegistroAtual);

  if Ptr = nil then
  begin
    if GetMode = gmPrior then
      Result := grBOF
    else
      Result := grEOF;

    Exit;
  end;

  Inner^.BufferRegistro := FRegistroAtual;
end;

function TRCustomDataSet.GetRecordCount: Integer;
begin
  Result := FTamanhoBuffer;
end;

function TRCustomDataSet.GetRecordSize: Word;
begin
  Result := SizeOf(TInnerDataSetRecord);
end;

procedure TRCustomDataSet.InicializarRegistro(Buf: PBufferRecord);
var
  i: Integer;
begin
  SetLength(Buf^.Campos, Self.InternoGetFieldCountForBuffer());
  SetLength(Buf^.Calculados, Self.CalcFieldsSize);
  SetLength(Buf^.Blobs, Self.BlobFieldCount);

  for i := 0 to Length(Buf^.Campos) - 1 do
    Buf^.Campos[i] := Null;

  for i := 0 to Length(Buf^.Calculados) - 1 do
    Buf^.Calculados[i] := Null;

  for i := 0 to Length(Buf^.Blobs) - 1 do
  begin
    if Buf^.Blobs[i] <> nil then
      FreeAndNil(Buf^.Blobs[i]);
  end;
end;

procedure TRCustomDataSet.InternalAddRecord(Buffer: Pointer; Append: Boolean);
begin
  inherited;

end;

procedure TRCustomDataSet.InternalClose;
var
  i: Integer;
begin
  inherited;

  Self.LiberarRegistro(@FBufferEdicao);

  for i := 0 to Length(FBuffer) - 1 do
    Self.LiberarRegistro(@FBuffer[i]);

  SetLength(FBuffer, 0);
  FTamanhoBuffer := 0;

  FRegistrosInseridos := 0;
  FRegistrosDeletados := 0;
  FRegistroAtual := -1;
end;

procedure TRCustomDataSet.InternalDelete;
begin
  inherited;

end;

procedure TRCustomDataSet.InternalEdit;
var
  Inner: PInnerDataSetRecord;
  Rec, RecEdit: PBufferRecord;
begin
  inherited;
  Inner := Pointer(Self.ActiveBuffer);

  if Inner = nil then
    Exit;

  FAntigoBufferRegistro := Inner^.BufferRegistro;

  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);
  Inner^.BufferRegistro := BUF_REG_EM_EDICAO;
  RecEdit := Self.GarantirRegistro(Inner^.BufferRegistro);
  Self.CopiarRecord(Rec, RecEdit);
end;

procedure TRCustomDataSet.InternalFirst;
begin
  inherited;
  FRegistroAtual := -1;
end;

procedure TRCustomDataSet.InternalGotoBookmark(Bookmark: Pointer);
begin
  inherited;
  FRegistroAtual := PInteger(Bookmark)^;
end;

procedure TRCustomDataSet.InternalHandleException;
begin
  inherited;

end;

procedure TRCustomDataSet.InternalInitFieldDefs;
begin
end;

procedure TRCustomDataSet.InternalInitRecord(Buffer: PDataSetBufferPoint);
var
  Inner: PInnerDataSetRecord;
begin
  inherited;
  Inner := Pointer(Buffer);

  if Inner <> nil then
    Inner^.BufferRegistro := BUF_REG_INEXISTENTE;
end;

procedure TRCustomDataSet.InternalInsert;
var
  Inner: PInnerDataSetRecord;
  Rec: PBufferRecord;
begin
  inherited;
  Inner := Pointer(Self.ActiveBuffer);

  FAntigoBufferRegistro := Inner^.BufferRegistro;
  Inner^.BufferRegistro := BUF_REG_NOVO;   
  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);
  Self.InicializarRegistro(Rec);

  Self.CursorPosChanged();
end;

procedure TRCustomDataSet.InternalLast;
begin
  inherited;
  Self.GarantirRegistro(FETCH_ALL);
  FRegistroAtual := FTamanhoBuffer - 1 + 1; // Temos que apontar para EOF
end;

procedure TRCustomDataSet.InternalOpen;
begin
  inherited;
  FRegistrosInseridos := 0;
  FRegistrosDeletados := 0;
  FTamanhoBuffer := 0;
end;

procedure TRCustomDataSet.InternalPost;
begin
  inherited;

end;

procedure TRCustomDataSet.InternalRefresh;
var
  Inner: PInnerDataSetRecord;
  Rec: PBufferRecord;
begin
  inherited;
  Inner := Pointer(Self.ActiveBuffer);
  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);

  if Rec <> nil then
    Self.ExecutarRefreshInterno(Rec);

  Self.GetCalcFields(Self.ActiveBuffer);
end;

procedure TRCustomDataSet.InternalSetToRecord(Buffer: PDataSetBufferPoint);
var
  Inner: PInnerDataSetRecord;
begin
  inherited;
  Inner := Pointer(Buffer);
  FRegistroAtual := Inner^.BufferRegistro;
end;

function TRCustomDataSet.InternoAlimentarProximoRecordBuffer(var Buf: TBufferRecord): Boolean;
begin
  Result := False;
end;

function TRCustomDataSet.InternoGetFieldCountForBuffer: Integer;
begin
  Result := 0;
end;

function TRCustomDataSet.IsCursorOpen: Boolean;
begin
  Result := FAberto;
end;

procedure TRCustomDataSet.LiberarRegistro(Buf: PBufferRecord);
var
  i: Integer;
begin
  for i := 0 to Length(Buf^.Blobs) - 1 do
    FreeAndNil(Buf^.Blobs[i]);
end;

procedure TRCustomDataSet.RemoverRegistroDataSet;
var
  Inner: PInnerDataSetRecord;
  i: Integer;
begin
  Inner := Pointer(Self.ActiveBuffer);

  if (Inner^.BufferRegistro >= 0) and
     (Inner^.BufferRegistro <= FTamanhoBuffer - 1) then
  begin
    Self.LiberarRegistro(@FBuffer[Inner^.BufferRegistro]);

    for i := Inner^.BufferRegistro to FTamanhoBuffer - 2 do
      FBuffer[i] := FBuffer[i + 1];

    for i := 0 to Length(Buffer[FTamanhoBuffer - 1].Blobs) - 1 do
      Buffer[FTamanhoBuffer - 1].Blobs[i] := nil;

    Dec(FTamanhoBuffer);

    if FRegistroAtual > FTamanhoBuffer -1 then
      FRegistroAtual := FTamanhoBuffer -1;
  end;

  Inc(FRegistrosDeletados);
end;

procedure TRCustomDataSet.SetBookmarkData(Buffer: PDataSetBufferPoint; Data: Pointer);
var
  Inner: PInnerDataSetRecord;
begin
  inherited;
  Inner := Pointer(Buffer);

  if Data <> nil then
    Inner^.BufferRegistro := PInteger(Data)^
  else
    Inner^.BufferRegistro := BUF_REG_INEXISTENTE;  
end;

procedure TRCustomDataSet.SetBookmarkFlag(Buffer: PDataSetBufferPoint; Value: TBookmarkFlag);
var
  Inner: PInnerDataSetRecord;
begin
  inherited;
  Inner := Pointer(Buffer);

  case Value of
    bfCurrent: Inner^.BufferRegistro := FRegistroAtual;
    bfBOF: Inner^.BufferRegistro := BUF_REG_BOF;
    bfEOF:
    begin
      Self.GarantirRegistro(FETCH_ALL);
      Inner^.BufferRegistro := FTamanhoBuffer;
    end;
    
    bfInserted: Inner^.BufferRegistro := BUF_REG_NOVO;
  end;
end;

procedure TRCustomDataSet.SetFieldData(Field: TField; Buffer: Pointer);
var
  Inner: PInnerDataSetRecord;
  Rec: PBufferRecord;
  V: Variant;
begin
  inherited;
  Inner := Pointer(Self.ActiveBuffer);
  Rec := Self.GarantirRegistro(Inner^.BufferRegistro);

  if Rec <> nil then
  begin
    LerVariantDeBuffer(V, Buffer^, Field.DataSize, Field);

    if Field.FieldNo > 0 then
      Rec.Campos[Field.FieldNo - 1] := V
    else
      if not Field.IsBlob then
        Rec.Calculados[Field.Offset] := V
      else
        Rec.Blobs[Field.Offset].Clear;

    if not (Self.State in [dsCalcFields, dsFilter, dsNewValue]) then
      DataEvent(deFieldChange, Longint(Field));
  end;
end;

procedure TRCustomDataSet.SetRecNo(Value: Integer);
begin
  inherited;
  FRegistroAtual := Value;
  Resync([]);
end;

{ TRBlobStream }

constructor TRBlobStream.Create(Stream: TStream; DS: TRCustomDataSet);
begin
  FMStream := Stream;
  FDataSet := DS;
end;

function TRBlobStream.GetSize: Int64;
begin
  if FMStream = nil then
  begin
    Result := 0;
    Exit;
  end;

  Result := FMStream.Size;
end;

procedure TRBlobStream.Posicionar;
begin
  FAntigaPosicao := FMStream.Position;
  FMStream.Position := FPosicao;
end;

function TRBlobStream.Read(var Buffer; Count: Integer): Longint;
begin
  if FMStream = nil then
  begin
    Result := 0;
    Exit;
  end;

  Self.Posicionar;

  try
    Result := FMStream.Read(Buffer, Count);
    FPosicao := FPosicao + Result;
  finally
    Self.VoltarPosicao();
  end;
end;

function TRBlobStream.Seek(const Offset: Int64; Origin: TSeekOrigin): Int64;
begin
  if FMStream = nil then
  begin
    Result := 0;
    Exit;
  end;

  case Origin of
    soBeginning:
    begin
      FPosicao := Offset;
      Result := FPosicao;
    end;

    soCurrent:
    begin
      FPosicao := FPosicao + Offset;
      Result := FPosicao;
    end;

    soEnd:
    begin
      FPosicao := FMStream.Size + Offset;
      Result := FPosicao;
    end;

    else
      Result := 0;
  end;
end;

procedure TRBlobStream.SetSize(NewSize: Integer);
begin
  if FMStream <> nil then
    FMStream.Size := NewSize;
end;

procedure TRBlobStream.SetSize(const NewSize: Int64);
begin
  if FMStream <> nil then
    FMStream.Size := NewSize;
end;

procedure TRBlobStream.VoltarPosicao;
begin
  FMStream.Position := FAntigaPosicao;
end;

function TRBlobStream.Write(const Buffer; Count: Integer): Longint;
begin
  if FMStream = nil then
  begin
    Result := Count;
    Exit;
  end;

  Self.Posicionar();

  try
    Result := FMStream.Write(Buffer, Count);
    FPosicao := FPosicao + Result;
  finally
    Self.VoltarPosicao();
  end;
end;

{ TRMemoryStream }

constructor TRMemoryStream.Create;
begin
  inherited Create;
  Inc(_CountMStream);
end;

destructor TRMemoryStream.Destroy;
begin
  Dec(_CountMStream);
  inherited;
end;

function TRMemoryStream.ObterString: AnsiString;
var
  P: Int64;
begin
  SetLength(Result, Self.Size);
  P := Self.Position;
  Self.Position := 0;

  try
    Self.Read(Result[1], Self.Size);
  finally
    Self.Position := P;
  end;
end;

end.
