unit RRPGComboBox;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, RRPGURLGetter,
  RRPGUrlPicture, StdCtrls, ExtCtrls, RRPGColorScheme, CommCtrl,
  RRPGDrawUtil;  // DBCtrls

type
  TRComboBox = class(TComboBox)
  private
    FCanvasDraw: TCanvas;
    FMouseNoControle: Boolean;
    FMouseNoCombo: Boolean;
  protected
    procedure DesenharBorda(C: TCanvas; const Situacao: TSituacaoEsquema);  virtual;
    procedure DesenharSeta(C: TCanvas; Area: TRect; const Situacao: TSituacaoEsquema);  virtual;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseMove(Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure Change; override;
    function IsMouseSobreControle: Boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;

    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

const
  CB_GETCOMBOBOXINFO = $0164;

implementation

uses
  GraphUtil, UxTheme, RRPGUtil, Themes, GDIPlus, Math, Types;

{ TRComboBox }

procedure TRComboBox.Change;
begin
  inherited;

  if Self.Style = csDropDownList then
    Self.Invalidate;
end;

procedure TRComboBox.CMMouseEnter(var Message: TMessage);
var
  AntigoMouseNoControle: Boolean;
begin
  AntigoMouseNoControle := FMouseNoControle;

  if not ThemeControl(Self) then
    FMouseNoControle := IsMouseSobreControle()
  else
    FMouseNoControle := True;

  inherited;

  if (not ThemeControl(Self)) and (FMouseNoControle <> AntigoMouseNoControle) then
    Self.Invalidate;
end;

procedure TRComboBox.CMMouseLeave(var Message: TMessage);
var
  AntigoMouseControle: Boolean;
begin
  AntigoMouseControle := FMouseNoControle;

  if not ThemeControl(Self) then
    FMouseNoControle := IsMouseSobreControle()
  else
    FMouseNoControle := False;

  inherited;

  if (not ThemeControl(Self)) and (FMouseNoControle <> AntigoMouseControle) then
  begin
    Self.Invalidate;
  end;
end;

procedure TRComboBox.CNCommand(var Message: TWMCommand);
var
  AntigoMouseNoControle: Boolean;
begin
  inherited;

  case Message.NotifyCode of
    CBN_CLOSEUP:
    begin
      AntigoMouseNoControle := FMouseNoControle;
      FMouseNoControle := IsMouseSobreControle();

      if (not ThemeControl(Self)) and (FMouseNoControle <> AntigoMouseNoControle) then
        Self.Invalidate;
    end;
  end;
end;

procedure TRComboBox.ConfigurarVisual;
begin
  Self.Font.Color := EsquemaAtual.CorTextoNormal;
  Self.Color := EsquemaAtual.CorWindow;
  Self.Font.Name := SegoeFontName();
  Self.Font.Size := 8;
end;

constructor TRComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FCanvasDraw := TCanvas.Create;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);
end;

procedure TRComboBox.DesenharBorda(C: TCanvas; const Situacao: TSituacaoEsquema);
var
  R: TRect;
  Esq: PCoresSituacaoEsquema;
begin
  R := Rect(0, 0, Self.Width, Self.Height);
  C.Brush.Color := Self.Color;
  C.Pen.Style := psClear;
  C.Rectangle(R);

  C.Pen.Style := psSolid;
  C.Pen.Width := 1;

  Esq := @EsquemaAtual.Cores[Situacao];

  C.Pen.Color := Esq.BordasEdit[0];
  C.MoveTo(0, 0);
  C.LineTo(Self.ClientWidth - 1, 0);

  C.Pen.Color := Esq.BordasEdit[1];
  C.LineTo(Self.ClientWidth - 1, Self.ClientHeight - 1);

  C.Pen.Color := Esq.BordasEdit[2];
  C.LineTo(0, Self.ClientHeight - 1);

  C.Pen.Color := Esq.BordasEdit[3];
  C.LineTo(0, 0);
end;

procedure TRComboBox.DesenharSeta(C: TCanvas; Area: TRect;
  const Situacao: TSituacaoEsquema);
var
  Pts: array [0..2] of TPoint;
  CW, CH, i: Integer;
  Angulo, Ang2, X, Y: Single;
  Seno, Cosseno: Extended;
const
  TamSeta = 3;
begin
  CW := Area.Right - Area.Left;
  CH := Area.Bottom - Area.Top;
  X := Area.Left + CW div 2;
  Y := Area.Top + CH div 2;

  //Pts[0].Initialize(Area.Left + CW div 2 - TamSeta div 2, Area.Top + CH div 2 - TamSeta);
  Angulo := 270;

  for i := 0 to 2 do
  begin
    Ang2 := (Angulo * Pi) / 180;
    SinCos(Ang2, Seno, Cosseno);

    Pts[i].X := Round(X + Cosseno * TamSeta);
    Pts[i].Y := Round(Y + (-Seno) * TamSeta);
    Angulo := Angulo + (360 / 3);
  end;

//  Pts[0].Y := Pts[0].Y - 1;
  Pts[1].X := Pts[1].X + 1;
  Pts[2].X := Pts[2].X - 1;

  C.Brush.Color := EsquemaAtual.Cores[Situacao].ComboBoxCorSeta;
  C.Brush.Style := bsSolid;
  C.Pen.Style := psClear;

  C.Polygon(Pts);
end;

destructor TRComboBox.Destroy;
begin
  DesregistrarNotScheme(Self);
  FCanvasDraw.Free;
  inherited;
end;

procedure TRComboBox.DoEnter;
begin
  inherited;

  if Self.Style = csDropDownList then
    Self.Invalidate;
end;

procedure TRComboBox.DoExit;
begin
  inherited;

end;

function TRComboBox.IsMouseSobreControle: Boolean;
var
  PtMouse, PtOrigem, Pt: TPoint;
begin
  PtOrigem := Self.ClientOrigin;
  GetCursorPos(PtMouse);

  Pt.X := PtMouse.X - PtOrigem.X;
  Pt.Y := PtMouse.Y - PtOrigem.Y;

  Result := (Pt.X >=0) and (Pt.Y >= 0) and (PT.X < Self.ClientWidth) and
            (Pt.Y < Self.ClientHeight);
end;

procedure TRComboBox.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

procedure TRComboBox.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Inf: TComboBoxInfo;
  NovoMouseNoBotao: Boolean;
begin
  inherited;

  if Self.Style <> csDropDown then
    Exit;

  Inf.cbSize := SizeOf(Inf);
  SendMessage(Self.Handle, CB_GETCOMBOBOXINFO, 0, Integer(@Inf));

  NovoMouseNoBotao := PtInRect(Inf.rcButton, Point(X, Y));

  if NovoMouseNoBotao <> FMouseNoCombo then
  begin
    FMouseNoCombo := NovoMouseNoBotao;
    Self.Invalidate;
  end;
end;

procedure TRComboBox.WMPaint(var Message: TWMPaint);
var
  PT: TPaintStruct;
  DC: HDC;
  Situacao, SitDraw: TSituacaoEsquema;
  Inf: TComboBoxInfo;
  R: TRect;
  Origem, Mouse: TPoint;
  CW, CH: Integer;
  Dropped: Boolean;
  Texto: String;
begin
  if not (Self.Style in [csDropDown, csDropDownList]) then
  begin
    inherited;
    Exit;
  end;
//  inherited;
  DC := BeginPaint(Self.Handle, PT);

  try
    FCanvasDraw.Handle := DC;

    try
      Dropped := SendMessage(Self.Handle, CB_GETDROPPEDSTATE, 0, 0) = 1;

      if Self.Enabled then
      begin
        if Dropped then
          Situacao := sePressionado
        else
          if FMouseNoControle then
            Situacao := seHot
          else
            Situacao := seNormal;
      end else
        Situacao := seDisabled;


      Inf.cbSize := SizeOf(Inf);
      SendMessage(Self.Handle, CB_GETCOMBOBOXINFO, 0, Integer(@Inf));

      Origem := Self.ClientOrigin;
      GetCursorPos(Mouse);

      CW := Self.ClientWidth;
      CH := Self.ClientHeight;
      Mouse.X := Mouse.X - Origem.X;
      Mouse.Y := Mouse.Y - Origem.Y;

      if Situacao = sePressionado then
        if Inf.stateButton and STATE_SYSTEM_PRESSED = 0 then
          if FMouseNoControle then
            Situacao := seHot
          else
            Situacao := seNormal;

      case Self.Style of
        csDropDown:
          begin
            SitDraw := Situacao;

            if SitDraw = seNormal then
              if (Mouse.X >= 0) and (Mouse.Y >= 0) and
                 (Mouse.X <= CW) and (Mouse.Y <= CH) then
                SitDraw := seHot;

            if Self.Focused then
              SitDraw := sePressionado;

            Self.DesenharBorda(FCanvasDraw, SitDraw);

            R := Inf.rcButton;
            SitDraw := Situacao;

            if Dropped then
              SitDraw := sePressionado;

            if (SitDraw = seHot) and (not FMouseNoCombo) then
                SitDraw := seNormal;

            if SitDraw <> seNormal then
              TRRPGDrawUtil.DrawBkgButton(FCanvasDraw, R, SitDraw, False, False, ebRetangular);

            Self.DesenharSeta(FCanvasDraw, R, SitDraw);
          end;

        csDropDownList:
        begin
//          Exit;
          SitDraw := Situacao;
          R := Self.ClientRect;

          if Dropped then
            SitDraw := sePressionado;

          //if (not Dropped) and (FMouseNoCombo) and (FMouseNoControle) then
//            SitDraw := seHot;

          TRRPGDrawUtil.DrawBkgButton(FCanvasDraw, R, SitDraw);

          Texto := Self.Text;

          if Texto <> '' then
          begin
            R := Inf.rcItem;
            R.Left := R.Left + 1;

            FCanvasDraw.Font.Assign(Self.Font);
            FCanvasDraw.Font.Color := EsquemaAtual.Cores[Situacao].CorFonte;
            FCanvasDraw.TextRect(R, Texto, [tfSingleLine, tfVerticalCenter, tfNoPrefix]);
          end;

          if Self.Focused and (not Dropped) then
             TRRPGDrawUtil.DrawFocusRect(FCanvasDraw, Inf.rcItem, Situacao);

          R := Inf.rcButton;
          Self.DesenharSeta(FCanvasDraw, R, SitDraw);
        end;
      end;
    finally
      FCanvasDraw.Handle := 0;
    end;
  finally
    EndPaint(Self.Handle, PT);
  end;

  Message.Result := 1;
end;

end.
