unit RRPGControls;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, RRPGURLGetter,
  RRPGUrlPicture, StdCtrls, ExtCtrls, RRPGDrawUtil, RRPGColorScheme; // DBCtrls

type
  TRCustomURLImage = class(TGraphicControl)
  private
    FPicture: TURLPicture;
    FBorderStyle: TBorderStyle;
    FStretch: Boolean;
    FCenter: Boolean;
    FProporcional: Boolean;
    FIgnorarNeedRedraw: Boolean;
    FMaxBytes: Integer;
    FExibirProgresso: Boolean;

    FOnNoImageDraw: TNotifyEvent;
    FOnPictureLoadedChanged: TNotifyEvent;
    FOtimizarDesenho: Boolean;
    FBoaQualidade: Boolean;
    function GetPicture: TPicture;
    procedure SetMaxBytes(const Value: Integer);
    procedure SetExibirProgresso(const Value: Boolean);
    procedure SetBoaQualidade(const Value: Boolean);
    procedure SetOtimizarDesenho(const Value: Boolean);
  protected
    procedure NeedRedraw; virtual;
    procedure SetBorderStyle(const Value: TBorderStyle); virtual;
    procedure SetProporcional(const Value: Boolean); virtual;

    procedure Paint; override;
    function DestRect: TRect; virtual;

    procedure SetStretch(Value: Boolean); virtual;
    procedure SetCenter(Value: Boolean); virtual;

    // Eventos do TPicture
    procedure PictureChanged(Sender: TObject); virtual;
    procedure PictureLoadedChanged(Sender: TObject); virtual;

    // Eventos Messages
    procedure Resize; override;
  protected
    procedure CarregarImagemDeURL(const URL: String); virtual;
    procedure Loaded; override;

    // Propertys
    property BorderStyle
      : TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Center: Boolean read FCenter write SetCenter default True;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property Proporcional
      : Boolean read FProporcional write SetProporcional default False;
    property MaxBytes: Integer read FMaxBytes write SetMaxBytes default 0;
    property OnNoImageDraw
      : TNotifyEvent read FOnNoImageDraw write FOnNoImageDraw;
    property ExibirProgresso: Boolean read FExibirProgresso write
      SetExibirProgresso default True;
    property OtimizarDesenho: Boolean read FOtimizarDesenho write
      SetOtimizarDesenho default True;
    property BoaQualidade
      : Boolean read FBoaQualidade write SetBoaQualidade default True;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property URLPicture: TURLPicture read FPicture;
    property Picture: TPicture read GetPicture;
  published
    property OnPictureLoadedChanged: TNotifyEvent read FOnPictureLoadedChanged write FOnPictureLoadedChanged;
  end;

  TRURLImage = class(TRCustomURLImage)
  private
    FURLImagem: String;

    procedure SetURLImagem(const Value: String);
  public
    property Canvas;
  published
    property Align;
    property Anchors;
    property BorderStyle;
    property Center;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Stretch;
    property Proporcional;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;

    property URLImagem: String read FURLImagem write SetURLImagem;
    property MaxBytes;
    property OnNoImageDraw;
    property ExibirProgresso;
    property OtimizarDesenho;
    property BoaQualidade;
  end;

  TTipoDesenhoGradiente = (tdgSolido, tdgVertical2quartos, tdgVerticalTitulo);
  TFilhosTransparentes = (ftSemSuporte, ftNoClipChildren, ftComposited);
  TPaintEvent = procedure(Sender: TObject; Canvas: TCanvas) of object;

  TRPanel = class(TPanel)
  private
    FTipoFundo: TTipoDesenhoGradiente;
    FCorSecundaria: TColor;
    FCanvasDraw: TCanvas;
    FDesenharBorda: Boolean;
    FCorBorda: TColor;
    FAlturaGradienteTitulo: Integer;
    FAlpha: Byte;
    FFilhosTransparentes: TFilhosTransparentes;
    FOnPaint: TPaintEvent;
    procedure SetCorSecundaria(const Value: TColor);
    procedure SetTipoFundo(const Value: TTipoDesenhoGradiente);
    procedure SetDesenharBorda(const Value: Boolean);
    procedure SetCorBorda(const Value: TColor);
    procedure SetAlturaGradienteTitulo(const Value: Integer);
    procedure SetAlpha(const Value: Byte);
    procedure SetFilhosTransparentes(const Value: TFilhosTransparentes);
  protected
    procedure DesenharEm(DC: HDC); virtual;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property CorSecundaria: TColor read FCorSecundaria write SetCorSecundaria;
    property CorBorda: TColor read FCorBorda write SetCorBorda;
    property TipoFundo
      : TTipoDesenhoGradiente read FTipoFundo write SetTipoFundo default
      tdgSolido;
    property DesenharBorda
      : Boolean read FDesenharBorda write SetDesenharBorda default False;
    property AlturaGradienteTitulo: Integer read FAlturaGradienteTitulo write
      SetAlturaGradienteTitulo default 100;
    property Alpha: Byte read FAlpha write SetAlpha default 255;
    property FilhosTransparentes
      : TFilhosTransparentes read FFilhosTransparentes write
      SetFilhosTransparentes default ftSemSuporte;

    property OnPaint: TPaintEvent read FOnPaint write FOnPaint;
  end;

  TEstiloLabel = (elNormal, elTitulo, elGrupoHeader, elPequenoTitulo, elLink,
    elLinkDefinicao, elLinkMaior, elLinkMedio, elCostumizado, elNormalMenor,
    elNegrito);

const
  LABEL_LINKS = [elLink, elLinkDefinicao, elLinkMaior, elLinkMedio];

type
  TRLabel = class(TCustomLabel)
  private
    FAutoUnderline: Boolean;
    FEstilo: TEstiloLabel;
    FLinkTravado: Boolean;
    procedure SetAutoUnderline(const Value: Boolean);
    procedure SetEstilo(const Value: TEstiloLabel);
    procedure SetLinkTravado(const Value: Boolean);
  protected
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property LinkTravado: Boolean read FLinkTravado write SetLinkTravado;
  published
    property OnMouseLeave;
    property OnMouseEnter;
    property AutoUnderline: Boolean read FAutoUnderline write SetAutoUnderline;
    property Estilo: TEstiloLabel read FEstilo write SetEstilo default elNormal;

    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color nodefault;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property EllipsisPosition;
    property Enabled;
    property FocusControl;
    property Font;
    property GlowSize; // Windows Vista only
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Touch;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnGesture;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TEstiloImageButton = (eibToolbutton, eibButton);

  TStatusImageButton = record
    Caption: String;
    Estilo: TEstiloImageButton;
    GraphDesenhar: TGraphic;
    Situacao: TSituacaoEsquema;
    Default: Boolean;
    Focado: Boolean;
    MousePressionado: Boolean;
    MouseNoControle: Boolean;
    Enabled: Boolean;
  end;

  TRImageButton = class(TCustomControl)
  private
    FPicture: TPicture;
    FPictureHover: TPicture;
    FPictureDesativado: TPicture;
    FMouseNoControle: Boolean;
    FMousePressionado: Boolean;
    FTecladoEngatilhado: Boolean;
    FCaption: String;
    FEstilo: TEstiloImageButton;
    FFocado: Boolean;
    FDefault: Boolean;
    FModalResult: TModalResult;
    FTransparente: Boolean;
    FDrawEdge: Boolean;

    // Animao do controle
    FStatusAtual: TStatusImageButton;
    FStatusNovo: TStatusImageButton;
    procedure SetPicture(const Value: TPicture);
    procedure PictureChanged(Sender: TObject);
    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;
    procedure WMErase(var Message: TMessage);
    message WM_ERASEBKGND;
    procedure SetPictureHover(const Value: TPicture);
    procedure SetPictureDesativado(const Value: TPicture);
    procedure SetCaption(const Value: String);
    procedure SetEstilo(const Value: TEstiloImageButton);
    procedure SetDefault(const Value: Boolean);
    procedure SetTransparente(const Value: Boolean);
  protected
    function EstadoMudou: Boolean;
    procedure CalcularEstado(var Estado: TStatusImageButton);
    function CustomDrawToolButton(C: TCanvas; Status: TStatusImageButton)
      : Boolean; virtual;

    procedure CMDialogKey(var Message: TCMDialogKey);
    message CM_DIALOGKEY;
    procedure PaintToolButton(C: TCanvas; Status: TStatusImageButton); virtual;
    procedure PaintButton(C: TCanvas; Status: TStatusImageButton); virtual;

    procedure Paint; override;
    procedure PaintEm(DC: HDC; Status: TStatusImageButton); virtual;

    procedure WMPaint(var Message: TWMPaint);
    message WM_PAINT;
    procedure WMPrintClient(var Message: TWMPrintClient);
    message WM_PRINTCLIENT;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;

    procedure Loaded; override;
    procedure SetEnabled(Value: Boolean); override;
    procedure ConfigurarVisual; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property Align;
    property TabStop;
    property TabOrder;
    property Anchors;
    property Caption: String read FCaption write SetCaption;
    property Estilo: TEstiloImageButton read FEstilo write SetEstilo default
      eibButton;
    property Picture: TPicture read FPicture write SetPicture;
    property PictureHover: TPicture read FPictureHover write SetPictureHover;
    property PictureDesativado: TPicture read FPictureDesativado write
      SetPictureDesativado;
    property ModalResult
      : TModalResult read FModalResult write FModalResult default 0;
    property ShowHint;
    property Enabled;
    property OnClick;
    property Font;
    property Default: Boolean read FDefault write SetDefault default False;
    property Transparente
      : Boolean read FTransparente write SetTransparente default True;
    property Color;
  end;

  TRSpeedButton = class(TGraphicControl)
  private
    FPicture: TPicture;
    FPictureHover: TPicture;
    FPictureDesativado: TPicture;
    FMouseNoControle: Boolean;
    FMousePressionado: Boolean;
    FCaption: String;
    procedure SetPicture(const Value: TPicture);
    procedure SetPictureDesativado(const Value: TPicture);
    procedure SetPictureHover(const Value: TPicture);
    procedure SetCaption(const Value: String);
  protected
    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;

    procedure PaintToolButton(C: TCanvas); virtual;
    procedure PictureChanged(Sender: TObject); virtual;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property Visible;
    property ShowHint;
    property Caption: String read FCaption write SetCaption;
    property Picture: TPicture read FPicture write SetPicture;
    property PictureHover: TPicture read FPictureHover write SetPictureHover;
    property PictureDesativado: TPicture read FPictureDesativado write
      SetPictureDesativado;
    property OnClick;
    property Font;
  end;

  TStatusCheckbox = record
    FTecladoEngatilhado: Boolean;
    FMouseNoControle: Boolean;
    FMousePressionado: Boolean;
    FFocado: Boolean;
    Enabled: Boolean;
    Checked: Boolean;
  end;

  TRCheckBox = class(TCheckBox)
  private
    FTecladoEngatilhado: Boolean;
    FMouseNoControle: Boolean;
    FMousePressionado: Boolean;
    FFocado: Boolean;

    // Animao
    FStatusAtual: TStatusCheckbox;
    FStatusNovo: TStatusCheckbox;
  protected
    procedure CalcularEstado(var Estado: TStatusCheckbox); virtual;
    function EstadoMudou: Boolean; virtual;

    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;
    procedure PaintCheckBox(DC: HDC; Status: TStatusCheckbox); virtual;
    procedure WMEraseBkg(var Message: TWMEraseBkgnd);
    message WM_ERASEBKGND;
    procedure WMPaint(var Message: TWMPaint);
    message WM_PAINT;
    procedure WMPrintClient(var Message: TWMPrintClient);
    message WM_PRINTCLIENT;
    procedure WMPrint(var Message: TWMPrint);
    message WM_PRINT;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure SetEnabled(Value: Boolean); override;
    procedure Toggle; override;

    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
    procedure SetChecked(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TRRadioButton = class(TRadioButton)
  private
    FTecladoEngatilhado: Boolean;
    FMouseNoControle: Boolean;
    FMousePressionado: Boolean;
    FFocado: Boolean;
    FCaption: String;

    FStatusAtual: TStatusCheckbox;
    FStatusNovo: TStatusCheckbox;
  protected
    procedure CalcularEstado(var Estado: TStatusCheckbox); virtual;
    function EstadoMudou: Boolean; virtual;

    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;
    procedure PaintRadio(DC: HDC; Status: TStatusCheckbox); virtual;
    procedure WMPaint(var Message: TWMPaint);
    message WM_PAINT;
    procedure WMPrintClient(var Message: TWMPrintClient);
    message WM_PRINTCLIENT;
    procedure WMPrint(var Message: TWMPrint);
    message WM_PRINT;
    procedure WMErase(var Message: TWMEraseBkgnd);
    message WM_ERASEBKGND;
    procedure WMSetText(var Message: TWMSetText);
    message WM_SETTEXT;
    procedure WMGetText(var Message: TWMGetText);
    message WM_GETTEXT;
    procedure WMGetTextLength(var Message: TWMGetTextLength);
    message WM_GETTEXTLENGTH;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure SetEnabled(Value: Boolean); override;
    procedure SetChecked(Value: Boolean); override;

    procedure Loaded; override;
    procedure ConfigurarVisual; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TREdit = class(TEdit)
  private
    FCanvasDraw: TCanvas;
    FMouseNoControle: Boolean;
  protected
    procedure ExcluirChildRect(DC: HDC); virtual;

    procedure WMNCPaint(var Message: TWMNCPaint);
    message WM_NCPAINT;
    procedure CMMouseEnter(var Message: TMessage);
    message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage);
    message CM_MOUSELEAVE;
    procedure CNCtlColorEdit(var Message: TMessage);
    message CN_CTLCOLOREDIT;
    procedure CNCtlColorStatic(var Message: TMessage);
    message CN_CTLCOLORSTATIC;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TRMemo = class(TMemo)
  protected
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TRPanelCustomizado = class(TRPanel)
  protected
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TRHeaderPanel = class(TRPanelCustomizado)
  protected
    procedure ConfigurarVisual; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TRMainPanel = class(TRPanelCustomizado)
  protected
    procedure ConfigurarVisual; override;
  end;

  TRContentPanel = class(TRPanelCustomizado)
  protected
    procedure ConfigurarVisual; override;
  end;

  TConfigAtualSB = (casBuscar, casCancelar);

  TSearchImageButton = class(TRImageButton)
  private
    FConfigAtual: TConfigAtualSB;
  protected
    function CustomDrawToolButton(C: TCanvas; Status: TStatusImageButton)
      : Boolean; override;

    procedure CustomDrawBuscar(C: TCanvas; Status: TStatusImageButton);
    procedure CustomDrawCancelar(C: TCanvas; Status: TStatusImageButton);

    procedure SetParent(AParent: TWinControl); override;
    procedure SetConfig(NovaConfig: TConfigAtualSB); virtual;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TSearchBoxEvent = procedure(Sender: TObject; Termo: String) of object;

  TRSearchBox = class(TREdit)
  private
    FBotBuscar: TSearchImageButton;
    FOnSearch: TSearchBoxEvent;
    FTimer: TTimer;
    FTickLastChange: Cardinal;
  protected
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure Change; override;

    procedure OnClickSearchButton(Sender: TObject);
    procedure OnTimer(Sender: TObject);
    procedure KeyPress(var Key: Char); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property OnSearch: TSearchBoxEvent read FOnSearch write FOnSearch;
  end;

const
  TEMPO_ANIM = 100;
  TEMPO_DEFOCAR = 400;
  TEMPO_TIRAR_HOT = 700;
  TEMPO_TIRAR_HOT_HALF = 400;

implementation

uses
  GraphUtil, UxTheme, RRPGUtil, Themes, GDIPlus, Math;

threadvar _BufferedPaintInitied: Boolean;

{ TRCustomURLImage }

procedure TRCustomURLImage.CarregarImagemDeURL(const URL: String);
begin
  FPicture.URL := URL;
end;

constructor TRCustomURLImage.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csOpaque, csReplicatable, csNeedsBorderPaint];
  Width := 105;
  Height := 105;
  // TabStop := True;

  ParentColor := False;
  FPicture := TURLPicture.Create;
  FPicture.OnChange := Self.PictureChanged;
  FPicture.OnPictureLoadedChange := Self.PictureLoadedChanged;
  FBorderStyle := bsSingle;
  FCenter := True;
  FExibirProgresso := True;
  FOtimizarDesenho := True;
  FBoaQualidade := True;

  Self.ControlStyle := Self.ControlStyle - [csOpaque];
end;

function TRCustomURLImage.DestRect: TRect;
var
  w, h, cw, ch: Integer;
  xyaspect: Double;
begin
  w := Picture.Width;
  h := Picture.Height;
  cw := ClientWidth;
  ch := ClientHeight;

  if Stretch or (FProporcional and ((w > cw) or (h > ch))) then
  begin
    if FProporcional and (w > 0) and (h > 0) then
    begin
      xyaspect := w / h;
      if w > h then
      begin
        w := cw;
        h := Trunc(cw / xyaspect);
        if h > ch then // woops, too big
        begin
          h := ch;
          w := Trunc(ch * xyaspect);
        end;
      end
      else
      begin
        h := ch;
        w := Trunc(ch * xyaspect);

        if w > cw then // woops, too big
        begin
          w := cw;
          h := Trunc(cw / xyaspect);
        end;
      end;
    end
    else
    begin
      w := cw;
      h := ch;
    end;
  end;

  with Result do
  begin
    Left := 0;
    Top := 0;
    Right := w;
    Bottom := h;
  end;

  if Center then
    OffsetRect(Result, (cw - w) div 2, (ch - h) div 2);
end;

destructor TRCustomURLImage.Destroy;
begin
  FreeAndNil(FPicture);
  inherited;
end;

function TRCustomURLImage.GetPicture: TPicture;
begin
  Result := FPicture.Picture;
end;

procedure TRCustomURLImage.Loaded;
begin
  inherited;

  if FOtimizarDesenho then
    FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
      FProporcional);
end;

procedure TRCustomURLImage.NeedRedraw;
begin
  if (not FIgnorarNeedRedraw) and not(csLoading in Self.ComponentState) then
    Self.Invalidate;
end;

procedure TRCustomURLImage.Paint;
begin
  with Canvas do
  begin
    if (csDesigning in Self.ComponentState) and (not FPicture.PictureLoaded) then
    begin
      Brush.Style := bsSolid;
      Brush.Color := Color;
      FillRect(ClientRect);
    end;

    if ((not FExibirProgresso) and (not FPicture.PictureLoaded)) or
      (FPicture.URL = '') then
    begin
      if Assigned(FOnNoImageDraw) then
        FOnNoImageDraw(Self);
    end
    else
      FPicture.DesenharEm(Canvas, ClientRect, FStretch, FCenter, FProporcional);
  end;
end;

procedure TRCustomURLImage.PictureChanged(Sender: TObject);
begin
  if FExibirProgresso then
    Self.NeedRedraw();
end;

procedure TRCustomURLImage.PictureLoadedChanged(Sender: TObject);
begin
  if not FExibirProgresso then
    Self.NeedRedraw();

  if Assigned(FOnPictureLoadedChanged) then
    FOnPictureLoadedChanged(Self);
end;

procedure TRCustomURLImage.Resize;
begin
  inherited;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;

procedure TRCustomURLImage.SetBoaQualidade(const Value: Boolean);
begin
  if FBoaQualidade <> Value then
  begin
    FBoaQualidade := Value;
    Self.NeedRedraw;
  end;
end;

procedure TRCustomURLImage.SetBorderStyle(const Value: TBorderStyle);
begin
  FBorderStyle := Value;
  Self.NeedRedraw();
end;

procedure TRCustomURLImage.SetCenter(Value: Boolean);
begin
  FCenter := Value;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;

procedure TRCustomURLImage.SetExibirProgresso(const Value: Boolean);
begin
  if FExibirProgresso <> Value then
  begin
    FExibirProgresso := Value;
  end;
end;

procedure TRCustomURLImage.SetMaxBytes(const Value: Integer);
begin
  if FMaxBytes <> Value then
  begin
    FMaxBytes := Value;
    FPicture.MaxBytes := FMaxBytes;
  end;
end;

procedure TRCustomURLImage.SetOtimizarDesenho(const Value: Boolean);
begin
  if FOtimizarDesenho <> Value then
  begin
    FOtimizarDesenho := Value;

    if not(csLoading in Self.ComponentState) then
      if FOtimizarDesenho then
        FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter,
          FStretch, FProporcional)
      else
        FPicture.OtimizarDesenhoPara(0, 0, FCenter, FStretch, FProporcional);
    // No otimizar
  end;
end;

procedure TRCustomURLImage.SetProporcional(const Value: Boolean);
begin
  if FProporcional <> Value then
  begin
    FProporcional := Value;

    if not(csLoading in Self.ComponentState) then
      if FOtimizarDesenho then
        FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter,
          FStretch, FProporcional);

    Self.NeedRedraw();
  end;
end;

procedure TRCustomURLImage.SetStretch(Value: Boolean);
begin
  FStretch := Value;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;

{ TRURLImage }

procedure TRURLImage.SetURLImagem(const Value: String);
begin
  if FURLImagem <> Value then
  begin
    FURLImagem := Value;
    Self.CarregarImagemDeURL(Value);
  end;
end;

{ TRPanel }

constructor TRPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCorSecundaria := clBtnFace;
  FTipoFundo := tdgSolido;
  FCanvasDraw := TCanvas.Create;
  FDesenharBorda := False;
  FCorBorda := clBtnFace;
  FAlturaGradienteTitulo := 100;
  FAlpha := 255;
end;

procedure TRPanel.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.WindowClass.hbrBackground := 0;

  if not(csDesigning in Self.ComponentState) then
    case FFilhosTransparentes of
      ftNoClipChildren:
        Params.Style := Params.Style and not(WS_CLIPCHILDREN);
      ftComposited:
        Params.ExStyle := Params.ExStyle or WS_EX_COMPOSITED;
    end;
end;

procedure TRPanel.DesenharEm(DC: HDC);
var
  AlturaUmQuarto, cw, ch: Integer;
  R: TRect;
begin
  FCanvasDraw.Handle := DC;

  try
    cw := Self.ClientWidth;
    ch := Self.ClientHeight;
    AlturaUmQuarto := ch div 4;

    case FTipoFundo of
      tdgSolido:
        begin
          FCanvasDraw.Pen.Style := psClear;
          FCanvasDraw.Brush.Style := bsSolid;
          FCanvasDraw.Brush.Color := Self.Color;
          // FCanvasDraw.Brush.Color := clRed;
          FCanvasDraw.Rectangle(0, 0, cw + 1, ch + 1);
        end;

      tdgVertical2quartos:
        begin
          R := Rect(0, 0, cw, AlturaUmQuarto);
          GraphUtil.GradientFillCanvas(FCanvasDraw, FCorSecundaria, Self.Color,
            R, gdVertical);

          FCanvasDraw.Pen.Style := psClear;
          FCanvasDraw.Brush.Style := bsSolid;
          FCanvasDraw.Brush.Color := Self.Color;
          FCanvasDraw.Rectangle(0, AlturaUmQuarto, cw + 1,
            AlturaUmQuarto * 3 + 1);

          R := Rect(0, AlturaUmQuarto * 3, cw, ch + 1);
          GraphUtil.GradientFillCanvas(FCanvasDraw, Self.Color, FCorSecundaria,
            R, gdVertical);
        end;

      tdgVerticalTitulo:
        begin
          R := Rect(0, 0, cw, FAlturaGradienteTitulo);
          GraphUtil.GradientFillCanvas(FCanvasDraw, FCorSecundaria, Self.Color,
            R, gdVertical);

          FCanvasDraw.Pen.Style := psClear;
          FCanvasDraw.Brush.Style := bsSolid;
          FCanvasDraw.Brush.Color := Self.Color;
          FCanvasDraw.Rectangle(0, FAlturaGradienteTitulo, cw + 1, ch + 1);
        end;
    end;

    if FDesenharBorda then
    begin
      FCanvasDraw.Brush.Style := bsClear;
      FCanvasDraw.Pen.Style := psSolid;
      FCanvasDraw.Pen.Width := 1;
      FCanvasDraw.Pen.Color := FCorBorda;

      R := Rect(0, 0, cw, ch);
      // InflateRect(R, -20, -20);
      // OffsetRect(R, 20, 20);
      // FCanvasDraw.Rectangle(R);
      FCanvasDraw.RoundRect(R, 5, 5);
    end;

    if Assigned(FOnPaint) then
      FOnPaint(Self, FCanvasDraw);
  finally
    FCanvasDraw.Handle := 0;
  end;
end;

destructor TRPanel.Destroy;
begin
  FCanvasDraw.Free;
  inherited;
end;

procedure TRPanel.Paint;
var
  DC: HDC;
  PB: HPAINTBUFFER;
  R: TRect;
  cw, ch: Integer;
  BF: TBlendFunction;
begin
  inherited;

  if (FAlpha < 255) and (UxTheme.IsAppThemed) and IsMaiorIgualWindows7 and not
    (csDesigning in Self.ComponentState) then
  begin
    R := Rect(0, 0, Self.ClientWidth, Self.ClientHeight);
    cw := Self.ClientWidth;
    ch := Self.ClientHeight;

    if not _BufferedPaintInitied then
    begin
      BufferedPaintInit();
      _BufferedPaintInitied := True;
    end;

    PB := BeginBufferedPaint(Self.Canvas.Handle, R, BPBF_TOPDOWNDIB, nil, DC);

    try

      DrawThemeParentBackground(Self.Handle, Self.Canvas.Handle, @R);
      Self.DesenharEm(DC);

      FillChar(BF, SizeOf(BF), 0);
      BF.BlendOp := AC_SRC_OVER;
      BF.SourceConstantAlpha := FAlpha;

      if not AlphaBlend(Self.Canvas.Handle, 0, 0, cw, ch, DC, 0, 0, cw, ch, BF)
        then
        RaiseLastOSError;
    finally
      EndBufferedPaint(PB, False);
    end;
  end
  else
    Self.DesenharEm(Self.Canvas.Handle);
end;

procedure TRPanel.SetAlpha(const Value: Byte);
begin
  if FAlpha <> Value then
  begin
    FAlpha := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.SetAlturaGradienteTitulo(const Value: Integer);
begin
  if Value <> FAlturaGradienteTitulo then
  begin
    FAlturaGradienteTitulo := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.SetCorBorda(const Value: TColor);
begin
  if FCorBorda <> Value then
  begin
    FCorBorda := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.SetCorSecundaria(const Value: TColor);
begin
  if FCorSecundaria <> Value then
  begin
    FCorSecundaria := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.SetDesenharBorda(const Value: Boolean);
begin
  if FDesenharBorda <> Value then
  begin
    FDesenharBorda := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.SetFilhosTransparentes(const Value: TFilhosTransparentes);
begin
  if FFilhosTransparentes <> Value then
  begin
    FFilhosTransparentes := Value;

    if not(csLoading in Self.ComponentState) then
      Self.RecreateWnd;
  end;
end;

procedure TRPanel.SetTipoFundo(const Value: TTipoDesenhoGradiente);
begin
  if FTipoFundo <> Value then
  begin
    FTipoFundo := Value;
    Self.Invalidate;
  end;
end;

procedure TRPanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if not(csDesigning in Self.ComponentState) then
  begin
    Message.Result := 1;

    if csLoading in Self.ComponentState then
      Exit;

    if FCanvasDraw = nil then
      Exit;
  end
  else
    inherited;
end;

{ TRLabel }

procedure TRLabel.CMMouseEnter(var Message: TMessage);
begin
  if FAutoUnderline then
    Self.Font.Style := Self.Font.Style + [fsUnderline];

  if FEstilo in LABEL_LINKS then
    if not FLinkTravado then
      case FEstilo of
        elLink, elLinkMaior, elLinkMedio:
          Self.Font.Color := EsquemaAtual.CorLinkHot;
        elLinkDefinicao:
          Self.Font.Color := EsquemaAtual.CorLinkDefHot;
      end;

  inherited;
end;

procedure TRLabel.CMMouseLeave(var Message: TMessage);
begin
  if FAutoUnderline then
    Self.Font.Style := Self.Font.Style - [fsUnderline];

  if FEstilo in LABEL_LINKS then
    if not FLinkTravado then
      case FEstilo of
        elLink, elLinkMaior, elLinkMedio:
          Self.Font.Color := EsquemaAtual.CorLink;
        elLinkDefinicao:
          Self.Font.Color := EsquemaAtual.CorLinkDef;
      end;

  inherited;
end;

procedure TRLabel.ConfigurarVisual;
begin
  Self.Cursor := crDefault;

  case Self.Estilo of
    elNormal:
      begin
        Self.Font.Color := EsquemaAtual.CorTextoNormal;
        Self.Font.Size := 8;
        Self.Font.Style := [];
      end;

    elNormalMenor:
      begin
        Self.Font.Color := EsquemaAtual.CorTextoNormal;
        Self.Font.Size := 7;
        Self.Font.Style := [];
      end;

    elPequenoTitulo:
      begin
        Self.Font.Color := EsquemaAtual.CorTextoNormal;
        Self.Font.Size := 8;
        Self.Font.Style := [];
      end;

    elLink, elLinkMaior, elLinkMedio:
      begin
        if FLinkTravado then
          Self.Font.Color := EsquemaAtual.CorLinkHot
        else
          Self.Font.Color := EsquemaAtual.CorLink;

        case Self.Estilo of
          elLink:
            Self.Font.Size := 8;
          elLinkMaior:
            Self.Font.Size := 10;
          elLinkMedio:
            Self.Font.Size := 9;
        end;

        Self.Font.Style := [];
        Self.Cursor := crHandPoint;
      end;

    elLinkDefinicao:
      begin
        if FLinkTravado then
          Self.Font.Color := EsquemaAtual.CorLinkDefHot
        else
          Self.Font.Color := EsquemaAtual.CorLinkDef;

        Self.Font.Size := 8;
        Self.Cursor := crHandPoint;
        Self.Font.Style := [];
      end;

    elTitulo:
      begin
        Self.Font.Color := EsquemaAtual.CorTextoTitulo;
        Self.Font.Size := 11;
        Self.Font.Style := [];
      end;

    elGrupoHeader:
      begin
        Self.Font.Color := EsquemaAtual.CorTextoTitulo;
        Self.Font.Size := 10;
        Self.Font.Style := [];
      end;

    elNegrito:
    begin
      Self.Font.Color := EsquemaAtual.CorTextoNormal;
      Self.Font.Size := 8;
      Self.Font.Style := [fsBold];
    end;
  end;

  Self.Font.Name := SegoeFontName();
end;

constructor TRLabel.Create(AOwner: TComponent);
begin
  inherited;
  Self.Estilo := elNormal;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);
end;

destructor TRLabel.Destroy;
begin
  DesregistrarNotScheme(Self);
  inherited;
end;

procedure TRLabel.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

procedure TRLabel.SetAutoUnderline(const Value: Boolean);
begin
  FAutoUnderline := Value;
end;

procedure TRLabel.SetEstilo(const Value: TEstiloLabel);
begin
  if FEstilo <> Value then
  begin
    FEstilo := Value;
    FLinkTravado := False;
    Self.ConfigurarVisual();
  end;
end;

procedure TRLabel.SetLinkTravado(const Value: Boolean);
begin
  if FLinkTravado <> Value then
  begin
    FLinkTravado := Value;

    if Self.Estilo in LABEL_LINKS then
      Self.ConfigurarVisual();
  end;
end;

{ TRImageButton }

procedure TRImageButton.CalcularEstado(var Estado: TStatusImageButton);
var
  TipEsq: TSituacaoEsquema;
  PicDraw: TPicture;
begin
  Estado.Caption := FCaption;
  Estado.Estilo := FEstilo;
  Estado.Default := FDefault;
  Estado.Focado := FFocado;
  Estado.MousePressionado := FMousePressionado;
  Estado.MouseNoControle := FMouseNoControle;
  Estado.Enabled := Self.Enabled;

  if Self.Enabled then
  begin
    if FTecladoEngatilhado then
      TipEsq := sePressionado
    else if FMouseNoControle then
    begin
      if FMousePressionado then
        TipEsq := sePressionado
      else
        TipEsq := seHot
    end
    else
      TipEsq := seNormal;
  end
  else
    TipEsq := seDisabled;

  Estado.Situacao := TipEsq;

  if Self.Enabled and FMouseNoControle and (FPictureHover.Graphic <> nil) then
    PicDraw := FPictureHover
  else if (not Self.Enabled) and (FPictureDesativado.Graphic <> nil) then
    PicDraw := FPictureDesativado
  else
    PicDraw := FPicture;

  if PicDraw <> nil then
    Estado.GraphDesenhar := PicDraw.Graphic
  else
    Estado.GraphDesenhar := nil;
end;

procedure TRImageButton.Click;
begin
  if FModalResult <> 0 then
  begin
    if Self.Owner is TForm then
      TForm(Self.Owner).ModalResult := FModalResult;
  end
  else
    inherited;
end;

procedure TRImageButton.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if (((CharCode = VK_RETURN) and (FModalResult = mrOk)) or
        ((CharCode = VK_ESCAPE) and (FModalResult = mrCancel)) and
        (KeyDataToShiftState(Message.KeyData) = []) and CanFocus) and
      (Self.Enabled) then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TRImageButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := True;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRImageButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := False;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRImageButton.ConfigurarVisual;
begin
  Self.Font.Size := 8;
  Self.Font.Name := SegoeFontName();
end;

constructor TRImageButton.Create(AOwner: TComponent);
begin
  FTransparente := True;
  FDrawEdge := True;
  inherited;
  FPicture := TPicture.Create;
  FPicture.OnChange := Self.PictureChanged;
  FPictureHover := TPicture.Create;
  FPictureHover.OnChange := Self.PictureChanged;

  FPictureDesativado := TPicture.Create;
  FPictureDesativado.OnChange := Self.PictureChanged;

  // Self.InterceptMouse := True;
  Brush.Style := bsClear;

  Self.Estilo := eibButton;

  Self.Width := 75;
  Self.Height := 25;

  if csDesigning in Self.ComponentState then
    Self.ConfigurarVisual();
end;

procedure TRImageButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);

  if FEstilo = eibToolbutton then
    if FTransparente then
      Params.ExStyle := Params.ExStyle or WS_EX_TRANSPARENT;
end;

function TRImageButton.CustomDrawToolButton(C: TCanvas;
  Status: TStatusImageButton): Boolean;
begin
  Result := False;
end;

destructor TRImageButton.Destroy;
begin
  FPictureDesativado.Free;
  FPictureHover.Free;
  FPicture.Free;
  inherited;
end;

procedure TRImageButton.DoEnter;
begin
  inherited;
  FFocado := True;
  Self.CalcularEstado(FStatusNovo);
  Self.Invalidate;
end;

procedure TRImageButton.DoExit;
begin
  inherited;
  FTecladoEngatilhado := False;
  FFocado := False;
  Self.CalcularEstado(FStatusNovo);
  Self.Invalidate;
end;

function TRImageButton.EstadoMudou: Boolean;
begin
  Result := (FStatusAtual.Caption <> FStatusNovo.Caption) or
    (FStatusAtual.Estilo <> FStatusNovo.Estilo) or
    (FStatusAtual.GraphDesenhar <> FStatusNovo.GraphDesenhar) or
    (FStatusAtual.Situacao <> FStatusNovo.Situacao) or
    (FStatusAtual.Default <> FStatusNovo.Default) or
    (FStatusAtual.Focado <> FStatusNovo.Focado) or
    (FStatusAtual.MousePressionado <> FStatusNovo.MousePressionado) or
    (FStatusNovo.MouseNoControle <> FStatusNovo.MouseNoControle) or
    (FStatusAtual.Enabled <> FStatusNovo.Enabled);
end;

procedure TRImageButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;

  if (Key = VK_SPACE) and (Shift = []) and (FEstilo = eibButton) and (FFocado)
    then
  begin
    if Self.Enabled then
    begin
      FTecladoEngatilhado := True;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRImageButton.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;

  if (Key = VK_SPACE) and (Shift = []) and (FEstilo = eibButton) and (FFocado)
    then
  begin
    Self.Click;
    FTecladoEngatilhado := False;
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
    Key := 0;
  end;
end;

procedure TRImageButton.Loaded;
begin
  inherited;
  Self.CalcularEstado(FStatusAtual);
  FStatusNovo := FStatusAtual;
  Self.ConfigurarVisual();
end;

procedure TRImageButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if FEstilo = eibButton then
    if not FFocado then
      Self.SetFocus;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := True;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRImageButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := False;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRImageButton.Paint;
begin

end;

procedure TRImageButton.PaintButton(C: TCanvas; Status: TStatusImageButton);
var
  ch, cw, PX, PY: Integer;
  R: TRect;
  Esq: PCoresSituacaoEsquema;
  TipEsq: TSituacaoEsquema;
  XTexto, WTexto: Integer;
const
  ArSingle: array [0 .. 1] of Single = (1.0, 1.0);
const
  BORDA_WIDTH = 1;
  MARGEM_IMAGEM = 5;
begin
  // inherited;
  C.Font.Assign(Self.Font);
  ch := Self.ClientHeight;
  cw := Self.ClientWidth;

  TipEsq := Status.Situacao;
  Esq := @EsquemaAtual.Cores[TipEsq];
  TRRPGDrawUtil.DrawBkgButton(C, Self.ClientRect, TipEsq, Status.Focado,
    Status.Default);

  // Desenhar Grfico e Texto

  if Status.Caption <> '' then
  begin
    R.Left := 0;
    R.Right := cw;
    R.Top := 0;
    R.Bottom := ch;
    C.TextRect(R, Status.Caption, [tfCalcRect, tfCenter, tfVerticalCenter,
      tfSingleLine]);

    WTexto := R.Right - R.Left;
  end
  else
    WTexto := 0;

  if Status.GraphDesenhar <> nil then
  begin
    PY := ch div 2 - Status.GraphDesenhar.Height div 2;
    PX := cw div 2 - (Status.GraphDesenhar.Width + WTexto + MARGEM_IMAGEM)
      div 2;

    XTexto := PX + Status.GraphDesenhar.Width + MARGEM_IMAGEM;
    C.Draw(PX, PY, Status.GraphDesenhar);
  end
  else
    XTexto := cw div 2 - WTexto div 2;

  if Status.Caption <> '' then
  begin
    C.Font.Assign(Self.Font);
    C.Font.Color := Esq.CorFonte;

    R.Top := 0;
    R.Bottom := ch;
    R.Left := XTexto;
    R.Right := XTexto + WTexto;
    C.TextRect(R, Status.Caption, [tfCenter, tfVerticalCenter, tfSingleLine]);
  end;
end;

procedure TRImageButton.PaintEm(DC: HDC; Status: TStatusImageButton);
begin
  Self.Canvas.Handle := DC;

  try
    case FEstilo of
      eibToolbutton:
        Self.PaintToolButton(Self.Canvas, Status);
      eibButton:
        Self.PaintButton(Self.Canvas, Status);
    end;
  finally
    Self.Canvas.Handle := 0
  end;
end;

procedure TRImageButton.PaintToolButton(C: TCanvas; Status: TStatusImageButton);
var
  ch, cw, PX, PY, Edge: Integer;
  TS: TClasseThemeServices;
  TB: TThemedToolBar;
  R: TRect;

  procedure LimparFundo;
  var
    R: TRect;
  begin
    R := Rect(0, 0, cw, ch);
    C.Brush.Color := Self.Color;
    C.Brush.Style := bsSolid;
    C.Pen.Color := C.Brush.Color;
    C.Pen.Style := psSolid;
    C.Rectangle(R);
  end;

begin
  ch := Self.ClientHeight;
  cw := Self.ClientWidth;

  if ThemeControl(Self) then
  begin
    TS := GetThemeServices();

    if FTransparente then
      TS.DrawParentBackground(Self.Handle, C.Handle, nil, False, Rect
          (0, 0, cw, ch))
    else
      LimparFundo();

    if Status.Enabled then
    begin
      if Status.MouseNoControle then
      begin
        if Status.MousePressionado then
          TB := ttbButtonPressed
        else
          TB := ttbButtonHot
      end
      else
        TB := ttbButtonNormal;
    end
    else
      TB := ttbButtonDisabled;

    if FDrawEdge then
      TS.DrawElement(C.Handle, TS.GetElementDetails(TB), Rect(0, 0, cw, ch));
  end
  else
  begin
    if (not FTransparente) then
      LimparFundo();

    if Status.Enabled then
    begin
      if Status.MouseNoControle then
      begin
        if Status.MousePressionado then
          Edge := BDR_SUNKENINNER
        else
          Edge := BDR_RAISEDINNER;

        R := Rect(0, 0, cw, ch);

        if FDrawEdge then
          DrawEdge(C.Handle, R, Edge, BF_RECT);
      end;
    end;
  end;

  if not Self.CustomDrawToolButton(C, Status) then
    if Status.GraphDesenhar <> nil then
    begin
      PX := cw div 2 - Status.GraphDesenhar.Width div 2;
      PY := ch div 2 - Status.GraphDesenhar.Height div 2;
      C.Draw(PX, PY, Status.GraphDesenhar);
    end
    else if csDesigning in Self.ComponentState then
    begin

    end;

  if (csDesigning in Self.ComponentState) and (Status.GraphDesenhar = nil) then
  begin
    C.Brush.Style := bsClear;
    C.Pen.Style := psSolid;
    C.Pen.Color := clBlack;
    C.Pen.Width := 1;

    R := Self.ClientRect;
    C.Refresh;
    C.Rectangle(R);
  end
end;

procedure TRImageButton.PictureChanged(Sender: TObject);
begin
  Self.CalcularEstado(FStatusNovo);
  Self.Invalidate;
end;

procedure TRImageButton.SetEnabled(Value: Boolean);
begin
  if Value <> Self.Enabled then
  begin
    inherited;
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRImageButton.SetEstilo(const Value: TEstiloImageButton);
begin
  if FEstilo <> Value then
  begin
    FEstilo := Value;
    Self.TabStop := FEstilo = eibButton;

    case FEstilo of
      eibToolbutton:
        ControlStyle := ControlStyle - [csOpaque] + [csParentBackground];
      eibButton:
        ControlStyle := ControlStyle + [csOpaque] - [csParentBackground];
    end;

    if not(csCreating in Self.ControlState) then
    begin
      Self.RecreateWnd();

      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRImageButton.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TRImageButton.SetPictureDesativado(const Value: TPicture);
begin
  FPictureDesativado.Assign(Value);
end;

procedure TRImageButton.SetPictureHover(const Value: TPicture);
begin
  FPictureHover.Assign(Value);
end;

procedure TRImageButton.SetTransparente(const Value: Boolean);
begin
  if FTransparente <> Value then
  begin
    FTransparente := Value;

    if Self.Estilo = eibToolbutton then
      Self.RecreateWnd();
  end;
end;

procedure TRImageButton.SetCaption(const Value: String);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Self.Invalidate;
  end;
end;

procedure TRImageButton.SetDefault(const Value: Boolean);
begin
  if Default <> Value then
  begin
    FDefault := Value;
    Self.CalcularEstado(FStatusNovo);
    Invalidate;
  end;
end;

procedure TRImageButton.WMErase(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TRImageButton.WMPaint(var Message: TWMPaint);
var
  DC: HDC;
  PT: TPaintStruct;
  B: TBufferPaint;
  Anim: TBufferAnim;
  Tempo: Integer;
begin
  DC := Message.DC;
  Message.Result := 1;

  if DC = 0 then
    DC := BeginPaint(Self.Handle, PT);

  try
    if TRRPGDrawBuffer.PrecisaDesenhar(Self.Handle, DC) then
    begin
      if Self.EstadoMudou() then
      begin
        Tempo := TEMPO_ANIM;

        if (FStatusAtual.Focado and not FStatusNovo.Focado) then
          Tempo := TEMPO_DEFOCAR
        else if (FStatusAtual.Situacao = seHot) and
          (FStatusNovo.Situacao = seNormal) then
        begin
          if Self.Estilo = eibToolbutton then
            Tempo := TEMPO_TIRAR_HOT
          else
            Tempo := TEMPO_TIRAR_HOT_HALF;
        end;

        Anim := TRRPGDrawBuffer.BeginAnimation
          (Self.Handle, DC, Self.ClientRect, Tempo);

        try
          if Anim.DCDe <> 0 then
            Self.PaintEm(Anim.DCDe, FStatusAtual);

          if Anim.DCPara <> 0 then
            Self.PaintEm(Anim.DCPara, FStatusNovo);
        finally
          TRRPGDrawBuffer.EndAnimation(Anim);
        end;
      end
      else
      begin
        B := TRRPGDrawBuffer.BeginBufferedPaint(DC, Self.ClientRect);

        try
          Self.PaintEm(B.DC, FStatusAtual);
        finally
          TRRPGDrawBuffer.EndBufferedPaint(B);
        end;
      end;

      FStatusAtual := FStatusNovo;
    end;
  finally
    if Message.DC = 0 then
      EndPaint(Self.Handle, PT);
  end;
end;

procedure TRImageButton.WMPrintClient(var Message: TWMPrintClient);
begin
  Self.PaintEm(Message.DC, FStatusAtual);
  Message.Result := 1;
end;

{ TRCheckBox }

procedure TRCheckBox.CalcularEstado(var Estado: TStatusCheckbox);
begin
  Estado.FTecladoEngatilhado := FTecladoEngatilhado;
  Estado.FMouseNoControle := FMouseNoControle;
  Estado.FMousePressionado := FMousePressionado;
  Estado.FFocado := FFocado;
  Estado.Enabled := Self.Enabled;
  Estado.Checked := Self.Checked;
end;

procedure TRCheckBox.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := True;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRCheckBox.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := False;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRCheckBox.ConfigurarVisual;
begin
  Self.Color := EsquemaAtual.CorWindow;
  Self.Font.Size := 8;
  Self.Font.Name := SegoeFontName();
end;

constructor TRCheckBox.Create(AOwner: TComponent);
begin
  inherited;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);

  if csDesigning in Self.ComponentState then
    Self.ConfigurarVisual();
end;

destructor TRCheckBox.Destroy;
begin
  DesregistrarNotScheme(Self);
  inherited;
end;

procedure TRCheckBox.DoEnter;
begin
  FFocado := True;
  Self.CalcularEstado(FStatusNovo);
  inherited;
end;

procedure TRCheckBox.DoExit;
begin
  FFocado := False;
  Self.CalcularEstado(FStatusNovo);
  inherited;
end;

function TRCheckBox.EstadoMudou: Boolean;
begin
  Result := (FStatusAtual.FTecladoEngatilhado <>
      FStatusNovo.FTecladoEngatilhado) or
    (FStatusAtual.FMouseNoControle <> FStatusNovo.FMouseNoControle) or
    (FStatusAtual.FMousePressionado <> FStatusNovo.FMousePressionado) or
    (FStatusAtual.FFocado <> FStatusNovo.FFocado) or
    (FStatusAtual.Enabled <> FStatusNovo.Enabled) or
    (FStatusAtual.Checked <> FStatusNovo.Checked);
end;

procedure TRCheckBox.Loaded;
begin
  inherited;
  Self.CalcularEstado(FStatusAtual);
  FStatusNovo := FStatusAtual;
  Self.ConfigurarVisual();
end;

procedure TRCheckBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := True;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRCheckBox.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := False;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRCheckBox.PaintCheckBox(DC: HDC; Status: TStatusCheckbox);
var
  GP: IGPGraphics;
  R, R2: TRect;
  GR: TGPRect;
  Brush: IGPBrush;
  Pen: IGPPen;
  cw, ch, TamCheckBox, Xcbx, Ycbx, XTexto, SmallW, SmallH, Temp: Integer;
  TipEsq: TSituacaoEsquema;
  Esq: PCoresSituacaoEsquema;
  PtsV: array [0 .. 2] of TGPPoint;
  DashPattern: array [0 .. 1] of Single;
const
  MARGEM_CHECKBOX = 2;
begin
  GP := TGPGraphics.Create(DC);
  GR := TGPRect.Create(Self.ClientRect);

  // Fundo do CheckBox
  Brush := TGPSolidBrush.Create(TGPColor.CreateFromColorRef(Self.Color));
  GP.FillRectangle(Brush, GR);

  ch := Self.ClientHeight;
  cw := Self.ClientWidth;

  if Status.Enabled then
  begin
    if Status.FTecladoEngatilhado then
      TipEsq := sePressionado
    else if Status.FMouseNoControle then
    begin
      if Status.FMousePressionado then
        TipEsq := sePressionado
      else
        TipEsq := seHot
    end
    else
      TipEsq := seNormal;
  end
  else
    TipEsq := seDisabled;

  Esq := @EsquemaAtual.Cores[TipEsq];

  TamCheckBox := 12;

  if ch < TamCheckBox then
    TamCheckBox := ch;

  if cw < TamCheckBox then
    TamCheckBox := cw;

  Xcbx := MARGEM_CHECKBOX;
  Ycbx := ch div 2 - TamCheckBox div 2;

  // Primeiro Retangulo CheckBox

  R := Rect(Xcbx, Ycbx, Xcbx + TamCheckBox, Ycbx + TamCheckBox);

  GR := TGPRect.Create(R);
  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorBorda));
  GP.DrawRectangle(Pen, GR);

  // Segundo Retangulo CheckBox

  InflateRect(R, -1, -1);

  GR := TGPRect.Create(R);
  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.Cor1));
  GP.DrawRectangle(Pen, GR);

  // Terceiro Retangulo CheckBox

  InflateRect(R, -1, -1);
  GR := TGPRect.Create(R);

  Brush := TGPLinearGradientBrush.Create
    (TGPPointF.Create(0, 0), TGPPointF.Create(R.Right + 1, R.Bottom + 1),

    TGPColor.CreateFromColorRef(Esq.CorBorda3), TGPColor.CreateFromColorRef
      (Esq.Cor1));
  Pen := TGPPen.Create(Brush);
  GP.DrawRectangle(Pen, GR);

  // Quarto Retangulo.. Fill
  // InflateRect(R, -1, -1);
  // GR := TGPRect.Create(R);

  Brush := TGPLinearGradientBrush.Create
    (TGPPointF.Create(0, 0), TGPPointF.Create(R.Right, R.Bottom),

    TGPColor.CreateFromColorRef(Esq.CorBorda3), TGPColor.CreateFromColorRef
      (Esq.Cor1));

  GP.FillRectangle(Brush, GR);

  if Status.Checked then
  begin
    // Marca de Seleo
    Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorNucleo), 2);

    R.Bottom := R.Bottom - 1;
    R.Left := R.Left + 1;
    R.Right := R.Right - 1;

    SmallW := R.Right - R.Left;
    SmallH := R.Bottom - R.Top;

    PtsV[0].Initialize(R.Left, R.Top + ((SmallH div 4) * 3) + 1);
    PtsV[1].Initialize(R.Left + SmallW div 2, R.Bottom);
    PtsV[2].Initialize(R.Right, R.Top);
    GP.SmoothingMode := SmoothingModeHighQuality;
    GP.DrawLines(Pen, PtsV);
    GP.SmoothingMode := SmoothingModeDefault;
  end;

  // Texto do CheckBox

  XTexto := Xcbx + TamCheckBox + MARGEM_CHECKBOX * 2;

  SelectObject(DC, Self.Font.Handle);
  R := Rect(XTexto, 0, cw, ch);
  SetBkMode(DC, Transparent);
  SetTextColor(DC, ColorToRGB(Esq.CorFonte));

  R2 := R;
  DrawText(DC, PChar(Self.Caption), Length(Self.Caption), R2,
    DT_CALCRECT or DT_SINGLELINE);

  Temp := R2.Bottom - R2.Top;
  R.Top := R.Top + (R.Bottom - R.Top) div 2 - Temp div 2;
  R.Bottom := R.Top + Temp;

  DrawText(DC, PChar(Self.Caption), Length(Self.Caption), R,
    DT_SINGLELINE or DT_VCENTER);

  if Status.FFocado then
  begin
    Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorTracejado));
    DashPattern[0] := 1.0;
    DashPattern[1] := 1.0;
    Pen.SetDashPattern(DashPattern);

    R.Left := R.Left - 1;
    R.Right := R.Right - 1;
    R.Bottom := R.Bottom - 1;
    GR := TGPRect.Create(R);
    GP.SmoothingMode := SmoothingModeHighSpeed;
    GP.DrawRectangle(Pen, GR);
  end;
end;

procedure TRCheckBox.SetChecked(Value: Boolean);
begin
  inherited;
  CalcularEstado(FStatusNovo);
end;

procedure TRCheckBox.SetEnabled(Value: Boolean);
var
  Mudou: Boolean;
begin
  Mudou := Self.Enabled <> Value;
  inherited;

  if Mudou then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRCheckBox.Toggle;
begin
  inherited;
  CalcularEstado(FStatusNovo);
end;

procedure TRCheckBox.WMEraseBkg(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TRCheckBox.WMPaint(var Message: TWMPaint);
var
  DC: HDC;
  PT: TPaintStruct;
  B: TBufferPaint;
  A: TBufferAnim;
  Tempo: Integer;
begin
  DC := Message.DC;

  if DC = 0 then
    DC := BeginPaint(Self.Handle, PT);

  try
    if TRRPGDrawBuffer.PrecisaDesenhar(Self.Handle, DC) then
    begin
      if Self.EstadoMudou() then
      begin
        Tempo := TEMPO_ANIM;

        if FStatusAtual.FFocado and not FStatusNovo.FFocado then
          Tempo := TEMPO_DEFOCAR
        else if FStatusAtual.FMouseNoControle and not
          (FStatusNovo.FMouseNoControle) then
          Tempo := TEMPO_TIRAR_HOT;

        A := TRRPGDrawBuffer.BeginAnimation(Self.Handle, DC, Self.ClientRect,
          Tempo);

        try
          if A.DCDe <> 0 then
            Self.PaintCheckBox(A.DCDe, FStatusAtual);

          if A.DCPara <> 0 then
            Self.PaintCheckBox(A.DCPara, FStatusNovo);
        finally
          TRRPGDrawBuffer.EndAnimation(A);
        end;

        FStatusAtual := FStatusNovo;
      end
      else
      begin
        B := TRRPGDrawBuffer.BeginBufferedPaint(DC, ClientRect);

        try
          Self.PaintCheckBox(B.DC, FStatusAtual);
        finally
          TRRPGDrawBuffer.EndBufferedPaint(B);
        end;
      end;
    end;
  finally
    if Message.DC = 0 then
      EndPaint(Self.Handle, PT);
  end;

  Message.Result := 1;
end;

procedure TRCheckBox.WMPrint(var Message: TWMPrint);
begin
  Self.PaintCheckBox(Message.DC, FStatusAtual);
end;

procedure TRCheckBox.WMPrintClient(var Message: TWMPrintClient);
begin
  Self.PaintCheckBox(Message.DC, FStatusAtual);
end;

{ TRRadioButton }

procedure TRRadioButton.CalcularEstado(var Estado: TStatusCheckbox);
begin
  Estado.FTecladoEngatilhado := FTecladoEngatilhado;
  Estado.FMouseNoControle := FMouseNoControle;
  Estado.FMousePressionado := FMousePressionado;
  Estado.FFocado := FFocado;
  Estado.Enabled := Self.Enabled;
  Estado.Checked := Self.Checked;
end;

procedure TRRadioButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := True;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRRadioButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := False;

  if Self.Enabled then
  begin
    Self.CalcularEstado(FStatusNovo);
    Self.Invalidate;
  end;
end;

procedure TRRadioButton.ConfigurarVisual;
begin
  Self.Font.Size := 8;
  Self.Font.Name := SegoeFontName();
end;

constructor TRRadioButton.Create(AOwner: TComponent);
begin
  inherited;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);

  if csDesigning in Self.ComponentState then
    Self.ConfigurarVisual();
end;

destructor TRRadioButton.Destroy;
begin
  DesregistrarNotScheme(Self);
  inherited;
end;

procedure TRRadioButton.DoEnter;
begin
  FFocado := True;
  Self.CalcularEstado(FStatusNovo);
  inherited;
end;

procedure TRRadioButton.DoExit;
begin
  FFocado := False;
  Self.CalcularEstado(FStatusNovo);
  inherited;
end;

function TRRadioButton.EstadoMudou: Boolean;
begin
  Result := (FStatusAtual.FTecladoEngatilhado <>
      FStatusNovo.FTecladoEngatilhado) or
    (FStatusAtual.FMouseNoControle <> FStatusNovo.FMouseNoControle) or
    (FStatusAtual.FMousePressionado <> FStatusNovo.FMousePressionado) or
    (FStatusAtual.FFocado <> FStatusNovo.FFocado) or
    (FStatusAtual.Enabled <> FStatusNovo.Enabled) or
    (FStatusAtual.Checked <> FStatusNovo.Checked);
end;

procedure TRRadioButton.Loaded;
begin
  inherited;
  Self.CalcularEstado(FStatusAtual);
  FStatusNovo := FStatusAtual;
  Self.ConfigurarVisual();
end;

procedure TRRadioButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := True;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRRadioButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := False;
      Self.CalcularEstado(FStatusNovo);
      Self.Invalidate;
    end;
  end;
end;

procedure TRRadioButton.PaintRadio(DC: HDC; Status: TStatusCheckbox);
var
  GP: IGPGraphics;
  R: TRect;
  GR: TGPRect;
  Brush: IGPBrush;
  Pen: IGPPen;
  cw, ch, TamRadio, Xrdb, Yrdb, XTexto: Integer;
  TipEsq: TSituacaoEsquema;
  Esq: PCoresSituacaoEsquema;
  DashPattern: array [0 .. 1] of Single;
const
  MARGEM_RADIO = 2;
begin
  GP := TGPGraphics.Create(DC);
  GR := TGPRect.Create(Self.ClientRect);

  // Fundo do CheckBox
  Brush := TGPSolidBrush.Create(TGPColor.CreateFromColorRef(Self.Color));
  GP.FillRectangle(Brush, GR);

  ch := Self.ClientHeight;
  cw := Self.ClientWidth;

  if Status.Enabled then
  begin
    if Status.FTecladoEngatilhado then
      TipEsq := sePressionado
    else if Status.FMouseNoControle then
    begin
      if Status.FMousePressionado then
        TipEsq := sePressionado
      else
        TipEsq := seHot
    end
    else
      TipEsq := seNormal;
  end
  else
    TipEsq := seDisabled;

  Esq := @EsquemaAtual.Cores[TipEsq];

  TamRadio := 12;

  if ch < TamRadio then
    TamRadio := ch;

  if cw < TamRadio then
    TamRadio := cw;

  Xrdb := MARGEM_RADIO;
  Yrdb := ch div 2 - TamRadio div 2;

  // Primeiro Circulo Radio

  R := Rect(Xrdb, Yrdb, Xrdb + TamRadio, Yrdb + TamRadio);

  GR := TGPRect.Create(R);
  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorBorda));
  GP.SmoothingMode := SmoothingModeHighQuality;
  GP.DrawEllipse(Pen, GR);

  // Segundo Circulo Radio

  InflateRect(R, -1, -1);

  GR := TGPRect.Create(R);
  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.Cor1));
  GP.DrawEllipse(Pen, GR);

  // Terceiro Circulo Radio

  InflateRect(R, -1, -1);
  GR := TGPRect.Create(R);

  Brush := TGPLinearGradientBrush.Create
    (TGPPointF.Create(0, 0), TGPPointF.Create(R.Right + 1, R.Bottom + 1),

    TGPColor.CreateFromColorRef(Esq.CorBorda3), TGPColor.CreateFromColorRef
      (Esq.Cor1));
  Pen := TGPPen.Create(Brush);
  GP.DrawEllipse(Pen, GR);

  // Quarto Circulo. Fill

  Brush := TGPLinearGradientBrush.Create
    (TGPPointF.Create(0, 0), TGPPointF.Create(R.Right, R.Bottom),

    TGPColor.CreateFromColorRef(Esq.CorBorda3), TGPColor.CreateFromColorRef
      (Esq.Cor1));

  GP.FillEllipse(Brush, GR);

  if Status.Checked then
  begin
    // Marca de Seleo - Borda Externa
    Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorNucleo), 2);
    InflateRect(R, -1, -1);

    GR := TGPRect.Create(R);
    GP.DrawEllipse(Pen, GR);

    // Marca de Seleo - Miolo

    Brush := TGPLinearGradientBrush.Create(TGPPointF.Create(R.Left, R.Top),
      TGPPointF.Create(R.Right, R.Bottom),

      TGPColor.CreateFromColorRef(Esq.Cor1), TGPColor.CreateFromColorRef
        (Esq.CorNucleo2));
    GR := TGPRect.Create(R);
    GP.FillEllipse(Brush, GR);
  end;

  // Texto do CheckBox

  XTexto := Xrdb + TamRadio + MARGEM_RADIO * 2;

  SelectObject(DC, Self.Font.Handle);
  R := Rect(XTexto, 0, cw, ch);
  SetBkMode(DC, Transparent);
  SetTextColor(DC, ColorToRGB(Esq.CorFonte));
  DrawText(DC, PChar(Self.Caption), Length(Self.Caption), R,
    DT_CALCRECT or DT_SINGLELINE or DT_VCENTER);

  OffsetRect(R, 0, 1);

  DrawText(DC, PChar(Self.Caption), Length(Self.Caption), R,
    DT_SINGLELINE or DT_VCENTER);

  if Status.FFocado then
  begin
    Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Esq.CorTracejado));
    DashPattern[0] := 1.0;
    DashPattern[1] := 1.0;
    Pen.SetDashPattern(DashPattern);

    R.Left := R.Left - 1;
    R.Bottom := R.Bottom - 1;
    R.Right := R.Right - 1;
    GR := TGPRect.Create(R);
    GP.SmoothingMode := SmoothingModeHighSpeed;
    GP.DrawRectangle(Pen, GR);
  end;
end;

procedure TRRadioButton.SetChecked(Value: Boolean);
begin
  if Value <> Self.Checked then
  begin
    Self.CalcularEstado(FStatusNovo);
    FStatusNovo.Checked := Value;
  end;

  inherited;
end;

procedure TRRadioButton.SetEnabled(Value: Boolean);
var
  Mudou: Boolean;
begin
  Mudou := Self.Enabled <> Value;

  if Mudou then
  begin
    Self.CalcularEstado(FStatusNovo);
    FStatusNovo.Enabled := Value;
  end;

  inherited;

  if Mudou then
    Self.Invalidate;
end;

procedure TRRadioButton.WMErase(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TRRadioButton.WMGetText(var Message: TWMGetText);
var
  Ct: Integer;
begin
  Ct := Message.TextMax - 1;

  if Ct > Length(FCaption) then
    Ct := Length(FCaption);

  if Ct > 0 then
    Move(FCaption[1], Message.Text^, Ct * SizeOf(Char));

  Message.Text[Ct] := #0;
  Message.Result := Ct;
end;

procedure TRRadioButton.WMGetTextLength(var Message: TWMGetTextLength);
begin
  Message.Result := Length(FCaption);
end;

procedure TRRadioButton.WMPaint(var Message: TWMPaint);
var
  DC: HDC;
  PT: TPaintStruct;
  B: TBufferPaint;
  A: TBufferAnim;
  Tempo: Integer;
begin
  DC := Message.DC;

  if DC = 0 then
    DC := BeginPaint(Self.Handle, PT);

  try
    if TRRPGDrawBuffer.PrecisaDesenhar(Self.Handle, DC) then
    begin
      if Self.EstadoMudou() then
      begin
        Tempo := TEMPO_ANIM;

        if FStatusAtual.FFocado and not FStatusNovo.FFocado then
          Tempo := TEMPO_DEFOCAR
        else if FStatusAtual.FMouseNoControle and not
          (FStatusNovo.FMouseNoControle) then
          Tempo := TEMPO_TIRAR_HOT;

        A := TRRPGDrawBuffer.BeginAnimation(Self.Handle, DC, Self.ClientRect,
          Tempo);

        try
          if A.DCDe <> 0 then
            Self.PaintRadio(A.DCDe, FStatusAtual);

          if A.DCPara <> 0 then
            Self.PaintRadio(A.DCPara, FStatusNovo);
        finally
          TRRPGDrawBuffer.EndAnimation(A);
        end;

        FStatusAtual := FStatusNovo;
      end
      else
      begin
        B := TRRPGDrawBuffer.BeginBufferedPaint(DC, Self.ClientRect);

        try
          Self.PaintRadio(B.DC, FStatusAtual);
        finally
          TRRPGDrawBuffer.EndBufferedPaint(B);
        end;
      end;
    end;
  finally
    if Message.DC = 0 then
      EndPaint(Self.Handle, PT);
  end;

  Message.Result := 1;
end;

procedure TRRadioButton.WMPrint(var Message: TWMPrint);
begin
  if Message.DC <> 0 then
    Self.PaintRadio(Message.DC, FStatusAtual);
end;

procedure TRRadioButton.WMPrintClient(var Message: TWMPrintClient);
begin
  if Message.DC <> 0 then
    Self.PaintRadio(Message.DC, FStatusAtual);
end;

procedure TRRadioButton.WMSetText(var Message: TWMSetText);
begin
  // inherited;
  FCaption := Message.Text;
  Self.CalcularEstado(FStatusNovo);

  if Self.HandleAllocated then
    RedrawWindow(Self.Handle, nil, 0, RDW_INVALIDATE or RDW_UPDATENOW);
end;

{ TREdit }

procedure TREdit.CMMouseEnter(var Message: TMessage);
begin
  FMouseNoControle := True;
  inherited;

  if not IsMaiorIgualWinVista then // Windows XP
    Self.Invalidate;
end;

procedure TREdit.CMMouseLeave(var Message: TMessage);
begin
  FMouseNoControle := False;
  inherited;

  if not IsMaiorIgualWinVista then // Windows XP
    Self.Invalidate;
end;

procedure TREdit.CNCtlColorEdit(var Message: TMessage);
begin
  inherited;
  Self.ExcluirChildRect(Message.WParam);
end;

procedure TREdit.CNCtlColorStatic(var Message: TMessage);
begin
  inherited;
  Self.ExcluirChildRect(Message.WParam);
end;

procedure TREdit.ConfigurarVisual;
begin
  Self.Color := EsquemaAtual.CorWindow;
  Self.Font.Color := EsquemaAtual.Cores[seNormal].CorFonte;
  // Self.Font.Size := 8;
  Self.Font.Name := SegoeFontName();
end;

constructor TREdit.Create(AOwner: TComponent);
begin
  inherited;
  FCanvasDraw := TCanvas.Create;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);

  if csDesigning in Self.ComponentState then
    Self.ConfigurarVisual();
end;

procedure TREdit.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.ExStyle := Params.ExStyle or WS_EX_COMPOSITED;
end;

destructor TREdit.Destroy;
begin
  DesregistrarNotScheme(Self);
  FCanvasDraw.Free;
  inherited;
end;

procedure TREdit.DoEnter;
begin
  inherited;

  if not IsMaiorIgualWinVista then // Windows XP
  begin
    Self.Invalidate;
    RedrawWindow(Self.Handle, Self.ClientRect, 0,
      RDW_INVALIDATE or RDW_FRAME or RDW_INTERNALPAINT);
  end;
end;

procedure TREdit.DoExit;
begin
  inherited;

  if not IsMaiorIgualWinVista then // Windows XP
  begin
    Self.Invalidate;
    RedrawWindow(Self.Handle, Self.ClientRect, 0,
      RDW_INVALIDATE or RDW_FRAME or RDW_INTERNALPAINT);
  end;
end;

procedure TREdit.ExcluirChildRect(DC: HDC);
var
  i: Integer;
begin
  for i := 0 to Self.ControlCount - 1 do
    ExcludeClipRect(DC, Controls[i].Left, Controls[i].Top,
      Controls[i].Left + Controls[i].Width, Controls[i].Top + Controls[i]
        .Height);
end;

procedure TREdit.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

procedure TREdit.WMNCPaint(var Message: TWMNCPaint);
var
  DC: HDC;
  C: TCanvas;
  R: TRect;
  Esq: PCoresSituacaoEsquema;
  TipEsq: TSituacaoEsquema;
  Flat: Boolean;
begin
  if Self.BorderStyle = bsNone then
  begin
    inherited;
    Exit;
  end;

  Message.Result := 1;
  DC := GetWindowDC(Self.Handle);

  try
    C := FCanvasDraw;
    C.Handle := DC;

    try
      R := Rect(0, 0, Self.Width, Self.Height);
      C.Brush.Color := Self.Color;
      C.Pen.Style := psSolid;
      C.Pen.Color := C.Brush.Color;
      SelectClipRgn(C.Handle, Message.RGN);

      // Self.ExcluirChildRect(C.Handle);
      C.Rectangle(R);

      C.Pen.Width := 1;

      if Self.Enabled then
      begin
        if Self.Focused then
          TipEsq := sePressionado
        else if FMouseNoControle then
          TipEsq := seHot
        else
          TipEsq := seNormal;
      end
      else
        TipEsq := seDisabled;

      Esq := @EsquemaAtual.Cores[TipEsq];
      Flat := (Self.BevelInner = bvNone) and (Self.BevelKind = bkNone) and
        (Self.BevelOuter = bvNone);

      if Flat then
      begin
        // if TipEsq in [seNormal, seDisabled] then
        C.Pen.Color := Esq.BordasEdit[0]
        // else
        // C.Pen.Color := EsquemaAtual.CorTextoNormal;
      end
      else
        C.Pen.Color := Esq.BordasEdit[0];

      C.MoveTo(0, 0);
      C.LineTo(Self.Width - 1, 0);

      if not Flat then
        C.Pen.Color := Esq.BordasEdit[1];

      C.LineTo(Self.Width - 1, Self.Height - 1);

      if not Flat then
        C.Pen.Color := Esq.BordasEdit[2];

      C.LineTo(0, Self.Height - 1);

      if not Flat then
        C.Pen.Color := Esq.BordasEdit[3];

      C.LineTo(0, 0);
    finally
      C.Handle := 0;
    end;

    // for i := 0 to Self.ControlCount - 1 do // Desenhar Filhos
    // Controls[i].Invalidate;

    // Controls[i].Repaint;

    { for i := 0 to Self.ControlCount - 1 do // Desenhar Filhos
      ExcludeClipRect(C.Handle, Controls[i].Left, Controls[i].Top,
      Controls[i].Left + Controls[i].Width,
      Controls[i].Top + Controls[i].Height); }
  finally
    ReleaseDC(Self.Handle, DC)
  end;
end;

{ TRPanelCustomizado }

procedure TRPanelCustomizado.ConfigurarVisual;
begin

end;

constructor TRPanelCustomizado.Create(AOwner: TComponent);
begin
  inherited;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);

  if csDesigning in Self.ComponentState then
    Self.ConfigurarVisual();
end;

destructor TRPanelCustomizado.Destroy;
begin
  DesregistrarNotScheme(Self);
  inherited;
end;

procedure TRPanelCustomizado.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

{ TRHeaderPanel }

procedure TRHeaderPanel.ConfigurarVisual;
begin
  inherited;
  Self.TipoFundo := tdgVerticalTitulo;
  Self.Color := EsquemaAtual.CorHeaderPanel;
  Self.CorSecundaria := EsquemaAtual.CorHeaderPanel2;
end;

constructor TRHeaderPanel.Create(AOwner: TComponent);
begin
  inherited;

  if csDesigning in Self.ComponentState then
  begin
    Self.Height := 100;
    Self.ConfigurarVisual();
  end;
end;

{ TRMainPanel }

procedure TRMainPanel.ConfigurarVisual;
begin
  inherited;
  Self.TipoFundo := tdgVertical2quartos;
  Self.Color := EsquemaAtual.CorHeaderPanel;
  Self.CorSecundaria := EsquemaAtual.CorHeaderPanel2;
  Self.ParentBackground := False;
  Self.ParentColor := False;
end;

{ TRMContentPanel }

procedure TRContentPanel.ConfigurarVisual;
begin
  inherited;

  if EsquemaAtual.CorHeaderPanel = 0 then
    Self.Color := $010101
  else
    Self.Color := EsquemaAtual.CorHeaderPanel;

  Self.CorBorda := EsquemaAtual.CorBordaPanel;
  Self.DesenharBorda := True;
  Self.Alpha := 230;
end;

{ TRSpeedButton }

procedure TRSpeedButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := True;

  if Self.Enabled then
    Self.Invalidate;
end;

procedure TRSpeedButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseNoControle := False;

  if Self.Enabled then
    Self.Invalidate;
end;

constructor TRSpeedButton.Create(AOwner: TComponent);
begin
  inherited;
  FPicture := TPicture.Create;
  FPicture.OnChange := Self.PictureChanged;
  FPictureHover := TPicture.Create;
  FPictureHover.OnChange := Self.PictureChanged;

  FPictureDesativado := TPicture.Create;
  FPictureDesativado.OnChange := Self.PictureChanged;

  // Self.InterceptMouse := True;
  Self.Width := 75;
  Self.Height := 25;
end;

destructor TRSpeedButton.Destroy;
begin

  inherited;
end;

procedure TRSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := True;
      Self.Invalidate;
    end;
  end;
end;

procedure TRSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if Self.Enabled then
    begin
      FMousePressionado := False;
      Self.Invalidate;
    end;
  end;
end;

procedure TRSpeedButton.Paint;
begin
  Self.PaintToolButton(Self.Canvas);
end;

procedure TRSpeedButton.PaintToolButton(C: TCanvas);
var
  ch, cw, PX, PY, Edge, WTexto, XTexto: Integer;
  TS: TClasseThemeServices;
  TB: TThemedToolBar;
  R: TRect;
  PicDraw: TPicture;
  TempStr: String;
const
  MARGEM_IMAGEM = 5;
begin
  ch := Self.ClientHeight;
  cw := Self.ClientWidth;

  if ThemeControl(Self) then
  begin
    TS := GetThemeServices();

    if Self.Enabled then
    begin
      if FMouseNoControle then
      begin
        if FMousePressionado then
          TB := ttbButtonPressed
        else
          TB := ttbButtonHot
      end
      else
        TB := ttbButtonNormal;
    end
    else
      TB := ttbButtonDisabled;

    TS.DrawElement(C.Handle, TS.GetElementDetails(TB), Rect(0, 0, cw, ch));
  end
  else
  begin
    if Self.Enabled then
    begin
      if FMouseNoControle then
      begin
        if FMousePressionado then
          Edge := BDR_SUNKENINNER
        else
          Edge := BDR_RAISEDINNER;

        R := Rect(0, 0, cw, ch);

        DrawEdge(C.Handle, R, Edge, BF_RECT);
      end;
    end;
  end;

  if Self.Enabled and FMouseNoControle and (FPictureHover.Graphic <> nil) then
    PicDraw := FPictureHover
  else if (not Self.Enabled) and (FPictureDesativado.Graphic <> nil) then
    PicDraw := FPictureDesativado
  else
    PicDraw := FPicture;

  if Self.Caption <> '' then
  begin
    R.Left := 0;
    R.Right := cw;
    R.Top := 0;
    R.Bottom := ch;
    TempStr := Self.Caption;
    C.TextRect(R, TempStr, [tfCalcRect, tfCenter, tfVerticalCenter,
      tfSingleLine]);

    WTexto := R.Right - R.Left;
  end
  else
    WTexto := 0;

  if PicDraw.Graphic <> nil then
  begin
    PY := ch div 2 - PicDraw.Graphic.Height div 2;

    if WTexto <> 0 then
      PX := cw div 2 - (PicDraw.Graphic.Width + WTexto + MARGEM_IMAGEM) div 2
    else
      PX := cw div 2 - PicDraw.Graphic.Width div 2;

    XTexto := PX + PicDraw.Graphic.Width + MARGEM_IMAGEM;
    C.Draw(PX, PY, PicDraw.Graphic);
  end
  else
    XTexto := cw div 2 - WTexto div 2;

  if Self.Caption <> '' then
  begin
    C.Font.Assign(Self.Font);
    SetBkMode(C.Handle, Transparent);
    C.Font.Color := EsquemaAtual.Cores[seNormal].CorFonte;

    R.Top := 0;
    R.Bottom := ch;
    R.Left := XTexto;
    R.Right := XTexto + WTexto;
    TempStr := Self.Caption;
    C.TextRect(R, TempStr, [tfCenter, tfVerticalCenter, tfSingleLine]);
  end;

  (* if PicDraw.Graphic <> nil then
    begin
    PX := cw div 2 - PicDraw.Graphic.Width div 2;
    PY := ch div 2 - PicDraw.Graphic.Height div 2;
    C.Draw(PX, PY, PicDraw.Graphic);
    end
    else *)

  if (csDesigning in Self.ComponentState) and (PicDraw.Graphic = nil) then
  begin
    C.Brush.Style := bsClear;
    C.Pen.Style := psSolid;
    C.Pen.Color := clBlack;
    C.Pen.Width := 1;

    R := Self.ClientRect;
    C.Refresh;
    C.Rectangle(R);
  end
end;

procedure TRSpeedButton.PictureChanged(Sender: TObject);
begin
  Self.Invalidate;
end;

procedure TRSpeedButton.SetCaption(const Value: String);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Self.Invalidate;
  end;
end;

procedure TRSpeedButton.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TRSpeedButton.SetPictureDesativado(const Value: TPicture);
begin
  FPictureDesativado.Assign(Value);
end;

procedure TRSpeedButton.SetPictureHover(const Value: TPicture);
begin
  FPictureHover.Assign(Value);
end;

{ TRSearchBox }

procedure TRSearchBox.Change;
var
  Vazio: Boolean;
begin
  inherited;
  Vazio := Trim(Self.Text) = '';

  if Vazio then
  begin
    if FBotBuscar.FConfigAtual <> casBuscar then
      FBotBuscar.SetConfig(casBuscar);
  end
  else if FBotBuscar.FConfigAtual <> casCancelar then
    FBotBuscar.SetConfig(casCancelar);

  FTickLastChange := Windows.GetTickCount;
  FTimer.Enabled := True;
end;

constructor TRSearchBox.Create(AOwner: TComponent);
  procedure ConfigurarBotao(Bot: TRImageButton);
  begin
    Bot.Width := 14;
    Bot.Align := alRight;
    Bot.Enabled := False;
    Bot.Transparente := False;
    Bot.Estilo := eibToolbutton;
  end;

begin
  FTimer := TTimer.Create(nil);
  FTimer.OnTimer := Self.OnTimer;
  FTimer.Enabled := False;
  FTimer.Interval := 100;
  inherited;
  FBotBuscar := TSearchImageButton.Create(nil);
  FBotBuscar.OnClick := Self.OnClickSearchButton;

  ConfigurarBotao(FBotBuscar);

  if not(csDesigning in Self.ComponentState) then
    FBotBuscar.Parent := Self;

  FTickLastChange := Windows.GetTickCount();
end;

procedure TRSearchBox.CreateWnd;
begin
  inherited;

  if not(csDesigning in Self.ComponentState) then
    SendMessage(Self.Handle, EM_SETMARGINS, EC_RIGHTMARGIN, MakeLong(0, 17));
end;

destructor TRSearchBox.Destroy;
begin
  FTimer.Free;
  FBotBuscar.Free;
  inherited;
end;

procedure TRSearchBox.DestroyWnd;
begin
  FBotBuscar.Parent := nil;
  inherited;
end;

procedure TRSearchBox.KeyPress(var Key: Char);
begin
  inherited;

  if Ord(Key) = VK_ESCAPE then
  begin
    Key := #0;
    Self.OnClickSearchButton(Self);
  end;
end;

procedure TRSearchBox.OnClickSearchButton(Sender: TObject);
begin
  Self.Clear;
  FTimer.Enabled := False;

  if Assigned(FOnSearch) then
    FOnSearch(Self, Self.Text);

  if Self.CanFocus then
    try
      Self.SetFocus;
    except
      // Ignorar Exception aqui
    end;
end;

procedure TRSearchBox.OnTimer(Sender: TObject);
begin
  if Windows.GetTickCount - FTickLastChange >= 300 then
  begin
    FTimer.Enabled := False;

    if Assigned(FOnSearch) then
      FOnSearch(Self, Self.Text);
  end;
end;

{ TSearchCancelImageButton }

{ TSearchImageButton }

constructor TSearchImageButton.Create(AOwner: TComponent);
begin
  inherited;
  FConfigAtual := casBuscar;
  Self.Cursor := crDefault;
  Self.Enabled := False;
  FDrawEdge := False;
end;

procedure TSearchImageButton.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.Style := Params.Style or WS_CLIPSIBLINGS;
end;

procedure TSearchImageButton.CustomDrawBuscar(C: TCanvas;
  Status: TStatusImageButton);
var
  GP: IGPGraphics;
  Pen: IGPPen;
  TamQuad, cw, ch, CX, CY, BoundX, BoundY, DistX, DistY: Integer;
  R, RCirculo: TRect;
  RaioCirculo, Hipotenusa, FatorCos, FatorSin: Double;
  Cor: TColor;
begin
  GP := TGPGraphics.Create(C.Handle);

  cw := Self.ClientWidth;
  ch := Self.ClientHeight;
  TamQuad := Min(cw, ch);

  R.Left := cw div 2 - TamQuad div 2;
  R.Top := ch div 2 - TamQuad div 2;
  R.Right := R.Left + TamQuad;
  R.Bottom := R.Top + TamQuad;

  CX := R.Left + Round(TamQuad * 0.5);
  CY := R.Top + Round(TamQuad * 0.4);
  RaioCirculo := TamQuad * 0.3;

  RCirculo.Top := Round(CY - RaioCirculo);
  RCirculo.Left := Round(CX - RaioCirculo);
  RCirculo.Right := Round(RCirculo.Left + RaioCirculo * 2);
  RCirculo.Bottom := Round(RCirculo.Top + RaioCirculo * 2);

  DistX := CX - R.Left;
  DistY := R.Top - CY;
  Hipotenusa := Sqrt(DistX * DistX + DistY * DistY);

  FatorCos := DistY / Hipotenusa; // Cosseno
  FatorSin := DistX / Hipotenusa; // Seno

  BoundX := Round(CX + FatorCos * RaioCirculo);
  BoundY := Round(CY + FatorSin * RaioCirculo);

  // Cor := EsquemaAtual.Cores[seNormal].BordasEdit[0];
  Cor := EsquemaAtual.CorSearchIcon;
  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Cor), 2);

  GP.SmoothingMode := SmoothingModeHighQuality;
  GP.DrawLine(Pen, TGPPoint.Create(R.Left, R.Bottom), TGPPoint.Create
      (BoundX, BoundY));
  GP.DrawEllipse(Pen, TGPRect.Create(RCirculo));
end;

procedure TSearchImageButton.CustomDrawCancelar(C: TCanvas;
  Status: TStatusImageButton);
var
  GP: IGPGraphics;
  Pen: IGPPen;
  TamQuad, cw, ch: Integer;
  R: TRect;
  Cor: TColor;
begin
  GP := TGPGraphics.Create(C.Handle);

  cw := Self.ClientWidth;
  ch := Self.ClientHeight;
  TamQuad := Min(cw - 6, ch - 6);

  R.Left := cw div 2 - TamQuad div 2;
  R.Top := ch div 2 - TamQuad div 2;
  R.Right := R.Left + TamQuad;
  R.Bottom := R.Top + TamQuad;

//  CX := R.Left + Round(TamQuad * 0.5);
//  CY := R.Top + Round(TamQuad * 0.5);

  if Status.Situacao = seNormal then
    Cor := EsquemaAtual.CorSearchIcon
  else
    Cor := EsquemaAtual.Cores[Status.Situacao].BordasEdit[0];

  Pen := TGPPen.Create(TGPColor.CreateFromColorRef(Cor), 2);

  GP.SmoothingMode := SmoothingModeHighQuality;
  GP.DrawLine(Pen, TGPPoint.Create(R.Left, R.Top), TGPPoint.Create
      (R.Right, R.Bottom));
  GP.DrawLine(Pen, TGPPoint.Create(R.Left, R.Bottom), TGPPoint.Create
      (R.Right, R.Top));
end;

function TSearchImageButton.CustomDrawToolButton
  (C: TCanvas; Status: TStatusImageButton): Boolean;
begin
  Result := True;

  if FConfigAtual = casBuscar then
    Self.CustomDrawBuscar(C, Status)
  else
    Self.CustomDrawCancelar(C, Status);
end;

procedure TSearchImageButton.SetConfig(NovaConfig: TConfigAtualSB);
begin
  if FConfigAtual <> NovaConfig then
  begin
    case NovaConfig of
      casBuscar:
        begin
          Self.Cursor := crDefault;
          Self.Enabled := False;
        end;

      casCancelar:
        begin
          Self.Cursor := crHandPoint;
          Self.Enabled := True;
        end;
    end;

    FConfigAtual := NovaConfig;
  end;
end;

procedure TSearchImageButton.SetParent(AParent: TWinControl);
var
  Mudou: Boolean;
begin
  Mudou := Self.Parent <> AParent;
  inherited;

  if Mudou and (AParent <> nil) then
  begin
    FMouseNoControle := False;
    FMousePressionado := False;
    Self.CalcularEstado(FStatusNovo);
  end;
end;

{ TRMemo }

procedure TRMemo.ConfigurarVisual;
begin
  Self.Color := EsquemaAtual.CorWindow;
  Self.Font.Color := EsquemaAtual.Cores[seNormal].CorFonte;
end;

constructor TRMemo.Create(AOwner: TComponent);
begin
  inherited;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);
end;

destructor TRMemo.Destroy;
begin
  DesregistrarNotScheme(Self);
  inherited;
end;

procedure TRMemo.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

end.
