unit RRPGDBControls;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, DBCtrls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, UxTheme, RRPGURLGetter,
  RRPGControls;  // DBCtrls

type
  TRDBUrlImage = class;               // System


  TRDBUrlImage = class(TRCustomURLImage)
  private
    FDataLink: TFieldDataLink;

    function GetField: TField;
  protected
    // Eventos do DataLink
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);

    // Outros
    procedure CarregarImagem; virtual;

    // Outros Mtodos
    function GetDataField: string; virtual;
    function GetDataSource: TDataSource; virtual;
    procedure SetDataField(const Value: string); virtual;
    procedure SetDataSource(Value: TDataSource); virtual;
    procedure SetReadOnly(Value: Boolean); virtual;
    function GetReadOnly: Boolean; virtual;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Field: TField read GetField;
  published
    property Align;
    property Anchors;
    property BorderStyle;
    property Center;
    property Color;
    property Constraints;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property Stretch;
    property Proporcional;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;

    property OtimizarDesenho;
    property BoaQualidade;
  end;

  TRDBBolinhaBase = class(TGraphicControl)
  private
    FDataLink: TFieldDataLink;
    FPenVerdadeiro: TPen;
    FPenFalso: TPen;
    FBrushVerdadeiro: TBrush;
    FBrushFalso: TBrush;
    FBrushFundo: TBrush;
    procedure SetBrushFundo(const Value: TBrush);
  private
    function GetField: TField;
    procedure SetBrushFalso(const Value: TBrush);
    procedure SetBrushVerdadeiro(const Value: TBrush);
    procedure SetPenFalso(const Value: TPen);
    procedure SetPenVerdadeiro(const Value: TPen);
  protected
    // Eventos de Estilo
    procedure OnEstiloChanged(Sender: TObject); virtual;

    // Eventos do DataLink
    procedure DataChange(Sender: TObject); virtual;
    procedure UpdateData(Sender: TObject); virtual;

    // Outros Mtodos
    function GetDataField: string; virtual;
    function GetDataSource: TDataSource; virtual;
    procedure SetDataField(const Value: string); virtual;
    procedure SetDataSource(Value: TDataSource); virtual;
    procedure SetReadOnly(Value: Boolean); virtual;
    function GetReadOnly: Boolean; virtual;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure DesenharFundo;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Field: TField read GetField;      
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;

    property PenVerdadeiro: TPen read FPenVerdadeiro write SetPenVerdadeiro;
    property PenFalso: TPen read FPenFalso write SetPenFalso;
    property BrushVerdadeiro: TBrush read FBrushVerdadeiro write SetBrushVerdadeiro;
    property BrushFalso: TBrush read FBrushFalso write SetBrushFalso;
    property BrushFundo: TBrush read FBrushFundo write SetBrushFundo;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ParentColor;
  end;

  TRDBBolinhaBooleana = class(TRDBBolinhaBase)
  private

    FIsVerdadeiro: Boolean;
  protected
    // Eventos do DataLink
    procedure DataChange(Sender: TObject); override;
    procedure UpdateData(Sender: TObject); override;

    function GetIsVerdadeiro: Boolean; virtual;
    procedure SetIsVerdadeiro(const Valor: Boolean); virtual;

    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TRDBBolinhaNumerica = class(TRDBBolinhaBase)
  private
    FNumeroMaximo: Integer;
    FValorAtual: Integer;
    FEspacamento: Integer;
    procedure SetNumeroMaximo(const Value: Integer);
    procedure SetEspacamento(const Value: Integer);
  protected
    // Eventos do DataLink
    procedure DataChange(Sender: TObject); override;
    procedure UpdateData(Sender: TObject); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X: Integer; Y: Integer); override;

    procedure CalcularAlturaLarguraBolinha(var Altura, Largura: Integer); virtual;
  public
    constructor Create(AOwner: TComponent); override;    
  published
    property NumeroMaximo: Integer read FNumeroMaximo write SetNumeroMaximo default 5;
    property Espacamento: Integer read FEspacamento write SetEspacamento default 4;
  end;

implementation

{ TRDBUrlImage }

procedure TRDBUrlImage.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

constructor TRDBUrlImage.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TRDBUrlImage.Destroy;
begin
  FreeAndNil(FDataLink);
  inherited;
end;

procedure TRDBUrlImage.DataChange(Sender: TObject);
begin
  Self.CarregarImagem();
end;

function TRDBUrlImage.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TRDBUrlImage.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TRDBUrlImage.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TRDBUrlImage.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TRDBUrlImage.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  case Operation of
    opRemove:
      if FDataLink <> nil then
        if AComponent = Self.DataSource then
          Self.DataSource := nil;
  end;
end;

procedure TRDBUrlImage.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TRDBUrlImage.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;

  if Value <> nil then
    Value.RemoveFreeNotification(Self);
end;


procedure TRDBUrlImage.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TRDBUrlImage.UpdateData(Sender: TObject);
begin

end;

procedure TRDBUrlImage.CarregarImagem;
var
  F: TField;
  URL: String;
begin
  F := FDataLink.Field;

  if F = nil then
    URL := ''
  else
    URL := F.AsString;

  Self.CarregarImagemDeURL(URL);
end;

{ TRDBBolinha }

procedure TRDBBolinhaBase.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

constructor TRDBBolinhaBase.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;

  FPenVerdadeiro := TPen.Create;
  FPenVerdadeiro.OnChange := Self.OnEstiloChanged;

  FPenFalso := TPen.Create;
  FPenFalso.OnChange := Self.OnEstiloChanged;

  FBrushVerdadeiro := TBrush.Create;
  FBrushVerdadeiro.OnChange := Self.OnEstiloChanged;

  FBrushFalso := TBrush.Create;
  FBrushFalso.OnChange := Self.OnEstiloChanged;

  FBrushFundo := TBrush.Create;
  FBrushFundo.OnChange := Self.OnEstiloChanged;

  FPenVerdadeiro.Color := clBlack;
  FPenFalso.Color := clBlack;
  FBrushVerdadeiro.Color := clBlack;
  
  FBrushFalso.Style := bsClear;
  FBrushFundo.Style := bsClear;

  Self.Width := 32;
  Self.Height := 32;
end;

destructor TRDBBolinhaBase.Destroy;
begin
  FreeAndNil(FBrushFundo);
  FreeAndNil(FBrushFalso);
  FreeAndNil(FBrushVerdadeiro);
  FreeAndNil(FPenFalso);
  FreeAndNil(FPenVerdadeiro);

  FreeAndNil(FDataLink);
  inherited;
end;

constructor TRDBBolinhaBooleana.Create(AOwner: TComponent);
begin
  inherited;
  Self.Width := 32;
  Self.Height := 32;
end;

destructor TRDBBolinhaBooleana.Destroy;
begin
  inherited;
end;

procedure TRDBBolinhaBase.DataChange(Sender: TObject);
begin

end;

function TRDBBolinhaBase.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TRDBBolinhaBase.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TRDBBolinhaBase.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TRDBBolinhaBase.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TRDBBolinhaBase.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  
  case Operation of
    opRemove:
      if FDataLink <> nil then
        if AComponent = Self.DataSource then
          Self.DataSource := nil;
  end;
end;

procedure TRDBBolinhaBase.OnEstiloChanged(Sender: TObject);
begin
  Self.Invalidate;
end;

procedure TRDBBolinhaBase.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TRDBBolinhaBase.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;

  if Value <> nil then
    Value.RemoveFreeNotification(Self);  
end;

procedure TRDBBolinhaBase.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TRDBBolinhaBase.UpdateData(Sender: TObject);
begin

end;

procedure TRDBBolinhaBase.SetBrushFalso(const Value: TBrush);
begin
  FBrushVerdadeiro.Assign(Value);
end;

procedure TRDBBolinhaBase.SetBrushVerdadeiro(const Value: TBrush);
begin
  FBrushVerdadeiro.Assign(Value);
end;

procedure TRDBBolinhaBase.SetPenFalso(const Value: TPen);
begin
  FPenFalso.Assign(Value);
end;

procedure TRDBBolinhaBase.SetPenVerdadeiro(const Value: TPen);
begin
  FPenVerdadeiro.Assign(Value);
end;

function TRDBBolinhaBooleana.GetIsVerdadeiro: Boolean;
begin
  if FDataLink.Field <> nil then
    FIsVerdadeiro := FDataLink.Field.AsBoolean
  else
    FIsVerdadeiro := False;  

  Result := FIsVerdadeiro;
end;

procedure TRDBBolinhaBooleana.SetIsVerdadeiro(const Valor: Boolean);
begin
  if Valor <> FIsVerdadeiro then
  begin
    FIsVerdadeiro := Valor;
    FDataLink.Modified;
    FDataLink.UpdateRecord;

    Self.Invalidate;
  end;
end;

procedure TRDBBolinhaBooleana.Paint;
begin
  inherited;
  Self.DesenharFundo();

  if Self.GetIsVerdadeiro() then
  begin
    Self.Canvas.Brush.Assign(FBrushVerdadeiro);
    Self.Canvas.Pen.Assign(FPenVerdadeiro);
  end else
  begin
    Self.Canvas.Brush.Assign(FBrushFalso);
    Self.Canvas.Pen.Assign(FPenFalso);
  end;

  Self.Canvas.Ellipse(Self.ClientRect);
end;

procedure TRDBBolinhaBooleana.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  if Button = mbLeft then
    if FDataLink.Edit then
      Self.SetIsVerdadeiro(not Self.GetIsVerdadeiro);
end;

procedure TRDBBolinhaBooleana.DataChange(Sender: TObject);
begin
  inherited;
  Self.Invalidate;
end;

procedure TRDBBolinhaBooleana.UpdateData(Sender: TObject);
begin
  inherited;
  
  if FDataLink.Field = nil then
    Exit;

  FDataLink.Field.AsBoolean := FIsVerdadeiro;
end;

{ TRDBIntegerBolinha }

procedure TRDBBolinhaNumerica.CalcularAlturaLarguraBolinha(var Altura,
  Largura: Integer);
begin
  Largura := (Self.ClientWidth div FNumeroMaximo) - FEspacamento;
  Altura := Largura;

  if Altura > Self.ClientHeight then
  begin
    Altura := Self.ClientHeight;
    Largura := Altura;
  end;
end;

constructor TRDBBolinhaNumerica.Create(AOwner: TComponent);
begin
  inherited;
  FNumeroMaximo := 5;
  FEspacamento := 4;
  Self.Width := 90;
  Self.Height := 32;
end;

procedure TRDBBolinhaNumerica.DataChange(Sender: TObject);
begin
  inherited;

  if FDataLink.Field = nil then
    Exit;

  FValorAtual := FDataLink.Field.AsInteger;
  Self.Invalidate;
end;

procedure TRDBBolinhaNumerica.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  NovoValor, AlturaB, LarguraB: Integer;
begin
  inherited;

  if Button = mbLeft then
  begin
    Self.CalcularAlturaLarguraBolinha(AlturaB, LarguraB);

    if LarguraB + FEspacamento <= 0 then
      Exit;

    NovoValor := (X div (LarguraB + FEspacamento)) + 1;

    if (NovoValor = 1) and (FValorAtual = 1) then
      NovoValor := 0;

    if NovoValor > FNumeroMaximo then
      NovoValor := FNumeroMaximo;

    if NovoValor <> FValorAtual then
      if FDataLink.Edit then
      begin
        FValorAtual := NovoValor;
        FDataLink.Modified;
        FDataLink.UpdateRecord;
        Self.Invalidate;
      end;
  end;
end;

procedure TRDBBolinhaNumerica.Paint;
var
  X, Y, AlturaB, LarguraB: Integer;
  i: Integer;
  EstaVerdadeiro: Boolean;
begin
  inherited;
  Self.DesenharFundo();
  Self.CalcularAlturaLarguraBolinha(AlturaB, LarguraB);

  if LarguraB <= 0 then
    Exit;

  Y := Self.ClientHeight div 2 - AlturaB div 2;

  if FValorAtual > 0 then
  begin
    Self.Canvas.Brush.Assign(FBrushVerdadeiro);
    Self.Canvas.Pen.Assign(FPenVerdadeiro);
    EstaVerdadeiro := True;
  end else
  begin
    Self.Canvas.Brush.Assign(FBrushFalso);
    Self.Canvas.Pen.Assign(FPenFalso);
    EstaVerdadeiro := False;
  end;

  for i := 0 to FNumeroMaximo - 1 do
  begin
    X := i * (LarguraB + FEspacamento) + Espacamento div 2;

    if i + 1 = FValorAtual + 1 then
    begin
      // Mudar para falso
      
      if EstaVerdadeiro then
      begin
        Self.Canvas.Brush.Assign(FBrushFalso);
        Self.Canvas.Pen.Assign(FPenFalso);
        EstaVerdadeiro := False;
      end;  
    end;

    Self.Canvas.Ellipse(X, Y, X + LarguraB, Y + LarguraB);
  end;
end;

procedure TRDBBolinhaNumerica.SetEspacamento(const Value: Integer);
begin
  if FEspacamento <> Value then
  begin
    if Value < 0 then
      raise Exception.Create('O valor deve ser maior ou igual a zero');

    FEspacamento := Value;
    Self.Invalidate;
  end;
end;

procedure TRDBBolinhaNumerica.SetNumeroMaximo(const Value: Integer);
begin
  if FNumeroMaximo <> Value then
  begin
    if Value < 1 then
      raise Exception.Create(' preciso pelo menos 1 bolinha');

    if Value > 255 then
      raise Exception.Create('Nmero mximo de bolinhas: 255');

    FNumeroMaximo := Value;
    Self.Invalidate;
  end;
end;

procedure TRDBBolinhaNumerica.UpdateData(Sender: TObject);
begin
  inherited;
  FDataLink.Field.AsInteger := FValorAtual
end;

procedure TRDBBolinhaBase.SetBrushFundo(const Value: TBrush);
begin
  FBrushFundo.Assign(Value);
end;

procedure TRDBBolinhaBase.DesenharFundo;
begin
  if FBrushFundo.Style = bsClear then
    Exit;
    
  Self.Canvas.Brush.Assign(FBrushFundo);
  Self.Canvas.Pen.Style := psClear;
  Self.Canvas.Rectangle(Self.ClientRect);
end;

end.
