unit RRPGDBRichEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics,
  ComCtrls, Forms, StdCtrls, ExtCtrls, CommCtrl, RichEdit, RichOle,
  ShellApi, Menus, ActiveX, RRPGRichEdit, DB, DBCtrls, RRPGControls;

type
  {$if CompilerVersion < 23}
    NativeInt = Integer;
  {$ifend}

  TRDBRichEdit = class;

  TRichEditFieldDataLink = class(TFieldDataLink)
  private
    FRich: TRDBRichEdit;
  protected
    procedure AtualizarStatusLink;
    procedure DataEvent(Event: TDataEvent; Info: NativeInt); override;
    procedure EditingChanged; override;
  end;

  TRDBRichEdit = class(TRRichEdit)
  private
    FDataLink: TRichEditFieldDataLink;
    FAutoDisplay: Boolean;
    FFocused: Boolean;
    FMemoLoaded: Boolean;
    FDataSave: string;
    FCreatingWnd: integer;
    FIgnorarRestaurarCharPos: Boolean;
    FRestaurarCharPosRange: TCharRange;

    // Flags
    FInLoadMemo: Boolean;
    FMD5Conteudo: String; // Vazio significa desconhecido


    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: WideString;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure SetDataField(const Value: WideString);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetFocused(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure EMSetCharFormat(var Message: TMessage); message EM_SETCHARFORMAT;
    procedure EMSetParaFormat(var Message: TMessage); message EM_SETPARAFORMAT;
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMClear(var Message: TMessage); message WM_CLEAR;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    function GetInheritedReadOnly: Boolean;
    procedure SetInheritedReadOnly(const Value: Boolean);
  protected
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;

    procedure BeforeSalvarNoDataSet; virtual;
    procedure CreateWnd; override;
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;

    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SugestaoClick(Sender: TObject); override;


    property InheritedReadOnly: Boolean read GetInheritedReadOnly write SetInheritedReadOnly;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    procedure LoadMemo; virtual;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    function UseRightToLeftAlignment: Boolean; override;
    property Field: TField read GetField;


    function BeginEditing: Boolean;
  published
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property DataField: WideString read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
  end;

implementation

uses
  MD5;

{ TDBRichEdit }

constructor TRDBRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  FAutoDisplay := True;

  FDataLink := TRichEditFieldDataLink.Create;
  FDataLink.FRich := Self;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
end;

procedure TRDBRichEdit.CreateWnd;
begin
  Inc(FCreatingWnd);

  try
    inherited;
  finally
    Dec(FCreatingWnd);
  end;
end;

destructor TRDBRichEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TRDBRichEdit.Loaded;
begin
  inherited Loaded;

  if (csDesigning in ComponentState) then
    DataChange(Self);
end;

procedure TRDBRichEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);

  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource) then
    DataSource := nil;
end;

function TRDBRichEdit.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;

procedure TRDBRichEdit.BeforeSalvarNoDataSet;
begin

end;

function TRDBRichEdit.BeginEditing: Boolean;
begin
  Result := FDataLink.Editing;

  if not FDataLink.Editing then
    try
      if FDataLink.Field.IsBlob then
        FDataSave := FDataLink.Field.AsString;

      Result := FDataLink.Edit;
    finally
      FDataSave := '';
    end;

  if Result then
    FMD5Conteudo := '';
end;

procedure TRDBRichEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);

  if FMemoLoaded then
  begin
    if (Key = VK_DELETE) or (Key = VK_BACK) or (Key = VK_RETURN) or
      ((Key = VK_INSERT) and (ssShift in Shift)) or
      (((Key = Ord('V')) or (Key = Ord('X'))) and (ssCtrl in Shift)) then
    begin
      if not BeginEditing then
        Key := 0;
    end;
  end;
end;

procedure TRDBRichEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);

  if FMemoLoaded then
  begin
    if (Key >= #32) and (FDataLink.Field <> nil) and
      not FDataLink.Field.IsValidChar(Key) then
    begin
      MessageBeep(0);
      Key := #0;
    end;

    case Key of
      ^H, ^I, ^J, ^M, ^V, ^X, #32..High(Char):
      begin
        if not BeginEditing then
          Key := #0;
      end;

      #27: FDataLink.Reset;
    end;
  end else
  begin
    if Key = #13 then
      LoadMemo;

    Key := #0;
  end;
end;

procedure TRDBRichEdit.Change;
begin
  if FMemoLoaded then
    FDataLink.Modified;

  FMemoLoaded := True;
  inherited Change;
end;

function TRDBRichEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TRDBRichEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;

  if Value <> nil then
    Value.FreeNotification(Self);
end;

function TRDBRichEdit.GetDataField: WideString;
begin
  Result := FDataLink.FieldName;
end;

procedure TRDBRichEdit.SetDataField(const Value: WideString);
begin
  FDataLink.FieldName := Value;
end;

function TRDBRichEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TRDBRichEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TRDBRichEdit.SugestaoClick(Sender: TObject);
begin
  if Self.BeginEditing then
    inherited;
end;

function TRDBRichEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TRDBRichEdit.GetInheritedReadOnly: Boolean;
begin
  Result := inherited ReadOnly;
end;

procedure TRDBRichEdit.LoadMemo;
const
  RTFHeader = '{\rtf';  { Do not localize }
  URTFHeader = '{urtf'; { Do not localize }
var
  Data, MD5Novo: string;
  Stream: TStringStream;
  CharPos: TCharRange;

begin
  if FInLoadMemo then
    Exit;

  FInLoadMemo := True;

  try
    if not FMemoLoaded and Assigned(FDataLink.Field) and FDataLink.Field.IsBlob then
    begin
      SendMessage(Self.Handle, EM_EXGETSEL, 0, Integer(@CharPos));

      if not FIgnorarRestaurarCharPos then
        FRestaurarCharPosRange := CharPos;

      try
        Data := FDataLink.Field.AsString;
        FIgnorarRestaurarCharPos := Data = '';

        if (Pos(RTFHeader, Data) = 1) or (Pos(URTFHeader, Data) = 1) then
        begin
          Stream := TStringStream.Create(Data);

          try
            MD5Novo := MD5DigestToStr(MD5Stream(Stream));
            Stream.Position := 0;

            if FMD5Conteudo <> MD5Novo then
            begin
              Lines.LoadFromStream(Stream);
              FMD5Conteudo := MD5Novo;
            end;
          finally
            Stream.Free;
          end;
        end else
        begin
          Text := Data;
          FMD5Conteudo := '';
        end;

        FMemoLoaded := True;
      except
        { Rich Edit Load failure }
        on E: EOutOfResources do
          Lines.Text := Format('(%s)', [E.Message]);
      end;

      EditingChange(Self);

      if not FIgnorarRestaurarCharPos then
        SendMessage(Self.Handle, EM_EXSETSEL, 0, Integer(@FRestaurarCharPosRange));
    end;
  finally
    FInLoadMemo := False;
  end;
end;

procedure TRDBRichEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    if FDataLink.Field.IsBlob then
    begin
      if FAutoDisplay or (FDataLink.Editing and FMemoLoaded) then
      begin
        { Check if the data has changed since we read it the first time }
        if (FDataSave <> '') and (FDataSave = FDataLink.Field.AsString) then Exit;
        FMemoLoaded := False;
        LoadMemo;
      end else
      begin
        Text := Format('(%s)', [FDataLink.Field.DisplayLabel]);
        FMemoLoaded := False;
      end;
    end else
    begin
      if FFocused and FDataLink.CanModify then
        Text := FDataLink.Field.Text
      else
        Text := FDataLink.Field.DisplayText;
      FMemoLoaded := True;
    end
  else
  begin
    if csDesigning in ComponentState then Text := Name else Text := '';
    FMemoLoaded := False;
  end;

  if Self.HandleAllocated then
    RedrawWindow(Handle, nil, 0, RDW_INVALIDATE or RDW_ERASE or RDW_FRAME);
end;

procedure TRDBRichEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not (FDataLink.Editing and FMemoLoaded);
end;

procedure TRDBRichEdit.EMSetCharFormat(var Message: TMessage);
begin
  if FCreatingWnd = 0 then
    Self.BeginEditing;

  inherited;
end;

procedure TRDBRichEdit.EMSetParaFormat(var Message: TMessage);
begin
  Self.BeginEditing;
  inherited;
end;

procedure TRDBRichEdit.UpdateData(Sender: TObject);
var
  Stream: TStringStream;
begin
  Self.BeforeSalvarNoDataSet();

  if FDataLink.Field.IsBlob then
  begin
    Stream := TStringStream.Create('');

    try
      Lines.SaveToStream(Stream);
      FDataLink.Field.AsString := Stream.DataString;

      Stream.Position := 0;
      FMD5Conteudo := MD5DigestToStr(MD5Stream(Stream));
    finally
      Stream.Free;
    end;
  end else
    FDataLink.Field.AsString := Text;
end;

procedure TRDBRichEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;

    if not Assigned(FDataLink.Field) or not FDataLink.Field.IsBlob then
      FDataLink.Reset;
  end;
end;

procedure TRDBRichEdit.SetInheritedReadOnly(const Value: Boolean);
begin
  inherited ReadOnly := Value;
end;

procedure TRDBRichEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;

  {if FDataLink.CanModify then
    inherited ReadOnly := False;}
end;

procedure TRDBRichEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;

  SetFocused(False);
  inherited;
end;

procedure TRDBRichEdit.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then
  begin
    FAutoDisplay := Value;

    if Value then
      LoadMemo;
  end;
end;

procedure TRDBRichEdit.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  if not FMemoLoaded then
    LoadMemo
  else
    inherited;
end;

procedure TRDBRichEdit.WMCut(var Message: TMessage);
begin
  if BeginEditing then
    inherited;
end;

procedure TRDBRichEdit.WMPaste(var Message: TMessage);
begin
  if BeginEditing then
    inherited;
end;

procedure TRDBRichEdit.WMClear(var Message: TMessage);
begin
  if BeginEditing then
    inherited;
end;

procedure TRDBRichEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;


procedure TRDBRichEdit.CNNotify(var Message: TWMNotify);
begin
  inherited;

  {case Message.NMHdr.code of
    EN_REQUESTRESIZE: DoRequestResize();
  end;                                  }
end;

function TRDBRichEdit.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TRDBRichEdit.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TRichEditFieldDataLink }

procedure TRichEditFieldDataLink.AtualizarStatusLink;
//var
//  ReadOnlyDesejado: Boolean;
begin
  //if Self.DataSet <> nil then
  begin
    //ReadOnlyDesejado := not Self.DataSet.CanModify;
    //ReadOnlyDesejado := not Self.CanModify;

    //if FRich.InheritedReadOnly <> ReadOnlyDesejado then
    //  FRich.InheritedReadOnly := ReadOnlyDesejado;
  end;
end;

procedure TRichEditFieldDataLink.DataEvent(Event: TDataEvent; Info: NativeInt);
begin
  inherited;

  if Event = deUpdateState then
    Self.AtualizarStatusLink();
end;

procedure TRichEditFieldDataLink.EditingChanged;
begin
  inherited;
  Self.AtualizarStatusLink();
end;

end.
