unit RRPGDataSetUtil;

interface

uses
  DB, Variants, FMTBcd, SysUtils;

function FieldTypeToVariantType(FieldType: TFieldType): Integer;

function EscreverVariantParaBuffer(V: Variant; var Buffer; var Tamanho: Integer;
  Field: TField): Boolean;
procedure LerVariantDeBuffer(var V: Variant; const Buffer; const Tamanho: Integer;
  Field: TField);

function ConverterVariant(V: Variant; Para: Integer): Variant;

implementation

{*******************************************************************************
*
*   ConverterVariant
*
*   Sumrio: Converte um Variant de um tipo para outro
*
*   @param Para
*     O tipo de variant desejado.. Exemplos:
*     varInteger, varString, varSmallint etc..
*
*   Autor: ALC
*   Data: 06/09/09
*}

function ConverterVariant(V: Variant; Para: Integer): Variant;
begin
  if V <> Null then
  begin
    if VarType(V) = Para then
      Result := V
    else
      Result := VarAsType(V, Para);
  end else
    Result := Null;
end;

{*******************************************************************************
*
*   FieldTypeToVariantType
*
*   Sumrio: Converte um FieldType para um tipo de Variant
*
*   Autor: ALC
*   Data: 06/09/09
*}

function FieldTypeToVariantType(FieldType: TFieldType): Integer;
begin
  case FieldType of
    ftString, ftBlob: Result := varString;
    ftWideString: Result := varUString;
    ftInteger: Result := varInteger;
    ftSmallint: Result := varSmallint;
    ftWord: Result := varWord;
    ftBoolean: Result := varBoolean;
    ftFloat: Result := varDouble;
    ftCurrency: Result := varCurrency;
    ftBCD: Result := varCurrency;
    ftDate, ftTime, ftDateTime: Result := varDate;
    ftMemo: Result := varArray;
    ftLargeint: Result := varInt64;

    else
      Result := varUnknown;
  end;
end;

{*******************************************************************************
*
*   EscreverVariantParaBuffer
*
*   Sumrio:
*     Escreve o valor interno de um Variant para um buffer, de acordo com o
*     tipo do Variant
*
*   Autor: ALC
*   Data: 06/09/09
*}

function EscreverVariantParaBuffer(V: Variant; var Buffer; var Tamanho: Integer;
  Field: TField): Boolean;
var
  VData: TVarData;
  MaxTamBuf: Integer;

  procedure Mover(var Dados; Tamanho: Integer);
  var
    BufClear: PByte;
  begin
    Move(Dados, Buffer, Tamanho);

    BufClear := @Buffer;
    Inc(BufClear, Tamanho);
    FillChar(BufClear^, MaxTamBuf - Tamanho, 0);
  end;
begin
  Result := True;

  if VarType(V) in [varNull, varEmpty] then
    Result := False;

  MaxTamBuf := Tamanho;
  VData := TVarData(V);

  if @Buffer = nil then
    Exit;

  case VarType(V) of
    varSmallint: Smallint(Buffer) := VData.VSmallint;
    varInteger: Integer(Buffer) := VData.VInteger;
    varUString: Mover(VData.VString^, Length(PChar(VData.VString)) * SizeOf(Char));
    varString: Mover(VData.VString^, Length(PAnsiChar(VData.VString)) * SizeOf(AnsiChar));

    varDouble: Double(Buffer) := VData.VDouble;
    varCurrency: Currency(Buffer) := VData.VCurrency;
    varInt64: Int64(Buffer) := VData.VInt64;
    varDate: TDateTime(Buffer) := VData.VDate;
    varBoolean: WordBool(Buffer) := VData.VBoolean;

    else
      Result := False;
  end;
end;

{*******************************************************************************
*
*   LerVariantDeBuffer
*
*   Sumrio: Transforma um buffer para um Variant de acordo com o tipo de dados
*    do Field
*
*   Autor: ALC
*   Data: 06/09/09
*}

procedure LerVariantDeBuffer(var V: Variant; const Buffer; const Tamanho: Integer;
  Field: TField);

  procedure LerString;
  var
    X: PAnsiChar;
  begin
    X := @Buffer;
    V := AnsiString(X);
  end;

  procedure LerWideString;
  var
    X: PChar;
  begin
    X := @Buffer;
    V := String(X);
  end;
begin
  if @Buffer = nil then
  begin
    V := Null;
    Exit;
  end;

  case Field.DataType of
    ftSmallint: V := Smallint(Buffer);
    ftInteger: V := Integer(Buffer);
    ftString: LerString();
    ftWideString: LerWideString();

    ftFloat: V := Double(Buffer);
    ftCurrency, ftBCD: V := Currency(Buffer);

    ftLargeint: V := Int64(Buffer);

    ftWord: V := Word(Buffer);
    ftBoolean: V := WordBool(Buffer);
    ftDate, ftTime, ftDateTime: V := TDateTime(Buffer);
  else
    V := Null;
  end;
end;


end.
