unit RRPGDockSitePanel;

interface

uses
  ExtCtrls, RRPGDockTree, Controls, Windows;

type
  TRDockSitePanel = class(TPanel)
  protected
    function CreateDockManager: IDockManager; override;
    function GetDockEdge(MousePos: TPoint): TAlign; override;
    procedure DockOver(Source: TDragDockObject; X: Integer; Y: Integer;
      State: TDragState; var Accept: Boolean); override;
  end;

implementation

{ TRDockSitePanel }

function TRDockSitePanel.CreateDockManager: IDockManager;
begin
  if (Self.DockManager = nil) and DockSite and UseDockManager then
    Result := TRRPGDockTree.Create(Self)
  else
    Result := Self.DockManager;

  DoubleBuffered := DoubleBuffered or (Result <> nil);
end;

procedure TRDockSitePanel.DockOver(Source: TDragDockObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  inherited;

end;

function TRDockSitePanel.GetDockEdge(MousePos: TPoint): TAlign;

  function MinVar(const Data: array of Double): Integer;
  var
    I: Integer;
  begin
    Result := 0;
    for I := Low(Data) + 1 to High(Data) do
      if Data[I] < Data[Result] then Result := I;
  end;

var
  T, L, B, R: Integer;
begin
  Result := alNone;
  R := Width;
  B := Height;

  // if Point is outside control, then we can determine side quickly
  if MousePos.X <= 0 then
    Result := alLeft
  else
    if MousePos.X >= R then
      Result := alRight
    else
      if MousePos.Y <= 0 then
        Result := alTop
      else
        if MousePos.Y >= B then
          Result := alBottom
        else
        begin
          // if MousePos is inside the control, then we need to figure out which side
          // MousePos is closest to.
          T := MousePos.Y;
          B := B - MousePos.Y;

          L := MousePos.X;
          R := R - MousePos.X;

          case MinVar([L, R, T, B]) of
            0: Result := alLeft;
            1: Result := alRight;
            2: Result := alTop;
            3: Result := alBottom;
          end;
        end;
end;

end.
