unit RRPGDrawUtil;

interface

uses
  Windows, Messages, Graphics, RRPGColorScheme, UxTheme, Themes;

type
  TEstiloBotao = (ebNormal, ebRetangular);

  TRRPGDrawUtil = class(TObject)
  public
    class procedure DrawBkgButton(C: TCanvas; Area: TRect; Situacao: TSituacaoEsquema;
      const Focado: Boolean = False;
      const Default: Boolean = False;
      const EstiloBotao: TEstiloBotao = ebNormal);

    class procedure DrawHeaderItemBkg(C: TCanvas; Area: TRect; Situacao: TSituacaoEsquema);

    class procedure DrawFocusRect(C: TCanvas; Area: TRect; Situacao: TSituacaoEsquema);
    class procedure DrawHotRoundRect(C: TCanvas; Area: TRect);
  end;

  TInternoBufferPaint = record
    DestinoDC: HDC;
    BIT: HBITMAP;
    PB: HPAINTBUFFER;
    Area: TRect;
  end;

  TBufferPaint = record
    DC: HDC;
    Int: TInternoBufferPaint;
  end;

  TInternoBufferAnim = record
    AB: HANIMATIONBUFFER;
    DestinoDC: HDC;
    BIT: HBITMAP;
    Area: TRect;
    HWND: HWND;
  end;

  TBufferAnim = record
    DCDe: HDC;
    DCPara: HDC;
    Int: TInternoBufferAnim;
  end;

  TRRPGDrawBuffer = class(TObject)
  public
    class function PrecisaDesenhar(HWND: HWND; DC: HDC): Boolean;
    class function BeginBufferedPaint(DC: HDC; R: TRect): TBufferPaint;
    class procedure EndBufferedPaint(B: TBufferPaint; Atualizar: Boolean = True);

    class function BeginAnimation(HWND: HWND; DC: HDC; R: TRect; TempoMS: Cardinal): TBufferAnim;
    class procedure EndAnimation(Anim: TBufferAnim; Atualizar: Boolean = True);
  end;

  {$if compilerversion >= 23} // A partir de Delphi XE2
    TClasseThemeServices = TCustomStyleServices;
  {$else}
    TClasseThemeServices = TThemeServices;
  {$ifend}

function GetThemeServices: TClasseThemeServices; inline;
function IsThemeEnabled: Boolean; inline;

implementation

uses
  GDIPlus, GraphUtil, RRPGUtil;

{ TRRPGDrawUtil }

threadvar
  BeginBufferedInitCalled: Boolean;

function GetThemeServices: TClasseThemeServices;
begin
  {$if compilerversion >= 23} // A partir de Delphi XE2
    Result := StyleServices();
  {$else}
    Result := ThemeServices();
  {$ifend}
end;

function IsThemeEnabled: Boolean; inline;
begin
  {$if compilerversion >= 23} // A partir de Delphi XE2
    Result := GetThemeServices().Enabled;
  {$else}
    Result := GetThemeServices().ThemesEnabled;
  {$ifend}
end;

class procedure TRRPGDrawUtil.DrawBkgButton(C: TCanvas; Area: TRect;
  Situacao: TSituacaoEsquema; const Focado: Boolean = False;
                              const Default: Boolean = False;
                              const EstiloBotao: TEstiloBotao = ebNormal);
var
//  CH, CW: Integer;
  CH: Integer;
  Esq: PCoresSituacaoEsquema;
  R: TRect;
const
  BORDA_WIDTH = 1;
begin
  CH := Area.Bottom - Area.Top;
//  CW := Area.Right - Area.Left;
  Esq := @EsquemaAtual.Cores[Situacao];

  R := Area;
  R.Top := Area.Top;
  R.Bottom := Area.Top + CH div 2 + 1;

  GradientFillCanvas(C, Esq.Cor1, Esq.Cor1End, R, gdVertical);

  R.Top := Area.Top + CH div 2 + 1;
  R.Bottom := Area.Top + CH;
  GradientFillCanvas(C, Esq.Cor2, Esq.Cor2End, R, gdVertical);

  R := Area;
  C.Pen.Style := psSolid;

  if Default then
    C.Pen.Color := EsquemaAtual.Cores[sePressionado].CorBorda
  else
    C.Pen.Color := Esq.CorBorda;

  C.Pen.Width := BORDA_WIDTH;
  C.Brush.Style := bsClear;

  if EstiloBotao = ebNormal then
    C.RoundRect(R, 4, 4)
  else
    C.Rectangle(R);

  InflateRect(R, -BORDA_WIDTH, -BORDA_WIDTH);

  if Default then
    C.Pen.Color := EsquemaAtual.Cores[sePressionado].CorBorda2
  else
    C.Pen.Color := Esq.CorBorda2;

  if EstiloBotao = ebNormal then
    C.RoundRect(R, 4, 4)
  else
    C.Rectangle(R);

  if Focado then
  begin
    InflateRect(R, -BORDA_WIDTH, -BORDA_WIDTH);
    R.Right := R.Right - 1;
    R.Bottom := R.Bottom - 1;
    TRRPGDrawUtil.DrawFocusRect(C, R, Situacao);
  end;
end;

class procedure TRRPGDrawUtil.DrawFocusRect(C: TCanvas; Area: TRect; Situacao: TSituacaoEsquema);
var
  GPR: TGPRect;
  GPPen: IGPPen;
  Graphics: IGPGraphics;
  TempCor: TColor;
const
  ArSingle: array [0..1] of Single = (1.0, 1.0);
begin
  Graphics := TGPGraphics.Create(C.Handle);

  TempCor := EsquemaAtual.Cores[Situacao].CorTracejado;
  GPPen := TGPPen.Create(TGPColor.CreateFromColorRef(TempCor));
  GPPen.SetDashPattern(ArSingle);
  GPR.Initialize(Area);
  Graphics.DrawRectangle(GPPen, GPR);
end;

class procedure TRRPGDrawUtil.DrawHeaderItemBkg(C: TCanvas; Area: TRect;
  Situacao: TSituacaoEsquema);
var
//  CH, CW: Integer;
  Esq: PCoresSituacaoEsquema;
  R, TempRect: TRect;
  YProporcao: Integer;
const
  BORDA_WIDTH = 1;
  PROPORCAO = 0.40; // 40%
begin
//  CH := Area.Bottom - Area.Top;
//  CW := Area.Right - Area.Left;
  Esq := @EsquemaAtual.Cores[Situacao];

  TempRect := Area;

  R := TempRect;

  C.Pen.Style := psClear;
  C.Brush.Color := Esq.Header.Fundo1;
  C.Brush.Style := bsSolid;

  // 30% do fundo

  YProporcao := Round(TempRect.Top + (TempRect.Bottom - TempRect.Top) * PROPORCAO);
  R.Bottom := YProporcao + 1;
  C.Rectangle(R);

  // 70% do fundo

  R.Top := YProporcao;
  R.Bottom := TempRect.Bottom;
  GradientFillCanvas(C, Esq.Header.Fundo2Cor1, Esq.Header.Fundo2Cor2, R, gdVertical);

  // Bordas
  C.Pen.Style := psSolid;
  C.Pen.Width := 1;

  R := Area;
  // Bordas Externas

  C.Pen.Color := Esq.Header.Bordas[0].CorExterna;
  C.MoveTo(R.Left, R.Top);
  C.LineTo(R.Right, R.Top);

  C.Pen.Color := Esq.Header.Bordas[1].CorExterna;
  C.LineTo(R.Right, YProporcao);

  C.Pen.Color := Esq.Header.Bordas[2].CorExterna;
  C.LineTo(R.Right, R.Bottom);

  C.Pen.Color := Esq.Header.Bordas[3].CorExterna;
  C.LineTo(R.Left, R.Bottom);

  C.Pen.Color := Esq.Header.Bordas[2].CorExterna;
  C.LineTo(R.Left, YProporcao);

  C.Pen.Color := Esq.Header.Bordas[1].CorExterna;
  C.LineTo(R.Left, R.Top);

  // Bordas Internas
  InflateRect(R, -1, -1);

  C.Pen.Color := Esq.Header.Bordas[0].CorInterna;
  C.MoveTo(R.Left, R.Top);
  C.LineTo(R.Right, R.Top);

  C.Pen.Color := Esq.Header.Bordas[1].CorInterna;
  C.LineTo(R.Right, YProporcao);

  C.Pen.Color := Esq.Header.Bordas[2].CorInterna;
  C.LineTo(R.Right, R.Bottom);

  C.Pen.Color := Esq.Header.Bordas[3].CorInterna;
  C.LineTo(R.Left, R.Bottom);

  C.Pen.Color := Esq.Header.Bordas[2].CorInterna;
  C.LineTo(R.Left, YProporcao);

  C.Pen.Color := Esq.Header.Bordas[1].CorInterna;
  C.LineTo(R.Left, R.Top);
end;

class procedure TRRPGDrawUtil.DrawHotRoundRect(C: TCanvas; Area: TRect);
begin
  GraphUtil.GradientFillCanvas(C, EsquemaAtual.RoundSel.CorNucleo1,
                                  EsquemaAtual.RoundSel.CorNucleo2, Area, gdVertical);

  C.Brush.Style := bsClear;
  C.Pen.Style := psSolid;
  C.Pen.Color := EsquemaAtual.RoundSel.CorLinhaExterna;

  C.Rectangle(Area);
end;

{ TRRPGDrawBuffer }

class function TRRPGDrawBuffer.BeginAnimation(HWND: HWND; DC: HDC; R: TRect;
  TempoMS: Cardinal): TBufferAnim;
var
  AnimPar: BP_ANIMATIONPARAMS;
begin
  if IsMaiorIgualWinVista and IsAppThemed then
  begin
    if not BeginBufferedInitCalled then
      BufferedPaintInit();

    FillChar(AnimPar, SizeOf(AnimPar), 0);
    AnimPar.cbSize := SizeOf(AnimPar);
    AnimPar.style := BPAS_LINEAR;
    AnimPar.dwDuration := TempoMS;

    Result.Int.AB := UxTheme.BeginBufferedAnimation(HWND, DC, R, BPBF_COMPATIBLEBITMAP,
      nil, AnimPar, Result.DCDe, Result.DCPara);

    Result.Int.BIT := 0;
  end else
  begin
    Result.Int.BIT := CreateCompatibleBitmap(DC, R.Right - R.Left, R.Bottom - R.Top);
    Result.DCPara := CreateCompatibleDC(DC);
    Result.DCDe := 0;
    Result.Int.AB := 0;
    SelectObject(Result.DCPara, Result.Int.BIT);
    SetWindowOrgEx(Result.DCPara, R.Left, R.Top, nil);
  end;

  Result.Int.DestinoDC := DC;
  Result.Int.Area := R;
end;

class function TRRPGDrawBuffer.BeginBufferedPaint(DC: HDC;
  R: TRect): TBufferPaint;
begin
  if IsMaiorIgualWinVista and IsAppThemed then
  begin
    if not BeginBufferedInitCalled then
      BufferedPaintInit();

    Result.Int.PB := UxTheme.BeginBufferedPaint(DC, R, BPBF_COMPATIBLEBITMAP,
      nil, Result.DC);

    Result.Int.BIT := 0;
  end else
  begin
    Result.Int.BIT := CreateCompatibleBitmap(DC, R.Right - R.Left, R.Bottom - R.Top);
    Result.DC := CreateCompatibleDC(DC);
    Result.Int.PB := 0;
    SelectObject(Result.DC, Result.Int.BIT);
    SetWindowOrgEx(Result.DC, R.Left, R.Top, nil);
  end;

  Result.Int.DestinoDC := DC;
  Result.Int.Area := R;
end;

class procedure TRRPGDrawBuffer.EndAnimation(Anim: TBufferAnim;
  Atualizar: Boolean);
var
  R: TRect;
begin
  if IsMaiorIgualWinVista and IsAppThemed then
  begin
    //UxTheme.EndBufferedPaint(B.Int.PB, Atualizar);
    UxTheme.EndBufferedAnimation(Anim.Int.AB, Atualizar);
  end else
  begin
    if Atualizar then
    begin
      R := Anim.Int.Area;
      BitBlt(Anim.Int.DestinoDC, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top,
             Anim.DCPara, R.Left, R.Top, SRCCOPY);
    end;

    if Anim.DCPara <> 0 then
    begin
      DeleteDC(Anim.DCPara);
      Anim.DCPara := 0;
    end;

    if Anim.Int.BIT <> 0 then
    begin
      DeleteObject(Anim.Int.BIT);
      Anim.Int.BIT := 0;
    end;
  end;
end;

class procedure TRRPGDrawBuffer.EndBufferedPaint(B: TBufferPaint;
  Atualizar: Boolean);
var
  R: TRect;
begin
  if IsMaiorIgualWinVista and IsAppThemed then
  begin
    BufferedPaintMakeOpaque(B.Int.PB, nil);
    UxTheme.EndBufferedPaint(B.Int.PB, Atualizar);
  end else
  begin
    if Atualizar then
    begin
      R := B.Int.Area;
      BitBlt(B.Int.DestinoDC, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top,
             B.DC, R.Left, R.Top, SRCCOPY);
    end;

    if B.DC <> 0 then
    begin
      DeleteDC(B.DC);
      B.DC := 0;
    end;

    if B.Int.BIT <> 0 then
    begin
      DeleteObject(B.Int.BIT);
      B.Int.BIT := 0;
    end;
  end;
end;

class function TRRPGDrawBuffer.PrecisaDesenhar(HWND: HWND; DC: HDC): Boolean;
begin
  if IsMaiorIgualWinVista and IsAppThemed() then
  begin
    Result := not BufferedPaintRenderAnimation(HWND, DC);
  end else
    Result := True;
end;

end.
