unit RRPGListView;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics,
  ComCtrls, Forms, StdCtrls, ExtCtrls, CommCtrl, GDIPlus;  // DBCtrls

type
  TOnLVWCustomDrawItem = procedure (Sender: TObject; Item: TListItem; Rect: TRect;
                                    Status: Integer; // ver LVM_GETITEMSTATE
                                    C: TCanvas) of object;

  TRListView = class(TListView)
  private
    FMarcaDAgua: TPicture;
    FCanvasDraw: TCanvas;
    FTextoListaVazia: String;
    FFonteListaVazia: TFont;

    FHandleHeader: HWND;
    FAntigoHeaderWndProc: Pointer;
    FHeaderProcObjectInstance: Pointer;
    FIsHeaderMouseDown: Boolean;
    FHeaderIndexMouseDown: Integer;
    FOnLVWCustomDrawItem: TOnLVWCustomDrawItem;

    FGPGraphicsDraw: IGPGraphics;
    FHDCDrawItem: HDC;

    FBackgroundTransparente: Boolean;
    FBackgroundAlpha: Byte;
    FDoubleBufferedDump: Boolean;  // O RListView usa o estilo WS_EX_COMPOSITED, quando for DoubleBuffered. No podemos executar o cdigo original de TListView quando DoubleBuffered est true

    procedure OnMarcaDAguaChange(Sender: TObject);
    procedure SetMarcaDAgua(const Value: TPicture);
    procedure SetFonteListaVazia(const Value: TFont);
    procedure SetTextoListaVazia(const Value: String);
    function GetGPGraphicsDrawItem: IGPGraphics;
  protected
    procedure DoDrawGroup(C: TCanvas; Item: TListGroup; RDraw: TRect); virtual;
    function DoCustomDrawLvw(CDraw: PNMLVCustomDraw): Integer; virtual;
    function DoCustomDrawGroup(CDraw: PNMLVCustomDraw): Integer; virtual;
    procedure DesenharListItemsEm(C: TCanvas; RArea: TRect); virtual;
    procedure DesenharGroupsEm(C: TCanvas; RArea: TRect); virtual;    
    procedure DesenharHeaderEm(C: TCanvas); virtual;
    procedure LimparDC(DC: HDC);
    procedure WMHeaderPaint(var Message: TWMPaint);
    procedure WMHeaderPrintClient(var Message: TWMPrintClient);
    procedure WMHeaderLButtonDown(var Message: TWMLButtonDown);
    procedure HeaderWndProc(var Message: TMessage); virtual;

    procedure OnFonteListaVaziaChange(Sender: TObject); virtual;

    procedure CNNotify(var Message: TMessage); message CN_NOTIFY;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;

    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;

    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property GPGraphicsDrawItem: IGPGraphics read GetGPGraphicsDrawItem;
  published
    property MarcaDAgua: TPicture read FMarcaDAgua write SetMarcaDAgua;
    property TextoListaVazia: String read FTextoListaVazia write SetTextoListaVazia;
    property FonteListaVazia: TFont read FFonteListaVazia write SetFonteListaVazia;
    property OnLVWCustomDrawItem: TOnLVWCustomDrawItem read FOnLVWCustomDrawItem write FOnLVWCustomDrawItem;
    property BackgroundTransparente: Boolean read FBackgroundTransparente write FBackgroundTransparente default False;
    property BackgroundAlpha: Byte read FBackgroundAlpha write FBackgroundAlpha default 255;

    property DoubleBuffered: Boolean read FDoubleBufferedDump write FDoubleBufferedDump stored True;
  end;

implementation

uses
  RRPGDrawUtil, RRPGColorScheme, Math, Themes, UxTheme, GraphUtil, RRPGUtil;

{ TRListView }

procedure TRListView.CNNotify(var Message: TMessage);
var
  ProcessarMensagem: Boolean;
  NMHDR: PNMHdr;
  CDraw: PNMLVCustomDraw;
begin
  ProcessarMensagem := True;
  NMHDR := PNMHdr(Message.LParam);

  case NMHDR.code of
    NM_CUSTOMDRAW:
      begin
        ProcessarMensagem := False;
        Message.Result := CDRF_DODEFAULT;
        CDraw := PNMLVCustomDraw(Message.LParam);

       if CDraw.dwItemType = LVCDI_GROUP then
          Message.Result := Self.DoCustomDrawGroup(CDraw)
        else
          if Assigned(FOnLVWCustomDrawItem) then
          begin
            Message.Result := Self.DoCustomDrawLvw(CDraw);
          end;
      end;
  end;

  if ProcessarMensagem then
    inherited;
end;

procedure TRListView.ConfigurarVisual;
begin
  Self.Color := EsquemaAtual.CorWindow;
  Self.Font.Color := EsquemaAtual.CorTextoNormal;
  FFonteListaVazia.Color := EsquemaAtual.CorTextoInformacao;
end;

constructor TRListView.Create(AOwner: TComponent);
begin
  inherited;
  FBackgroundTransparente := False;
  FBackgroundAlpha := 255;
  FHeaderIndexMouseDown := -1;
  FMarcaDAgua := TPicture.Create;
  FMarcaDAgua.OnChange := Self.OnMarcaDAguaChange;
  FCanvasDraw := TCanvas.Create;;
  FFonteListaVazia := TFont.Create;

  FFonteListaVazia.Name := 'Calibri';
  FFonteListaVazia.Size := 12;

  FFonteListaVazia.OnChange := Self.OnFonteListaVaziaChange;

  RegistrarNotScheme(Self, Self.ConfigurarVisual);
end;

procedure TRListView.CreateParams(var Params: TCreateParams);
begin
  inherited;

//  if FDoubleBufferedDump then
//   Params.ExStyle := Params.ExStyle or WS_EX_COMPOSITED;
end;

procedure TRListView.CreateWnd;
var
  BK: TLVBKImageA;
  HB: HBITMAP;
begin
  inherited DoubleBuffered := False;

  inherited;

  if IsMaiorIgualWinVista then
  begin
    FillChar(BK, SizeOf(BK), 0);
    HB := CreateCompatibleBitmap(Self.Canvas.Handle, 1, 1);
    BK.ulFlags := LVBKIF_STYLE_NORMAL or LVBKIF_SOURCE_HBITMAP;
    BK.xOffsetPercent := 101;
    BK.yOffsetPercent := 101;
    BK.hbm := HB;

    if not ListView_SetBkImageA(Self.Handle, @BK) then
      RaiseLastOSError;
  end;

  if (Self.ViewStyle = vsReport) and (Self.ShowColumnHeaders) then
  begin
    FHandleHeader := ListView_GetHeader(Self.Handle);

    if FHandleHeader <> 0 then
    begin
      FAntigoHeaderWndProc := Pointer(GetWindowLong(FHandleHeader, GWL_WNDPROC));

      if FHeaderProcObjectInstance = nil then
        FHeaderProcObjectInstance := MakeObjectInstance(Self.HeaderWndProc);

      SetWindowLong(FHandleHeader, GWL_WNDPROC, Integer(FHeaderProcObjectInstance));
    end;
  end else
    FHandleHeader := 0;

{  Estilos := ListView_GetExtendedListViewStyle(Self.Handle);
  Estilos := Estilos or LVS_EX_DOUBLEBUFFER;
  ListView_SetExtendedListViewStyle(Self.Handle, Estilos);}
end;

procedure TRListView.DesenharGroupsEm(C: TCanvas; RArea: TRect);
begin
//  for i := 0 to Self.Groups.Count - 1 do     
//    if not (lgsHidden in Self.Groups[i].State) then    
//      Self.DoDrawGroup(C, Self.Groups[i]);
end;

procedure TRListView.DesenharHeaderEm(C: TCanvas);
var
  CR, R: TRect;
  Count: Integer;
  i: Integer;
  MostRight, Focado: Integer;
  MousePos: TPoint;
  Situacao: TSituacaoEsquema;
  HT: THDHitTestInfo;

  procedure DrawHeader(const Indice: Integer; R: TRect);
  var
    Str: String;
    Formato: TTextFormat;
    Col: TListColumn;
  begin
    TRRPGDrawUtil.DrawHeaderItemBkg(C, R, Situacao);

    if Indice > Self.Columns.Count - 1 then
      Exit;

    Col := Self.Columns[Indice];
    Str := Col.Caption;

    if Str = '' then
      Exit;

    C.Font.Assign(Self.Font);
    SetTextColor(C.Handle, EsquemaAtual.CorTextoNormal);
    SetBkMode(C.Handle, TRANSPARENT);

    InflateRect(R, -7, 0);

    Formato := [tfVerticalCenter, tfSingleLine, tfEndEllipsis];

    case Col.Alignment of
      taRightJustify: Formato := Formato + [tfRight];
      taCenter: Formato := Formato + [tfCenter];
    end;

    C.TextRect(R, Str, Formato);
  end;
begin
  Windows.GetClientRect(FHandleHeader, CR);

  Count := Header_GetItemCount(FHandleHeader);
  MostRight := 0;

  GetCursorPos(MousePos);
  GetWindowRect(FHandleHeader, R);
  MousePos.X := MousePos.X - R.Left;
  MousePos.Y := MousePos.Y - R.Top;

  HT.Point := MousePos;
  HT.Flags := HHT_ONHEADER;
  Focado := SendMessage(FHandleHeader, HDM_HITTEST, 0, Integer(@HT));
  Situacao := seNormal;

  for i := 0 to Count - 1 do
  begin
    Header_GetItemRect(FHandleHeader, i, @R);

    if R.Right > MostRight then
      MostRight := R.Right;

    if i = Focado then
      Continue;

    DrawHeader(i, R);
  end;

  if MostRight < CR.Right then
  begin
    R := CR;
    R.Left := MostRight;
    TRRPGDrawUtil.DrawHeaderItemBkg(C, R, seNormal);
  end;

  if Focado > -1 then
  begin
    if IsThemeEnabled() then
    begin
      if (FIsHeaderMouseDown) and (Focado = FHeaderIndexMouseDown) then
        Situacao := sePressionado
      else
        Situacao := seHot;
    end else
      Situacao := seNormal;

    Header_GetItemRect(FHandleHeader, Focado, @R);
    DrawHeader(Focado, R);
  end;
end;

procedure TRListView.DesenharListItemsEm(C: TCanvas; RArea: TRect);
var
  i: Integer;
  Item: TListItem;
  R, CR: TRect;
  Status: Integer;
  Primeiro: Boolean;

  function RVisivel: Boolean;
  begin
    Result := RectVisible(C.Handle, R);
  end;
begin
  if not Assigned(FOnLVWCustomDrawItem) then
    Exit;

  try
    FHDCDrawItem := C.Handle;

    try
      Primeiro := True;

      CR := Self.ClientRect;

      for i := 0 to Self.Items.Count - 1 do
      begin
        Item := Self.Items[i];

        R := Item.DisplayRect(drBounds);

        if RVisivel() then
        begin
          if Primeiro then
          begin
            C.Font.Assign(Self.Font);
            Primeiro := False;
          end;

          Status := ListView_GetItemState(Self.Handle, i, LVIS_FOCUSED or LVIS_SELECTED);
          FOnLVWCustomDrawItem(Self, Item, R, Status, C);
        end;
      end;
    finally
      FHDCDrawItem := 0;
    end;
  finally
    FGPGraphicsDraw := nil;
  end;
end;

destructor TRListView.Destroy;
begin
  DesregistrarNotScheme(Self);
  FFonteListaVazia.Free;
  FCanvasDraw.Free;
  FMarcaDAgua.Free;
  inherited;

  if FHeaderProcObjectInstance <> nil then
  begin
    FreeObjectInstance(FHeaderProcObjectInstance);
    FHeaderProcObjectInstance := nil;
  end;
end;

procedure TRListView.DestroyWnd;
begin
  if FAntigoHeaderWndProc <> nil then
    if FHandleHeader <> 0 then
      SetWindowLong(FHandleHeader, GWL_WNDPROC, Integer(FAntigoHeaderWndProc));

  FHandleHeader := 0;
  FAntigoHeaderWndProc := nil;
  inherited;
end;

function TRListView.DoCustomDrawGroup(CDraw: PNMLVCustomDraw): Integer;
var
  Retorno: Integer;

  procedure Desenhar;
  var
    C: TCanvas;
    EstagioItem: Integer;

    procedure DesenharGrupo;
    var
      i: Integer;
      Item: TListGroup;
    begin
      Item := nil;

      for i := 0 to Self.Groups.Count - 1 do
        if DWORD_PTR(Self.Groups[i].GroupID) = CDraw.nmcd.dwItemSpec then
        begin
          Item := Self.Groups[i];
          Break;
        end;
             
      if Item = nil then
        Exit;

      Self.DoDrawGroup(C, Item, CDraw.rcText);
    end;

  begin
    EstagioItem := CDraw.nmcd.dwDrawStage and $F;

    if EstagioItem = CDDS_POSTPAINT then
    begin
      C := FCanvasDraw;
      C.Handle := CDraw.nmcd.hdc;

      try
        DesenharGrupo();
      finally
        C.Handle := 0;
      end;

      Retorno := Retorno or CDRF_NOTIFYPOSTERASE;
    end;

    if EstagioItem = CDDS_PREPAINT then
    begin
      Retorno := Retorno or CDRF_NOTIFYPOSTPAINT;
    end;
  end;
begin
  Retorno := CDRF_SKIPDEFAULT;
  Desenhar();
  Result := Retorno;
end;

function TRListView.DoCustomDrawLvw(CDraw: PNMLVCustomDraw): Integer;
var
  Retorno: Integer;

  procedure Desenhar;
  var
    C: TCanvas;
    EstagioItem: Integer;

    procedure DesenharItem;
    var
      R: TRect;
      Item: TListItem;
      Status: Integer;
    begin
      if CDraw.nmcd.dwItemSpec > DWORD_PTR(Self.Items.Count - 1) then
        Exit;

      Item := Self.Items[CDraw.nmcd.dwItemSpec];

      if Assigned(FOnLVWCustomDrawItem) then
      begin
        R := Item.DisplayRect(drBounds);
        Status := ListView_GetItemState(Self.Handle, Item.Index, LVIS_FOCUSED or LVIS_SELECTED);

        C.Font.Assign(Self.Font);
        FOnLVWCustomDrawItem(Self, Item, R, Status, C);
      end;
    end;

  begin
    if CDraw.nmcd.dwDrawStage and CDDS_ITEM <> 0 then
    begin
      EstagioItem := CDraw.nmcd.dwDrawStage and $F;

      if EstagioItem = CDDS_POSTPAINT then
      begin
        C := FCanvasDraw;
        C.Handle := CDraw.nmcd.hdc;

        try
          DesenharItem();
        finally
          C.Handle := 0;
        end;

        Retorno := Retorno or CDRF_NOTIFYPOSTERASE;
      end;

      if EstagioItem = CDDS_PREPAINT then
      begin
        Retorno := Retorno or CDRF_NOTIFYPOSTPAINT;
      end;
    end
    else
      Retorno := Retorno or CDRF_NOTIFYITEMDRAW;
  end;

begin
  Retorno := CDRF_DODEFAULT;
  Desenhar();
  Result := Retorno;
end;

procedure TRListView.DoDrawGroup(C: TCanvas; Item: TListGroup; RDraw: TRect);
var
  RTextoOriginal, RTexto, RLinha, RChevron, RTexto2: TRect;
  Txt: String;
  TextFormats: TTextFormat;
  ChevronSize: Integer;
const
  MARGEM = 10;
  MARGEM_TEXTO_LINHA = 5;
begin
  RTexto := RDraw;       
  RTextoOriginal := RTexto;

  if not IsMaiorIgualWinVista then
    OffsetRect(RTexto, 0, 6);

  Txt := Item.Header;

  InflateRect(RTexto, -MARGEM, 0);
  RLinha := RTexto;

  C.Brush.Style := bsClear;
  C.Font.Color := EsquemaAtual.CorTextoTitulo;
  C.Font.Name := 'Tahoma';
  C.Font.Size := 10;
  C.Font.Style := [];

  TextFormats := [tfSingleLine, tfVerticalCenter];

  RTexto2 := RTexto;
  C.TextRect(RTexto2, Txt, TextFormats + [tfCalcRect]);

  case Item.HeaderAlign of
    taCenter: TextFormats := TextFormats + [tfCenter];
    taRightJustify: TextFormats := TextFormats + [tfRight];
  end;

  RTexto.Top := RTexto2.Top;
  RTexto.Bottom := RTexto2.Bottom;

  if Item.HeaderAlign = taCenter then
    RTexto.Left := RTexto.Left + (RTexto.Right - RTexto.Left) div 2 -
                   (RTexto2.Right - RTexto2.Left) div 2
  else
    RTexto.Left := RTexto2.Left;

  RTexto.Right := RTexto.Left + (RTexto2.Right - RTexto2.Left);

  TextFormats := TextFormats - [tfVerticalCenter];
  C.TextRect(RTexto, Txt, TextFormats);

  RLinha.Left := RTexto.Right + MARGEM_TEXTO_LINHA;
  RLinha.Top := RTexto.Top + (RTexto.Bottom - RTexto.Top) div 2;

  C.Pen.Style := psSolid;
  C.Pen.Width := 1;
  C.Pen.Color := EsquemaAtual.CorLinhaTituloLVW;

  if IsMaiorIgualWinVista then
    if lgsCollapsible in Item.State then
    begin
      RChevron := RLinha;

      if IsThemeEnabled() then
      begin
        RChevron.Left := RChevron.Right - 8;
        ChevronSize := 4;
      end else
      begin
        RChevron.Left := RChevron.Right - 5;
        ChevronSize := 4;
      end;

      if lgsCollapsed in Item.State then
        DrawChevron(C, sdDown, Point(RChevron.Left, RChevron.Top + 0), ChevronSize)
      else
        DrawChevron(C, sdUp, Point(RChevron.Left, RChevron.Top + 0), ChevronSize);

      RLinha.Right := RLinha.Right - 8 - MARGEM_TEXTO_LINHA;
    end;

  C.MoveTo(RLinha.Left, RLinha.Top);
  C.LineTo(RLinha.Right, RLinha.Top);

  if Item.HeaderAlign = taCenter then
  begin
    C.MoveTo(RTextoOriginal.Left + MARGEM_TEXTO_LINHA + 8, RLinha.Top);
    C.LineTo(RTexto.Left - MARGEM_TEXTO_LINHA, RLinha.Top);
  end;
end;

function TRListView.GetGPGraphicsDrawItem: IGPGraphics;
begin
  Result := nil;

  if FHDCDrawItem = 0 then
    Exit;

  if FGPGraphicsDraw = nil then
    FGPGraphicsDraw := TGPGraphics.Create(FHDCDrawItem);

  Result := FGPGraphicsDraw ;
end;

procedure TRListView.HeaderWndProc(var Message: TMessage);
begin
  case Message.Msg of
    WM_PAINT:
    begin
      Self.WMHeaderPaint(TWMPaint(Message));
      Exit;
    end;

    WM_PRINTCLIENT:
    begin
      Self.WMHeaderPrintClient(TWMPrintClient(Message));
      Exit;
    end;

    WM_LBUTTONDOWN:
    begin
      Self.WMHeaderLButtonDown(TWMLButtonDown(Message));
    end;

    WM_LBUTTONUP:
    begin
      FIsHeaderMouseDown := False;
    end;
  end;

  if FAntigoHeaderWndProc <> nil then
    Message.Result := CallWindowProc(FAntigoHeaderWndProc, FHandleHeader,
                                     Message.Msg, Message.WParam, Message.LParam);
end;

procedure TRListView.LimparDC(DC: HDC);
var
  C: TCanvas;
  X, Y: Integer;
  R: TRect;
begin
//  if ((FTextoListaVazia <> '') and (Self.Items.Count = 0)) or
//     ((FMarcaDAgua.Graphic <> nil) and IsMaiorIgualWinVista) then
  begin
    C := FCanvasDraw;
    C.Handle := DC;

    try
      C.Brush.Color := Self.Color;
      C.Brush.Style := bsSolid;
      C.Pen.Color := Self.Color;
      C.Pen.Style := psSolid;
      C.Rectangle(Self.ClientRect);

      if (FMarcaDAgua.Graphic <> nil) and IsMaiorIgualWinVista then
      begin
        X := Self.ClientWidth - FMarcaDAgua.Width;
        Y := Self.ClientHeight - FMarcaDAgua.Height;

        C.Draw(X, Y, FMarcaDAgua.Graphic);
      end;

      if (FTextoListaVazia <> '') and (Self.Items.Count = 0) then
      begin
        X := Round(Self.ClientWidth * 0.75);
        R.Left := 0;
        R.Top := 0;
        R.Right := X;
        R.Bottom := 0;

        C.Font.Assign(FFonteListaVazia);
        C.Brush.Style := bsClear;
        C.TextRect(R, FTextoListaVazia, [tfCalcRect, tfCenter, tfWordBreak]);

        X := Self.ClientWidth div 2 - (R.Right - R.Left) div 2;
        Y := Self.ClientHeight div 2 - (R.Bottom - R.Top) div 2;

        OffsetRect(R, X, Y);

        SetTextColor(C.Handle, ColorToRGB(FFonteListaVazia.Color));
        DrawText(C.Handle, PChar(FTextoListaVazia), Length(FTextoListaVazia), R, DT_CENTER or DT_WORDBREAK);
      end;
    finally
      C.Handle := 0;
    end;
  end;
end;

procedure TRListView.Loaded;
begin
  inherited;
  inherited DoubleBuffered := False;
  FDoubleBufferedDump := True;
  Self.ConfigurarVisual();
end;

procedure TRListView.OnFonteListaVaziaChange(Sender: TObject);
begin
  if (FTextoListaVazia <> '') and (Self.Items.Count = 0) then
    Self.Invalidate;
end;

procedure TRListView.OnMarcaDAguaChange(Sender: TObject);
begin
  Self.Invalidate;
end;

procedure TRListView.SetFonteListaVazia(const Value: TFont);
begin
  FFonteListaVazia.Assign(Value);
end;

procedure TRListView.SetMarcaDAgua(const Value: TPicture);
begin
  FMarcaDAgua.Assign(Value);
end;

procedure TRListView.SetTextoListaVazia(const Value: String);
begin
  if FTextoListaVazia <> Value then
  begin
    FTextoListaVazia := Value;

    if Self.Items.Count = 0 then
      Self.Invalidate;
  end;
end;

procedure TRListView.WMEraseBkgnd(var Message: TWMEraseBkgnd);
var
  T: TClasseThemeServices;
  R: TRect;
  B, B2: TBufferPaint;
  BF: TBlendFunction;
begin
  Message.Result := 1;
  Exit;

  T := GetThemeServices();

  if (FBackgroundTransparente) and (FBackgroundAlpha < 255) and (IsThemeEnabled) then
  begin
    R := Self.ClientRect;
    B := TRRPGDrawBuffer.BeginBufferedPaint(Message.DC, R);

    try
      T.DrawParentBackground(Self.Handle, B.DC, nil, False, R);

      B2 := TRRPGDrawBuffer.BeginBufferedPaint(B.DC, R);

      try

        FCanvasDraw.Handle := B2.DC;

        try
          FCanvasDraw.Brush.Style := bsSolid;
          FCanvasDraw.Pen.Style := psSolid;
          FCanvasDraw.Brush.Color := Self.Color;
          FCanvasDRaw.Pen.Color := Self.Color;
          FCanvasDraw.Rectangle(R);
          Self.LimparDC(B.DC);

          FillChar(BF, SizeOf(BF), 0);
          BF.BlendOp := AC_SRC_OVER;
          BF.SourceConstantAlpha := FBackgroundAlpha;

          if not AlphaBlend(B.DC, 0, 0, Self.ClientWidth, Self.ClientHeight,
                            B2.DC, 0, 0, Self.ClientWidth, Self.ClientHeight, BF)
            then
            RaiseLastOSError;
        finally
          FCanvasDraw.Handle := 0;
        end;
      finally
        TRRPGDrawBuffer.EndBufferedPaint(B2, False);
      end;
    finally
      TRRPGDrawBuffer.EndBufferedPaint(B, True);
    end;
  end else
  begin
//    inherited;   
    Self.LimparDC(Message.DC);
    Message.Result := 1;    
  end;
end;

procedure TRListView.WMHeaderLButtonDown(var Message: TWMLButtonDown);
var
  HD: THDHitTestInfo;
begin
  FIsHeaderMouseDown := True;
  FHeaderIndexMouseDown := -1;

  HD.Point.X := Message.XPos;
  HD.Point.Y := Message.YPos;
  HD.Flags := HHT_ONHEADER or HHT_ONDIVIDER;

  FHeaderIndexMouseDown := SendMessage(FHandleHeader, HDM_HITTEST, 0, Integer(@HD));
end;

procedure TRListView.WMHeaderPaint(var Message: TWMPaint);
var
  DC: HDC;
  PT: TPaintStruct;
begin
  DC := Message.DC;

  if DC = 0 then
    DC := BeginPaint(FHandleHeader, PT);

  try
    FCanvasDraw.Handle := DC;

    try
      Self.DesenharHeaderEm(FCanvasDraw);
    finally
      FCanvasDraw.Handle := 0;
    end;

    Message.Result := 1;
  finally
    if Message.DC = 0 then
      EndPaint(FHandleHeader, PT);
  end;
end;

procedure TRListView.WMHeaderPrintClient(var Message: TWMPrintClient);
begin
  FCanvasDraw.Handle := Message.DC;

  try
    Self.DesenharHeaderEm(FCanvasDraw);
    Message.Result := 1;
  finally
    FCanvasDraw.Handle := 0;
  end;
end;

procedure TRListView.WMPaint(var Msg: TWMPaint);
var
  PT: TPaintStruct;
  DC: HDC;
  B: TBufferPaint;
  MM: TWMPaint;
begin
//  DefaultHandler(Msg);
//  Exit;

  DC := BeginPaint(Self.Handle, PT);

  try
    if (PT.rcPaint.Left = 0) and (PT.rcPaint.Top = 0) and 
       (PT.rcPaint.Right = 0) and (PT.rcPaint.Bottom = 0) then
      PT.rcPaint := Self.ClientRect;
    
    B := TRRPGDrawBuffer.BeginBufferedPaint(DC, PT.rcPaint);

    try
      if B.DC = 0 then
        Exit;

      Self.LimparDC(B.DC);
                                                            
      MM.Msg := WM_PAINT;
      MM.DC := B.DC;
      MM.Unused := 0;
      Self.DefaultHandler(MM);
    finally
      TRRPGDrawBuffer.EndBufferedPaint(B);
    end;
  finally
    EndPaint(Self.Handle, PT);
  end;
end;

end.
