{*******************************************************************************}
{*                                                                             *}
{*                    RRPGModuloDLL.pas                                        *}
{*                                                                             *}
{*    As DLLs modulos do RRPG feitas em Delphi devem utilizar esta unit        *}
{*                                                                             *}
{*    Esta unit contm algumas coisas prontas para fazer um Mdulo RRPG        *}
{*                                                                             *}
{*******************************************************************************}

unit RRPGModuloDLL;

interface

uses
  Windows, RRPGModulo, Classes, SysUtils, RRPGModuloSom;

type
  TMetodoNotificarModulo = procedure (Ativar: Boolean) of object;

  TDelphiRRPGModulo = class(TInterfacedObject, IRRPGModulo)
  private
    FAtivo: Boolean;
    FCentral: IRRPGCentral;
    FMetodosNotify: array of TMetodoNotificarModulo;
  protected
    procedure InterfaceNaoSuportada; virtual;

    // Interface IRRPGModulo
    procedure SetRRPGCentral(const RRPGCentral: IRRPGCentral); stdcall;
    procedure Ativar; stdcall;
    procedure Desativar; stdcall;
  public
    destructor Destroy; override;

    procedure QueryUtilitarios(var Saida: IRRPGCentralUtilitarios);
    procedure QueryCentralSons(var Saida: IRRPGCentralSons);
    procedure QueryRegistry(var Saida: IRRPGRegistry; const SubKey: String = '');

    procedure RegistrarMetodoNotify(Metodo: TMetodoNotificarModulo);
    procedure DesregistrarMetodoNotify(Metodo: TMetodoNotificarModulo);

    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;

    property Central: IRRPGCentral read FCentral;
  end;

procedure EntryPointRRPGModulo(out Modulo: IRRPGModulo); stdcall;

exports
  EntryPointRRPGModulo name 'EntryPointRRPGModulo';

var
  DelphiRRPGModulo: TDelphiRRPGModulo;

implementation

{*******************************************************************************
*
*
*
*
*}

{*******************************************************************************
*
*   EntryPointRRPGModulo
*
*   Este  o metodo que deve ser exportado pela DLL para que o RRPG carregue
*   a DLL como mdulo
*}

procedure EntryPointRRPGModulo(out Modulo: IRRPGModulo); stdcall;
begin
  Modulo := DelphiRRPGModulo;
end;

{ TDelphiRRPGModulo }

procedure TDelphiRRPGModulo.Ativar;
var
  i: Integer;
begin
  if not FAtivo then
  begin
    FAtivo := True;

    for i := 0 to Length(FMetodosNotify) - 1 do
      FMetodosNotify[i](True);
  end;
end;

procedure TDelphiRRPGModulo.Desativar;
var
  i: Integer;
begin
  if FAtivo then
  begin
    FAtivo := False;

    for i := 0 to Length(FMetodosNotify) - 1 do
      FMetodosNotify[i](False);
  end;
end;

procedure TDelphiRRPGModulo.DesregistrarMetodoNotify(Metodo: TMetodoNotificarModulo);
var
  i, IndRemover: Integer;
begin
  IndRemover := -1;

  for i := Length(FMetodosNotify) - 1 downto 0 do
    if (TMethod(FMetodosNotify[i]).Code = TMethod(Metodo).Code) and
       (TMethod(FMetodosNotify[i]).Data = TMethod(Metodo).Data) then
    begin
      IndRemover := i;
      Break;
    end;

  if IndRemover <> -1 then
  begin
    for i := IndRemover to Length(FMetodosNotify) - 2 do
      FMetodosNotify[i] := FMetodosNotify[i + 1];

    SetLength(FMetodosNotify, Length(FMetodosNotify) - 1);
  end;

  if FAtivo then
    Metodo(False);
end;

destructor TDelphiRRPGModulo.Destroy;
var
  i: Integer;
begin
  if DelphiRRPGModulo = Self then
    DelphiRRPGModulo := nil;

  if FAtivo then
    for i := Length(FMetodosNotify) - 1  downto 0 do
      FMetodosNotify[i](False);

  SetLength(FMetodosNotify, 0);      
  inherited;
end;

procedure TDelphiRRPGModulo.InterfaceNaoSuportada;
begin
  raise Exception.Create('Interface no suportada, verifique a verso do seu RRPG');
end;

procedure TDelphiRRPGModulo.QueryCentralSons(var Saida: IRRPGCentralSons);
begin
  Saida := nil;

  if not Succeeded(FCentral.QueryInterface(IRRPGCentralSons, Saida)) then
    Self.InterfaceNaoSuportada();
end;

procedure TDelphiRRPGModulo.QueryRegistry(var Saida: IRRPGRegistry; const SubKey: String = '');
var
  Util: IRRPGCentralUtilitarios;
  Path: WideString;
begin
  Self.QueryUtilitarios(Util);
  Path := Util.ObterCaminhoRegistroRRPG();

  if SubKey <> '' then
    Path := IncludeTrailingPathDelimiter(Path) + SubKey;

  if not Util.AbrirRegistro(PWideChar(Path), Saida, prReadWrite) then
    raise Exception.Create('O RRPG no criou a interface do Registro =/');
end;

procedure TDelphiRRPGModulo.QueryUtilitarios(var Saida: IRRPGCentralUtilitarios);
begin
  Saida := nil;

  if not Succeeded(FCentral.QueryInterface(IRRPGCentralUtilitarios, Saida)) then
    Self.InterfaceNaoSuportada();
end;

procedure TDelphiRRPGModulo.RegistrarMetodoNotify(Metodo: TMetodoNotificarModulo);
var
  N: Integer;
begin
  N := Length(FMetodosNotify);
  SetLength(FMetodosNotify, N + 1);
  FMetodosNotify[N] := Metodo;

  if FAtivo then
    Metodo(True);
end;

procedure TDelphiRRPGModulo.SetRRPGCentral(const RRPGCentral: IRRPGCentral);
begin
  FCentral := RRPGCentral;
end;

function TDelphiRRPGModulo._AddRef: Integer;
begin
  Result := -1;
end;

function TDelphiRRPGModulo._Release: Integer;
begin
  Result := -1;
end;

initialization
  DelphiRRPGModulo := TDelphiRRPGModulo.Create;
  DelphiRRPGModulo._AddRef;

finalization
  if DelphiRRPGModulo <> nil then
  begin
    DelphiRRPGModulo._Release;

    if DelphiRRPGModulo <> nil then
      DelphiRRPGModulo.Free;
  end;
end.
