unit RRPGModuloMesas;

interface

uses
  Windows, RRPGModulo;

type
  {*******************************************************************************
  *
  *   IRRPGBotao
  *
  *   Implemente esta interface para disponibilizar uma opcao via interface para
  *   o usuario RRPG
  *}

  IRRPGBotao = interface
    ['{8AE95A0B-B2CD-479C-AF94-52DBEE417540}']

    {GetCategoria
      Retorne um String, uma Categoria para seu boto}
    function GetCategoria: PAnsiChar; stdcall;

    {GetIcone
      Retorne um Handle de um Bitmap que representa o icone do boto. Retorne 0
      caso no tenha icone}
    function GetIcone: HBITMAP; stdcall;

    {GetDescricao
      Retorne um String contendo a Descrio Do Boto}
    function GetDescricao: PAnsiChar; stdcall;

    {OnClick
      O RRPG ir chamar este mtodo sempre que o usurio clicar no boto}
    procedure OnClick; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGMesa
  *
  *   Interface disponibilizada pelo RRPG. Representa uma mesa aberta no RRPG
  *}

  IRRPGMesa = interface
    ['{3A392680-69D6-4979-B754-66AF17BD974A}']

    {GetIDMesa
      Retorna o IDMesa desta mesa.

      IDMesa  um nmero identificador de mesas}
    function GetIDMesa: Integer; stdcall;

    {AdicionarBotaoUtilitario e RemoverBotaoUtilitario
      Adiciona e Remove um boto utilitrio da interface do RRPG}
    procedure AdicionarBotaoUtilitario(const Botao: IRRPGBotao); stdcall;
    procedure RemoverBotaoUtilitario(const Botao: IRRPGBotao); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGMesaAttachment
  *
  *   Implemente esta interface. Para cada mesa aberta no RRPG, existir uma
  *   instncia de IRRPGMesaAttachment para cada IRRPGPluginMesa
  *}

  IRRPGMesaAttachment = interface
    ['{5B0C3161-FE42-49B8-8196-FB4A80181DAF}']

    {SetMesa
      Chamado para te notificar qual  a mesa deste MesaAttachment}
    procedure SetMesa(const Mesa: IRRPGMesa); stdcall;

    {Inicializar
      O RRPG Deseja que voc execute o cdigo de inicializao}
    procedure Inicializar; stdcall;

    {Finalizar
      O RRPG deseja que voc desfaa tudo o que foi feito na funo "Inicializar"}
    procedure Finalizar; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGPluginMesa
  *
  *   Implemente esta interface para registrar um plugin de mesa.
  *   Para cada mesa que for entrada, ser chamado o mtodo CriarMesaAttachment
  *}

  IRRPGPluginMesa = interface
    ['{E64B73C1-51B2-40DB-8252-545841672DED}']

    {CriarMesaAttachment
      Chamada sempre que o usurio entra em uma mesa no RRPG.

      Retorne uma nova instncia de MesaAttachment. Retorne True caso consiga criar}
    function CriarMesaAttachment(out Attachment: IRRPGMesaAttachment): Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGCentralMesas
  *
  *   Obtenha esta interface  partir da interface IRRPGCentral
  *
  *   Esta interface  usada para trabalhar com mesas
  *}

  IRRPGCentralMesas = interface
    ['{142CE5F3-BDC5-4769-80D8-F87886A853DD}']

    {RegistrarPluginMesa e DesregistrarPluginMesa
      Registra e Desregistra um Plugin de Mesa}
    procedure RegistrarPluginMesa(const Plugin: IRRPGPluginMesa); stdcall;
    procedure DesregistrarPluginMesa(const Plugin: IRRPGPluginMesa); stdcall;

    {GetCountMesa
      Retorna o nmero de mesas abertas no RRPG}
    function GetCountMesa: Integer; stdcall;

    {GetMesa
      Retorna a mesa aberta cujo ndice seja igual ao parmetro Indice.

      Retorna TRUE caso tenha preenchido o parmetro "Mesa"}
    function GetMesa(const Indice: Integer; out Mesa: IRRPGMesa): Boolean; stdcall;

    {FindMesa
      Tenta localizar uma mesa cujo ID seja igual ao parmetro IDMesa..
      Retorna True se for localizado}
    function FindMesa(const IDMesa: Integer; out Mesa: IRRPGMesa): Boolean; stdcall;
  end;

implementation

end.
