unit RRPGPicasa;

interface

uses
  Windows, WinInet, Classes, SysUtils, Generics.Collections, Messages,
  xmldom, XMLIntf, msxmldom, XMLDoc, ActiveX, Variants;

const
  PM_BASE = WM_USER + 200;
  PM_THREAD_DIE = PM_BASE + 0;
  PM_NEED_AUTH = PM_BASE + 1;
  PM_NEED_USUARIO_SENHA = PM_BASE + 2;
  PM_SET_AUTH = PM_BASE + 3;
  PM_SINCRONIZAR = PM_BASE + 4;

type
  EPicasa = class(Exception);
  EOperacaoAbortada = class(EAbort);
  EPicasaDesconectado = class(EPicasa);
  EFalhaLogin = class(EPicasa);
  EPicasaAuthFailed = class(EPicasa);

  TPicasaAlbum = record
    ID: String;
    Titulo: String;
    Descricao: String;
    QtFotos: Integer;
    ThumbnailAlbum: String;

    LinkToEdit: String;
    EntryXML: String;
  end;

  TPicasaThumbnail = record
    Largura: Integer;
    Altura: Integer;
    URLThumb: String;
  end;

  TPicasaThumbnails = array of TPicasaThumbnail;

  TPicasaFoto = record
    ID: String;
    Titulo: String;
    Descricao: String;
    URLImagem: String;
    MIMEImagem: String;
    AlbumID: String;

    Altura: Integer;
    Largura: Integer;
    TamanhoBytes: Integer;

    Thumbnails: TPicasaThumbnails;
    Timestamp: Int64;

    LinkToEdit: String;
    LinkToEditMedia: String;
    EntryXML: String;
  end;

  TListaDeAlbuns = array of TPicasaAlbum;
  TListaDeFotos = array of TPicasaFoto;

  TPicasaNotifyEvent = procedure (Sender: TObject; Contexto: TObject) of object;
  TOnNeedUsuarioSenha = function (Sender: TObject; Contexto: TObject; var Usuario, Senha: String) : Boolean of object;
  TOnNeedCaptchaValue = function (Sender: TObject; Contexto: TObject; const URLImagCaptcha: String;
                                  var CaptchaValue: String): Boolean of object;

  TOnNeedAuthValue = procedure (Sender: TObject; Contexto: TObject; var AuthValue: String) of object;
  TOnHaveNewAuthValue = procedure (Sender: TObject; Contexto: TObject; const NovoAuthValue: String) of object;
  TOnHaveListaDeAlbuns = procedure (Sender: TObject; Contexto: TObject; ListaDeAlbuns: TListaDeAlbuns) of object;
  TAlbumEvent = procedure (Sender: TObject; Contexto: TObject; Album: TPicasaAlbum) of object;
  TOnListaDeFotos = procedure (Sender: TObject; Contexto: TObject; AlbumID: String; Fotos: TListaDeFotos) of object;
  TOnUploadFoto = procedure (Sender: TObject; Contexto: TObject; AlbumID: String; Foto: TPicasaFoto) of object;
  TOnUploadFotoProgress = procedure (Sender: TObject; Contexto: TObject; const Progresso: Double) of object;
  TFotoEvent = procedure (Sender: TObject; Contexto: TObject; Foto: TPicasaFoto) of object;

  TExceptionNotify = procedure (Sender: TObject; Contexto: TObject; E: Exception) of object;

  TRPicasa = class;

  TProgressoReqHTTPEvent = procedure (Progresso: Double) of object;

  TReqHTTP = class(TObject)
  private
    FInternet: HINTERNET;
    FConexao: HINTERNET;
    FHTTPReq: HINTERNET;
    FStatusCode: Integer;
    FFreeHInternet: Boolean;
    FOnProgresso: TProgressoReqHTTPEvent;
  public
    destructor Destroy; override;

    procedure PrepareRequest(const URL: String; const Verb: String = 'GET');
    procedure PrepareGoogleHeader(const Auth: String);

    procedure SetContentType(const ContentType: String);
    procedure AddHeader(const Header, Valor: String);

    procedure Executar(Conteudo: AnsiString); overload;
    procedure Executar(Stream: TStream); overload;
    procedure Executar; overload;

    procedure LerRetorno(Stream: TStream); overload;
    procedure LerRetorno(StrL: TStringList); overload;
    function LerRetornoXML: IXMLDocument;
    function LerRetornoStr: String;

    property Internet: HINTERNET read FInternet;
    property Conexao: HINTERNET read FConexao;
    property HTTPReq: HINTERNET read FHTTPReq;
    property StatusCode: Integer read FStatusCode;
    property OnProgresso: TProgressoReqHTTPEvent read FOnProgresso write FOnProgresso;
  end;

  TMetodoSinc = procedure of object;

  TThreadGReq = class(TThread)
  private
    FHwndComm: HWND;
    FPicasa: TRPicasa;
    FException: Exception;
    FContexto: TObject;
  protected
    function CriarAuthComLogin(Usuario, Senha: String): String;
    function ObterAuth(const ForceNewAuth: Boolean = False): String; virtual;

    procedure Execute; override;
    procedure Processar;

    procedure ProcReq(const Auth: String); virtual;
    procedure NotificarException; virtual;
    procedure Sincronizar(Metodo: TMetodoSinc; const Esperar: Boolean = True);
  public
    constructor Create(ASuspenso: Boolean; Picasa: TRPicasa; Contexto: TObject);
  end;

  TThreadAlbumBase = class(TThreadGReq)
  protected
    function LerAlbum(NodoEntry: IXMLNode): TPicasaAlbum;
  end;

  TThreadListAlbuns = class(TThreadAlbumBase)
  private
    FLista: TListaDeAlbuns;
  protected
    procedure NotificarLista;
    procedure ProcReq(const Auth: String); override;
    procedure NotificarException; override;
  end;

  TThreadNovoAlbum = class(TThreadAlbumBase)
  private
    FAlbum: TPicasaAlbum;
    FTitulo: String;
    FDescricao: String;
  protected
    procedure NotificarAlbum;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadDeleteAlbum = class(TThreadAlbumBase)
  private
    FLinkToEdit: String;
  protected
    procedure NotificarExclusao;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadUpdateAlbum = class(TThreadAlbumBase)
  private
    FAlbum: TPicasaAlbum;

    FLinkToEdit: String;
  protected
    procedure NotificarUpdate;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadPhotoBase = class(TThreadGReq)
  protected
    function LerFoto(NodoEntry: IXMLNode): TPicasaFoto;
  end;

  TThreadListarFotosBase = class(TThreadPhotoBase)
  private
    FFotos: TListaDeFotos;
  protected
    function ObterURL: String; virtual; abstract;
    procedure NotificarFotos; virtual; abstract;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadListarFotosAlbum = class(TThreadListarFotosBase)
  private
    FAlbumID: String;
  protected
    function ObterURL: String; override;
    procedure NotificarFotos; override;
  end;

  TThreadListarFotosRecentes = class(TThreadListarFotosBase)
  private
    FQt: Integer;
  protected
    function ObterURL: String; override;
    procedure NotificarFotos; override;
  end;

  TThreadUploadFoto = class(TThreadPhotoBase)
  private
    FFoto: TPicasaFoto;

    FArquivo: String;
    FAlbumID: String;
    FTitulo: String;
    FProgresso: Double;
  protected
    procedure OnUploadProgress(Progresso: Double); virtual;
    procedure NotificarProgresso;

    procedure NotificarUpload; virtual;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadDeleteFoto = class(TThreadPhotoBase)
  private
    FLinkToEdit: String;
  protected
    procedure NotificarDelete; virtual;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TThreadAlterarFoto = class(TThreadPhotoBase)
  private
    FLinkToEdit: String;
    FFoto: TPicasaFoto;
  protected
    procedure NotificarUpdate; virtual;
    procedure ProcReq(const Auth: string); override;
    procedure NotificarException; override;
  end;

  TPicasaLoginHandler = class(TObject)
  protected
    function GetUsuarioESenha(var Usuario, Senha: String): Boolean; virtual;
    function GetAuth: String; virtual;
    procedure SetAuth(const Auth: String); virtual;
  end;

  TRPicasa = class(TComponent)
  private
    FOnNeedUsuarioSenha: TOnNeedUsuarioSenha;
    FOnNeedAuthValue: TOnNeedAuthValue;
    FOnHaveNewAuthValue: TOnHaveNewAuthValue;
    FOnHaveListaDeAlbuns: TOnHaveListaDeAlbuns;
    FOnListaDeAlbunsException: TExceptionNotify;
    FOnNovoAlbum: TAlbumEvent;
    FOnNovoAlbumException: TExceptionNotify;
    FOnDeleteAlbum: TPicasaNotifyEvent;
    FOnDeleteAlbumException: TExceptionNotify;
    FOnUpdateAlbum: TAlbumEvent;
    FOnUpdateAlbumException: TExceptionNotify;
    FOnListaFotos: TOnListaDeFotos;
    FOnListaFotosException: TExceptionNotify;
    FOnUploadFoto: TOnUploadFoto;
    FOnUploadFotoException: TExceptionNotify;
    FOnUploadFotoProgress: TOnUploadFotoProgress;
    FOnDeleteFoto: TPicasaNotifyEvent;
    FOnDeleteFotoException: TExceptionNotify;
    FOnUpdateFoto: TFotoEvent;
    FOnUpdateFotoException: TExceptionNotify;

    FOnNeedCaptchaValue: TOnNeedCaptchaValue;

    FThreads: TList<TThreadGReq>;
    FHwndPicasa: HWND;
  protected
    procedure WndPicasa(var Message: TMessage);
    procedure ProcessarMensagensHWNDPicasa;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure EsperarRequisicoesPendentes;
    procedure CancelarRequisicoesPendentes;

    procedure ListarAlbuns(Contexto: TObject = nil);
    procedure CriarAlbum(const Titulo: String; Descricao: String = ''; Contexto: TObject = nil);
    procedure ExcluirAlbum(const LinkToEdit: String; Contexto: TObject = nil);
    procedure AlterarAlbum(const LinkToEdit: String; Album: TPicasaAlbum; Contexto: TObject = nil);

    procedure ListarFotosDeAlbum(const AlbumID: String; Contexto: TObject = nil);
    procedure ListarFotosRecentes(const Quantidade: Integer; Contexto: TObject = nil);

    procedure UploadFoto(const Arquivo, AlbumID: String; const Titulo: String = ''; Contexto: TObject = nil);
    procedure DeletarFoto(const LinkToEdit: String; Contexto: TObject = nil);
    procedure AlterarFoto(const LinkToEdit: String; Foto: TPicasaFoto; Contexto: TObject = nil);
  published
    property OnNeedUsuarioSenha: TOnNeedUsuarioSenha read FOnNeedUsuarioSenha write FOnNeedUsuarioSenha;
    property OnNeedAuthValue: TOnNeedAuthValue read FOnNeedAuthValue write FOnNeedAuthValue;
    property OnHaveNewAuthValue: TOnHaveNewAuthValue read FOnHaveNewAuthValue write FOnHaveNewAuthValue;
    property OnHaveListaDeAlbuns: TOnHaveListaDeAlbuns read FOnHaveListaDeAlbuns write FOnHaveListaDeAlbuns;
    property OnNeedCaptchaValue: TOnNeedCaptchaValue read FOnNeedCaptchaValue write FOnNeedCaptchaValue;
    property OnListaDeAlbunsException: TExceptionNotify read FOnListaDeAlbunsException write FOnListaDeAlbunsException;
    property OnNovoAlbum: TAlbumEvent read FOnNovoAlbum write FOnNovoAlbum;
    property OnNovoAlbumException: TExceptionNotify read FOnNovoAlbumException write FOnNovoAlbumException;
    property OnDeleteAlbum: TPicasaNotifyEvent read FOnDeleteAlbum write FOnDeleteAlbum;
    property OnDeleteAlbumException: TExceptionNotify read FOnDeleteAlbumException write FOnDeleteAlbumException;
    property OnUpdateAlbum: TAlbumEvent read FOnUpdateAlbum write FOnUpdateAlbum;
    property OnUpdateAlbumException: TExceptionNotify read FOnUpdateAlbumException write FOnUpdateAlbumException;
    property OnListaFotos: TOnListaDeFotos read FOnListaFotos write FOnListaFotos;
    property OnListaFotosException: TExceptionNotify read FOnListaFotosException write FOnListaFotosException;
    property OnUploadFoto: TOnUploadFoto read FOnUploadFoto write FOnUploadFoto;
    property OnUploadFotoException: TExceptionNotify read FOnUploadFotoException write FOnUploadFotoException;
    property OnUploadFotoProgress: TOnUploadFotoProgress read FOnUploadFotoProgress write FOnUploadFotoProgress;
    property OnDeleteFoto: TPicasaNotifyEvent read FOnDeleteFoto write FOnDeleteFoto;
    property OnDeleteFotoException: TExceptionNotify read FOnDeleteFotoException write FOnDeleteFotoException;
    property OnUpdateFoto: TFotoEvent read FOnUpdateFoto write FOnUpdateFoto;
    property OnUpdateFotoException: TExceptionNotify read FOnUpdateFotoException write FOnUpdateFotoException;
  end;

function HTTPEncode(const S: String): String;
function HTTPDecode(const S: String): String;
function DateTimeToTimestamp(const Dt: TDateTime): Int64;

var
  GlobalPicasaLoginHandler: TPicasaLoginHandler;

implementation

uses
  DateUtils;

var
  HInternetGlobal: HInternet;

const
  CONTENT_TYPE_ATOM = 'application/atom+xml';

function DateTimeToTimestamp(const Dt: TDateTime): Int64;
begin
  Result := SecondsBetween(Dt, EncodeDate(1970, 1, 1));
end;


{*******************************************************************************
*
*   HTTPEncode
*
*   Sumrio: Esta funo converte um String para um String codificado no padro de Strings HTTP
*   Descrio: Esta funo converte um String para um String codificado no padro de Strings HTTP
*
*   Exemplo: ao passar um texto "www.google.com.br/joo e maria"  retornado
*            "www.google.com.br/joo%20e%20maria"
*
*   Autor: ALC
*   Data: 17/08/09
*   @see HTTPDecode
*}

function HTTPEncode(const S: String): String;
const
  NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-',
                  '0'..'9','$','!','''','(',')'];
var
  i: Integer;
  C: Char;
begin
  Result := '';

  for i := 1 to Length(S) do
  begin
    C := S[i];

    if not CharInSet(C, NoConversion) then
    begin
      if C = ' ' then
        Result := Result + '+'
      else
        Result := Result + Format('%%%.2x', [Ord(C)]);
    end else
      Result := Result + C;
  end;
end;

{*******************************************************************************
*
*   HTTPDecode
*
*   Sumrio: Decodifica um string no padro HTTP para String normal
*   Descrio: Decodifica um string no padro HTTP para String normal
*
*   Exemplo: Ao passar um string "www.google.com.br/joo%20e%20maria",  retornado
*           "www.google.com.br/joo e maria"
*
*   Autor: ALC
*   Data: 17/08/09
*   @see HTTPEncode
*}

function HTTPDecode(const S: String): String;
var
  i, IgnoreCount: Integer;
  C: Char;
  Valor: Byte;
  Temp: String;
begin
  Result := '';
  IgnoreCount := 0;

  for i := 1 to Length(S) do
  begin
    C := S[i];

    if IgnoreCount > 0 then
    begin
      Dec(IgnoreCount);
      Continue;
    end;

    case C of
      '+': Result := Result + ' ';
      '%':
      begin
        IgnoreCount := 2;
        Temp := Copy(S, i + 1, 2);
        Valor := StrToIntDef('$' + Temp, 0);

        if Valor <> 0 then
          Result := Result + Char(Valor);
      end;

      else
        Result := Result + C;
    end;
  end;
end;

function GetSubNodeValue(Nodo: IXMLNode; Nome: String): Variant;
var
  i: Integer;
begin
  Result := Null;

  for i := 0 to Nodo.ChildNodes.Count - 1 do
  begin
    if Nodo.ChildNodes[i].NodeName = Nome then
    begin
      Result := Nodo.ChildNodes[i].NodeValue;
      Exit;
    end;
  end;
end;

function GetSubNodeStr(Nodo: IXMLNode; Nome: String): String;
var
  i: Integer;
  Temp: Variant;
begin
  Result := '';

  for i := 0 to Nodo.ChildNodes.Count - 1 do
  begin
    if Nodo.ChildNodes[i].NodeName = Nome then
    begin
      Temp := Nodo.ChildNodes[i].NodeValue;

      if Temp <> Null then
        Result := Temp;

      Exit;
    end;
  end;
end;


{ TRPicasa }

procedure TRPicasa.AlterarAlbum(const LinkToEdit: String; Album: TPicasaAlbum; Contexto: TObject = nil);
begin
  with TThreadUpdateAlbum.Create(True, Self, Contexto) do
  begin
    FLinkToEdit := LinkToEdit;
    FAlbum := Album;
    Start;
  end;
end;

procedure TRPicasa.AlterarFoto(const LinkToEdit: String; Foto: TPicasaFoto;
  Contexto: TObject);
begin
  with TThreadAlterarFoto.Create(True, Self, Contexto) do
  begin
    FLinkToEdit := LinkToEdit;
    FFoto := Foto;
    Start;
  end;
end;

procedure TRPicasa.CancelarRequisicoesPendentes;
var
  i: Integer;
begin
  for i := 0 to FThreads.Count - 1 do
  begin
    FThreads[i].FHwndComm := 0;
    FThreads[i].Terminate;
  end;
end;

constructor TRPicasa.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FThreads := TList<TThreadGReq>.Create;

  if not (csDesigning in Self.ComponentState) then
  begin
    FHwndPicasa := AllocateHWnd(Self.WndPicasa);
  end;
end;

procedure TRPicasa.CriarAlbum(const Titulo: String; Descricao: String;
  Contexto: TObject);
begin
  with TThreadNovoAlbum.Create(True, Self, Contexto) do
  begin
    FTitulo := Titulo;
    FDescricao := Descricao;
    Start;
  end;
end;

procedure TRPicasa.DeletarFoto(const LinkToEdit: String; Contexto: TObject);
begin
  with TThreadDeleteFoto.Create(True, Self, Contexto) do
  begin
    FLinkToEdit := LinkToEdit;
    Start;
  end;
end;

destructor TRPicasa.Destroy;
var
  i: Integer;
begin
  for i := 0 to FThreads.Count - 1 do
    FThreads[i].FHwndComm := 0;

  try
    Self.ProcessarMensagensHWNDPicasa();
  except
    //
  end;

  if FHwndPicasa <> 0 then
    DeallocateHWnd(FHwndPicasa);

  FThreads.Free;
  inherited;
end;

procedure TRPicasa.EsperarRequisicoesPendentes;
var
  Arr: Array of Integer;
  i, Retorno: Integer;
begin
  if FThreads.Count = 0 then
    Exit;

  SetLength(Arr, FThreads.Count);

  for i := 0 to FThreads.Count - 1 do
    Arr[i] := FThreads[i].Handle;

  while True do
  begin
    Retorno := Windows.WaitForMultipleObjects(Length(Arr), @Arr[0], True, 100);

    if Retorno = WAIT_TIMEOUT then
    begin
      Self.ProcessarMensagensHWNDPicasa();
    end else
      Break;
  end;
end;

procedure TRPicasa.ExcluirAlbum(const LinkToEdit: String; Contexto: TObject = nil);
begin
  with TThreadDeleteAlbum.Create(True, Self, Contexto) do
  begin
    FLinkToEdit := LinkToEdit;
    Start;
  end;
end;

procedure TRPicasa.ListarAlbuns(Contexto: TObject = nil);
begin
  TThreadListAlbuns.Create(False, Self, Contexto);
end;

procedure TRPicasa.ListarFotosDeAlbum(const AlbumID: String; Contexto: TObject = nil);
begin
  with TThreadListarFotosAlbum.Create(True, Self, Contexto) do
  begin
    FAlbumID := AlbumID;
    Start;
  end;
end;

procedure TRPicasa.ListarFotosRecentes(const Quantidade: Integer; Contexto: TObject = nil);
begin
  with TThreadListarFotosRecentes.Create(True, Self, Contexto) do
  begin
    FQt := Quantidade;
    Start;
  end;
end;

procedure TRPicasa.ProcessarMensagensHWNDPicasa;
var
  Msg: tagMSG;
begin
  while PeekMessage(Msg, FHwndPicasa, 0, 0, PM_REMOVE) do
  begin
    TranslateMessage(Msg);
    Dispatch(Msg);
  end;
end;

procedure TRPicasa.UploadFoto(const Arquivo, AlbumID: String; const Titulo: String = '';
  Contexto: TObject = nil);
begin
  with TThreadUploadFoto.Create(True, Self, Contexto) do
  begin
    FArquivo := Arquivo;
    FAlbumID := AlbumID;
    FTitulo := Titulo;
    Start;
  end;
end;

procedure TRPicasa.WndPicasa(var Message: TMessage);
var
  PStr, PStr2: PString;
  Metodo: TMetodoSinc;
begin
  case Message.Msg of
    PM_THREAD_DIE:
    begin
      FThreads.Remove(TThreadGReq(Message.WParam));
    end;

    PM_NEED_AUTH:
    begin
      PStr := PString(Message.WParam);

      if Assigned(FOnNeedAuthValue) then
        FOnNeedAuthValue(Self, nil, PStr^)
      else
        if GlobalPicasaLoginHandler <> nil then
          PStr^ := GlobalPicasaLoginHandler.GetAuth
        else
          PStr^ := '';
    end;

    PM_NEED_USUARIO_SENHA:
    begin
      PStr := PString(Message.WParam);
      PStr2 := PString(Message.LParam);

      if GlobalPicasaLoginHandler = nil then
      begin
        PStr^ := '';
        PStr2^ := '';
        Message.Result := 0;
      end else
        if GlobalPicasaLoginHandler.GetUsuarioESenha(PStr^, PStr2^) then
          Message.Result := 1
        else
          Message.Result := 0;

      if Assigned(FOnNeedUsuarioSenha) then
      begin
        if FOnNeedUsuarioSenha(Self, nil, PStr^, PStr2^) then
          Message.Result := 1
        else
          Message.Result := 0;
      end;
    end;

    PM_SET_AUTH:
    begin
      PStr := PString(Message.WParam);

      if Assigned(FOnHaveNewAuthValue) then
        FOnHaveNewAuthValue(Self, nil, PStr^)
      else
        if GlobalPicasaLoginHandler <> nil then
          GlobalPicasaLoginHandler.SetAuth(PStr^);
    end;

    PM_SINCRONIZAR:
    begin
      TMethod(Metodo).Code := Pointer(Message.WParam);
      TMethod(Metodo).Data := Pointer(Message.LParam);
      Metodo();
    end

    else
      Message.Result := DefWindowProc(FHwndPicasa, Message.Msg,
                                      Message.WParam, Message.LParam);
  end;
end;

{ TThreadGReq }

constructor TThreadGReq.Create(ASuspenso: Boolean; Picasa: TRPicasa; Contexto: TObject);
begin
  FPicasa := Picasa;
  FHwndComm := Picasa.FHwndPicasa;
  Picasa.FThreads.Add(Self);
  FContexto := Contexto;
  inherited Create(ASuspenso);
end;

function TThreadGReq.CriarAuthComLogin(Usuario, Senha: String): String;
var
  Req: TReqHTTP;
  Content: AnsiString;
  StrL: TStringList;
  Erro: String;
begin
  Req := TReqHTTP.Create;

  try
    Req.PrepareRequest('https://www.google.com/accounts/ClientLogin', 'POST');

    Content := AnsiString(Format('accountType=HOSTED_OR_GOOGLE&'+
                                'Email=%s&Passwd=%s&'+
                                'service=%s&source=%s',

                                [HTTPEncode(Usuario),
                                 HTTPEncode(Senha),
                                 HTTPEncode('lh2'),
                                 HTTPEncode('meintec-rrpgfirecast-4.x')]));

    Req.SetContentType('application/x-www-form-urlencoded');
    Req.AddHeader('GData-Version', '2');
    Req.Executar(Content);

    StrL := TStringList.Create;

    try
      if Req.StatusCode = HTTP_STATUS_OK then
      begin
        // Login OK
        Req.LerRetorno(StrL);
        Result := StrL.Values['Auth'];
      end else
        if Req.StatusCode = HTTP_STATUS_FORBIDDEN then
        begin
          Req.LerRetorno(StrL);
          Erro := UpperCase(StrL.Values['Error']);

(*          if Erro = UpperCase('CaptchaRequired') then
          begin
            // Captcha
          end else*)
            raise EFalhaLogin.Create('Falha ao logar no Picasa: ' + StrL.Values['Error']);

          // Vamos checar
        end else
          raise EPicasa.Create('Erro desconhecido ao efetuar login Picasa: ' + #13#10 +
                               Req.LerRetornoStr);
    finally
      StrL.Free;
    end;
  finally
    Req.Free;
  end;
end;

procedure TThreadGReq.Execute;
begin
  try
    try
      Self.FreeOnTerminate := True;

      try
        Self.Processar();
      except
        on E: Exception do
        begin
          FException := E;

          try
            Self.Sincronizar(Self.NotificarException);
          finally
            FException := nil;
          end;
        end;
      end;
    finally
      if FHwndComm <> 0 then
        SendMessage(FHwndComm, PM_THREAD_DIE, Integer(Self), 0);
    end;
  except
    on E: Exception do
    begin
      // Ignorar este erro
    end;
  end;
end;

procedure TThreadGReq.NotificarException;
begin

end;

function TThreadGReq.ObterAuth(const ForceNewAuth: Boolean = False): String;
var
  Temp, Usuario, Senha: String;
begin
  Result := '';
  Temp := '';

  if not ForceNewAuth then
    if FHwndComm <> 0 then
      SendMessage(FHwndComm, PM_NEED_AUTH, Integer(@Temp), 0);

  if Temp = '' then
  begin
    // Devemos obter um Auth
    Usuario := '';
    Senha := '';

    if (FHwndComm <> 0) and
       (SendMessage(FHwndComm, PM_NEED_USUARIO_SENHA, Integer(@Usuario), Integer(@Senha)) = 1) then
    begin
      Temp := Self.CriarAuthComLogin(Usuario, Senha);
      Result := Temp;
      SendMessage(FHwndComm, PM_SET_AUTH, Integer(@Temp), 0);
    end else
      raise EOperacaoAbortada.Create('Abortado ao informar usurio e senha Picasa');
  end else
    Result := Temp;
end;

procedure TThreadGReq.Processar;
var
  Auth: String;
begin
  Auth := Self.ObterAuth();

  try
    Self.ProcReq(Auth);
  except
    on E: EPicasaAuthFailed do
    begin
      Auth := Self.ObterAuth(True);
      Self.ProcReq(Auth);
    end;

    else
      raise;
  end;
end;

procedure TThreadGReq.ProcReq;
begin

end;

procedure TThreadGReq.Sincronizar(Metodo: TMetodoSinc; const Esperar: Boolean = True);
var
  M: TMethod;
begin
  M := TMethod(Metodo);

  if FHwndComm <> 0 then
    if Esperar then
      SendMessage(FHwndComm, PM_SINCRONIZAR, Integer(M.Code), Integer(M.Data))
    else
      PostMessage(FHwndComm, PM_SINCRONIZAR, Integer(M.Code), Integer(M.Data));
end;

{ TReqHTTP }

procedure TReqHTTP.AddHeader(const Header, Valor: String);
var
  H: String;
begin
  H := Header + ': ' + Valor;

  if not HttpAddRequestHeaders(FHTTPReq, PChar(H), Length(H), HTTP_ADDREQ_FLAG_ADD) then
    RaiseLastOSError;
end;

destructor TReqHTTP.Destroy;
begin
  if FHTTPReq <> nil then
  begin
    WinInet.InternetCloseHandle(FHTTPReq);
    FHTTPReq := nil;
  end;

  if FConexao <> nil then
  begin
    InternetCloseHandle(FConexao);
    FConexao := nil;
  end;

  if FFreeHInternet then
    if FInternet <> nil then
    begin
      WinInet.InternetCloseHandle(FInternet);
      FInternet := nil;
    end;

  inherited;
end;

procedure TReqHTTP.Executar(Conteudo: AnsiString);
var
  St: TStringStream;
begin
  St := TStringStream.Create(Conteudo);

  try
    Self.Executar(St);
  finally
    St.Free;
  end;
end;

procedure TReqHTTP.Executar(Stream: TStream);
var
  InB: TInternetBuffersW;
  Buffer: Pointer;
  Qt, Resto, i, TamanhoTotal: Integer;
  Temp, Reservado, TamanhoJaEnviado: Cardinal;

const
  BLOCK_SIZE = 4096;


  procedure DoStatus(Percentual: Double);
  begin
    if Assigned(FOnProgresso) then
      FOnProgresso(Percentual);
  end;

  procedure DoStatusMax(const Atual, Max: Integer);
  var
    Percentual: Double;
  begin
    if Max <> 0 then
      Percentual := (Atual / Max) * 100
    else
      Percentual := 0;

    DoStatus(Percentual);
  end;

  procedure EnviarPacote(Buffer: Pointer; Tamanho: Cardinal);
  var
    Escrito: Cardinal;
  begin
    Escrito := 0;

    if not InternetWriteFile(FHTTPReq, Buffer, Tamanho, Escrito) then
      RaiseLastOSError;

    if Escrito < Tamanho then
      raise EPicasaDesconectado.Create('Erro ao transmitir dados');

    TamanhoJaEnviado := TamanhoJaEnviado + Tamanho;
    DoStatusMax(TamanhoJaEnviado, TamanhoTotal);
  end;
begin
  TamanhoJaEnviado := 0;
  FillChar(InB, SizeOf(InB), 0);
  InB.dwStructSize := SizeOf(InB);

  if Stream <> nil then
    TamanhoTotal := Stream.Size
  else
    TamanhoTotal := 0;

  InB.dwBufferTotal := TamanhoTotal;

  DoStatus(0.0);

  if not HttpSendRequestEx(FHTTPReq, @InB, nil, 0, 0) then
    RaiseLastOSError;

  if (TamanhoTotal > 0) then
  begin
    GetMem(Buffer, BLOCK_SIZE);

    try
      Qt := TamanhoTotal div BLOCK_SIZE;
      Resto := TamanhoTotal mod BLOCK_SIZE;
      Stream.Position := 0;

      for i := 1 to Qt do
      begin
        Stream.Read(Buffer^, BLOCK_SIZE);
        EnviarPacote(Buffer, BLOCK_SIZE);
      end;

      if Resto > 0 then
      begin
        Stream.Read(Buffer^, Resto);
        EnviarPacote(Buffer, Resto);
      end;
    finally
      FreeMem(Buffer, BLOCK_SIZE);
    end;
  end else
    DoStatus(100);

  if not HttpEndRequest(FHTTPReq, nil, 0, 0) then
    RaiseLastOSError;

  Temp := 4;
  Reservado := 0;

  if not HttpQueryInfo(FHTTPReq, HTTP_QUERY_STATUS_CODE or HTTP_QUERY_FLAG_NUMBER, @FStatusCode, Temp,
                       Reservado) then
    RaiseLastOSError;
end;

procedure TReqHTTP.Executar;
begin
  Self.Executar(nil);
end;

procedure TReqHTTP.LerRetorno(Stream: TStream);
var
  Buf: Pointer;
  Lido: Cardinal;
const
  BLOCK_SIZE = 4096;
begin
  GetMem(Buf, BLOCK_SIZE);

  try
    while True do
    begin
      Lido := 0;

      if not InternetReadFile(FHTTPReq, Buf, BLOCK_SIZE, Lido) then
        raise EPicasaDesconectado.Create('Desconectado durante a leitura Picasa');

      if Lido = 0 then
        Break;

      Stream.Write(Buf^, Lido);
    end;
  finally
    FreeMem(Buf, BLOCK_SIZE);
  end;
end;

procedure TReqHTTP.LerRetorno(StrL: TStringList);
var
  M: TMemoryStream;
  Temp: AnsiString;
begin
  M := TStringStream.Create;

  try
    Self.LerRetorno(M);

    M.Position := 0;
    SetLength(Temp, M.Size);
    M.Read(Temp[1], M.Size);

    StrL.Text := String(Temp);
  finally
    M.Free;
  end;
end;

function TReqHTTP.LerRetornoStr: String;
var
  M: TMemoryStream;
  Temp: AnsiString;
begin
  M := TMemoryStream.Create;

  try
    Self.LerRetorno(M);

    SetLength(Temp, M.Size);
    M.Position := 0;
    M.Read(Temp[1], M.Size);

    Result := String(Temp);
  finally
    M.Free;
  end;
end;

function TReqHTTP.LerRetornoXML: IXMLDocument;
var
  M: TMemoryStream;
begin
  M := TMemoryStream.Create;

  try
    Self.LerRetorno(M);
    M.Position := 0;

    Result := NewXMLDocument();
    Result.LoadFromStream(M);
  finally
    M.Free;
  end;
end;

procedure TReqHTTP.PrepareGoogleHeader(const Auth: String);
begin
  Self.AddHeader('GData-Version', '2');
  Self.AddHeader('Authorization',  'GoogleLogin auth=' + Auth);
end;

procedure TReqHTTP.PrepareRequest(const URL: String; const Verb: String = 'GET');
var
  Comps: TURLComponents;
  Host, Documento: String;
  Flags: Integer;
begin
  FStatusCode := 0;

  if FInternet = nil then
  begin
    if HInternetGlobal = nil then
      HInternetGlobal := InternetOpen('RRPGFC', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);

    FInternet := HInternetGlobal;

    if FInternet = nil then
    begin
      FInternet := InternetOpen('RRPGFC', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);

      if FInternet = nil then
        RaiseLastOSError;

      FFreeHInternet := True;
    end;
  end;

  if FHTTPReq <> nil then
  begin
    InternetCloseHandle(FHTTPReq);
    FHTTPReq := nil;
  end;

  if FConexao <> nil then
  begin
    InternetCloseHandle(FConexao);
    FConexao := nil;
  end;

  FillChar(Comps, SizeOf(Comps), 0);
  SetLength(Host, 1024);
  Comps.lpszHostName := PChar(Host);
  Comps.dwHostNameLength := Length(Host);

  SetLength(Documento, 1024);
  Comps.lpszUrlPath := PChar(Documento);
  Comps.dwUrlPathLength := Length(Documento);

  if not InternetCrackUrl(PChar(URL), Length(URL), 0, Comps) then
    RaiseLastOSError;

  Host := PChar(Host);
  Documento := PChar(Documento);

  Flags := INTERNET_FLAG_IGNORE_CERT_CN_INVALID or INTERNET_FLAG_IGNORE_CERT_DATE_INVALID;

  if Comps.nScheme = INTERNET_SCHEME_HTTPS then
    Flags := Flags or INTERNET_FLAG_SECURE;

  FConexao := InternetConnect(FInternet, PChar(Host), Comps.nPort, nil, nil,
                              INTERNET_SERVICE_HTTP, Flags, 0);

  if FConexao = nil then
    RaiseLastOSError;

  FHTTPReq := HttpOpenRequest(FConexao, PChar(Verb), PChar(Documento), nil, nil, nil,
                              Flags, 0);

  if FHTTPReq = nil then
    RaiseLastOSError;
end;

procedure TReqHTTP.SetContentType(const ContentType: String);
begin
  Self.AddHeader('Content-type', ContentType);
end;

{ TThreadListAlbuns }

function TThreadAlbumBase.LerAlbum(NodoEntry: IXMLNode): TPicasaAlbum;
var
  i: Integer;
  N, N2: IXMLNode;
begin
  Result.ID := GetSubNodeStr(NodoEntry, 'gphoto:id');
  Result.Titulo := GetSubNodeStr(NodoEntry, 'title');
  Result.Descricao := GetSubNodeStr(NodoEntry, 'summary');
  Result.QtFotos := GetSubNodeValue(NodoEntry, 'gphoto:numphotos');

  Result.EntryXML := NodoEntry.XML;
  Result.ThumbnailAlbum := '';
  Result.LinkToEdit := '';

  for i := 0 to NodoEntry.ChildNodes.Count - 1 do
  begin
    N := NodoEntry.ChildNodes[i];

    if N.NodeName = 'link' then
      if N.Attributes['rel'] = 'edit' then
      begin
        Result.LinkToEdit := N.Attributes['href'];
      end;

    if N.NodeName = 'media:group' then
    begin
      N2 := N.ChildNodes.FindNode('media:thumbnail');

      if N2 <> nil then
        Result.ThumbnailAlbum := N2.Attributes['url'];
    end;
  end;
end;

procedure TThreadListAlbuns.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnListaDeAlbunsException) then
    FPicasa.OnListaDeAlbunsException(FPicasa, FContexto, FException);
end;

procedure TThreadListAlbuns.NotificarLista;
begin
  if Assigned(FPicasa.OnHaveListaDeAlbuns) then
    FPicasa.OnHaveListaDeAlbuns(FPicasa, FContexto, FLista);
end;

procedure TThreadListAlbuns.ProcReq(const Auth: String);
var
  Req: TReqHTTP;
  XML: IXMLDocument;
  NodoPai, Nodo: IXMLNode;
  i: Integer;

  procedure AddEntrada(Nodo: IXMLNode);
  var
    Item: TPicasaAlbum;
    N: Integer;
  begin
    Item := Self.LerAlbum(Nodo);

    N := Length(FLista);
    SetLength(FLista, N + 1);
    FLista[N] := Item;
  end;
begin
  inherited;
  Req := TReqHTTP.Create;

  try
    Req.PrepareRequest('https://picasaweb.google.com/data/feed/api/user/default', 'GET');
    Req.PrepareGoogleHeader(Auth);
    Req.Executar();

    if Req.StatusCode = HTTP_STATUS_OK then
    begin
      SetLength(FLista, 0);
      CoInitialize(nil);

      try
        XML := Req.LerRetornoXML();
        NodoPai := XML.ChildNodes.FindNode('feed');

        for i := 0 to NodoPai.ChildNodes.Count - 1 do
        begin
          Nodo := NodoPai.ChildNodes[i];

          if Nodo.NodeName = 'entry' then
            AddEntrada(Nodo);
        end;
      finally
        CoUninitialize;
      end;

      Self.Sincronizar(Self.NotificarLista);
    end else
      if Req.StatusCode = HTTP_STATUS_FORBIDDEN then
      begin
        raise EPicasaAuthFailed.Create(Req.LerRetornoStr);
      end else
        raise EPicasa.Create('Erro desconhecido ao obter Albuns do Picasa: ' +
                             Req.LerRetornoStr);
  finally
    Req.Free;
  end;
end;

{ TThreadNovoAlbum }

procedure TThreadNovoAlbum.NotificarAlbum;
begin
  if Assigned(FPicasa.OnNovoAlbum) then
    FPicasa.OnNovoAlbum(FPicasa, FContexto, FAlbum);
end;

procedure TThreadNovoAlbum.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnNovoAlbumException) then
    FPicasa.OnNovoAlbumException(FPicasa, FContexto, FException);
end;

procedure TThreadNovoAlbum.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
  XML: IXMLDocument;
begin
  CoInitialize(nil);

  try
    Req := TReqHTTP.Create;

    try
      XML := NewXMLDocument();
      XML.Encoding := 'UTF-8';

      with XML.AddChild('entry', 'http://www.w3.org/2005/Atom') do
      begin
        Attributes['xmlns:gphoto'] := 'http://schemas.google.com/photos/2007';
        Attributes['xmlns:media'] := 'http://search.yahoo.com/mrss/';

        with AddChild('title') do
        begin
          Attributes['type'] := 'text';
          Text := FTitulo;
        end;

        with AddChild('summary') do
        begin
          Attributes['type'] := 'text';
          Text := FDescricao;
        end;

        AddChild('gphoto:access').Text := 'public';

        with AddChild('category') do
        begin
          Attributes['scheme'] := 'http://schemas.google.com/g/2005#kind';
          Attributes['term'] := 'http://schemas.google.com/photos/2007#album';
        end;
      end;

      Req.PrepareRequest('https://picasaweb.google.com/data/feed/api/user/alyssonrpg', 'POST');
      Req.PrepareGoogleHeader(Auth);

      Req.SetContentType(CONTENT_TYPE_ATOM);
      Req.Executar(UTF8Encode(XML.XML.Text));

      if Req.StatusCode = 201 then
      begin
        XML := Req.LerRetornoXML();

        FAlbum := Self.LerAlbum(XML.ChildNodes.FindNode('entry'));
        Self.Sincronizar(Self.NotificarAlbum);
        // OK
      end else
        if Req.StatusCode = HTTP_STATUS_FORBIDDEN then
          raise EPicasaAuthFailed.Create(Req.LerRetornoStr)
        else
          raise EPicasa.Create('Erro desconhecido ao criar Album: ' + #13#10 +
                               Req.LerRetornoStr);
    finally
      Req.Free;
    end;
  finally
    CoUninitialize
  end;
end;

{ TThreadDeleteAlbum }

procedure TThreadDeleteAlbum.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.FOnDeleteAlbumException) then
    FPicasa.OnDeleteAlbumException(FPicasa, FContexto, FException);
end;

procedure TThreadDeleteAlbum.NotificarExclusao;
begin
  if Assigned(FPicasa.FOnDeleteAlbum) then
    FPicasa.FOnDeleteAlbum(FPicasa, FContexto);
end;

procedure TThreadDeleteAlbum.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
begin
  Req := TReqHTTP.Create;

  try
    Req.PrepareRequest(FLinkToEdit, 'DELETE');
    Req.PrepareGoogleHeader(Auth);
    Req.AddHeader('If-Match', '*');
    Req.Executar();

    case Req.StatusCode of
      HTTP_STATUS_OK: Self.Sincronizar(Self.NotificarExclusao);
      HTTP_STATUS_FORBIDDEN: raise EPicasaAuthFailed.Create(Req.LerRetornoStr);

      else
        raise EPicasa.Create('Erro desconhecido ao apagar album:' + #13#10 +
                              Req.LerRetornoStr);
    end;
  finally
    Req.Free;
  end;
end;

{ TThreadUpdateAlbum }

procedure TThreadUpdateAlbum.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnUpdateAlbumException) then
    FPicasa.OnUpdateAlbumException(FPicasa, FContexto, FException);
end;

procedure TThreadUpdateAlbum.NotificarUpdate;
begin
  if Assigned(FPicasa.OnUpdateAlbum) then
    FPicasa.OnUpdateAlbum(FPicasa, FContexto, FAlbum);
end;

procedure TThreadUpdateAlbum.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
  XML: IXMLDocument;
begin
  CoInitialize(nil);

  try
    Req := TReqHTTP.Create;

    try
      XML := LoadXMLData(FAlbum.EntryXML);

      with XML.ChildNodes.FindNode('entry') do
      begin
        ChildValues['title'] := FAlbum.Titulo;
        ChildValues['summary'] := FAlbum.Descricao;
      end;

      Req.PrepareRequest(FLinkToEdit, 'PUT');
      Req.PrepareGoogleHeader(Auth);

      Req.SetContentType(CONTENT_TYPE_ATOM);
      Req.AddHeader('If-Match', '*');
      Req.Executar(UTF8Encode(XML.XML.Text));

      if Req.StatusCode = 200 then
      begin
        XML := Req.LerRetornoXML();

        FAlbum := Self.LerAlbum(XML.ChildNodes.FindNode('entry'));
        Self.Sincronizar(Self.NotificarUpdate);
        // OK
      end else
        if Req.StatusCode = HTTP_STATUS_FORBIDDEN then
          raise EPicasaAuthFailed.Create(Req.LerRetornoStr)
        else
          raise EPicasa.Create('Erro desconhecido ao alterar o Album: ' +
                                Req.LerRetornoStr);
    finally
      Req.Free;
    end;
  finally
    CoUninitialize
  end;
end;

{ TThreadPhotoBase }

function TThreadPhotoBase.LerFoto(NodoEntry: IXMLNode): TPicasaFoto;
var
  N, N2: IXMLNode;
  i, j, TempInt: Integer;
  Thumb: TPicasaThumbnail;
begin
  Initialize(Result);
  FillChar(Result, SizeOf(Result), 0);

  Result.EntryXML := NodoEntry.XML;
  Result.ID := GetSubNodeStr(NodoEntry, 'gphoto:id');
  Result.AlbumID := GetSubNodeStr(NodoEntry, 'gphoto:albumid');
  Result.Titulo := GetSubNodeStr(NodoEntry, 'title');
  Result.Descricao := GetSubNodeStr(NodoEntry, 'summary');

  N := NodoEntry.ChildNodes.FindNode('content');

  if N <> nil then
  begin
    Result.MIMEImagem := N.Attributes['type'];
    Result.URLImagem := N.Attributes['src'];
  end;

  Result.Largura := StrToIntDef(GetSubNodeStr(NodoEntry, 'gphoto:width'), 0);
  Result.Altura := StrToIntDef(GetSubNodeStr(NodoEntry, 'gphoto:height'), 0);
  Result.TamanhoBytes := StrToIntDef(GetSubNodeStr(NodoEntry, 'gphoto:size'), 0);
  Result.Timestamp := StrToInt64Def(GetSubNodeStr(NodoEntry, 'gphoto:timestamp'), 0);

  for i := 0 to NodoEntry.ChildNodes.Count - 1 do
  begin
    N := NodoEntry.ChildNodes[i];

    if N.NodeName = 'link' then
    begin
      if N.Attributes['rel'] = 'edit' then
        Result.LinkToEdit := N.Attributes['href']
      else
        if N.Attributes['rel'] = 'edit-media' then
          Result.LinkToEditMedia := N.Attributes['href'];
    end else
      if N.NodeName = 'media:group' then
      begin
        for j := 0 to N.ChildNodes.Count - 1 do
        begin
          N2 := N.ChildNodes[j];

          if N2.NodeName = 'media:thumbnail' then
          begin
            Thumb.Largura := N2.Attributes['width'];
            Thumb.Altura := N2.Attributes['height'];
            Thumb.URLThumb := N2.Attributes['url'];

            TempInt := Length(Result.Thumbnails);
            SetLength(Result.Thumbnails, TempInt + 1);
            Result.Thumbnails[TempInt] := Thumb;
          end; {else
            if N2.NodeName = 'media:content' then
            begin
              Result.URLImagem := N2.Attributes['url'];
            end;}
        end;
      end;
  end;
end;

{ TThreadListarFotos }

procedure TThreadListarFotosBase.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnListaFotosException) then
    FPicasa.OnListaFotosException(FPicasa, FContexto, FException);
end;

procedure TThreadListarFotosBase.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
  URL: String;
  XML: IXMLDocument;
  i: Integer;
  N, NPai: IXMLNode;

  procedure Add(Node: IXMLNode);
  var
    Foto: TPicasaFoto;
    N: Integer;
  begin
    Foto := Self.LerFoto(Node);

    N := Length(FFotos);
    SetLength(FFotos, N + 1);
    FFotos[N] := Foto;
  end;
begin
  inherited;
  CoInitialize(nil);

  try
    Req := TReqHTTP.Create;

    try
      URL := Self.ObterURL();
      Req.PrepareRequest(URL, 'GET');
      Req.PrepareGoogleHeader(Auth);
      Req.Executar();

      case Req.StatusCode of
        HTTP_STATUS_OK:
        begin
          XML := Req.LerRetornoXML();
          SetLength(FFotos, 0);

          NPai := XML.ChildNodes.FindNode('feed');

          for i := 0 to NPai.ChildNodes.Count - 1 do
          begin
            N := NPai.ChildNodes[i];

            if N.NodeName = 'entry' then
              Add(N);
          end;

          Self.Sincronizar(Self.NotificarFotos);
        end;

        HTTP_STATUS_FORBIDDEN: raise EPicasaAuthFailed.Create(Req.LerRetornoStr);

        else
          raise EPicasa.Create('Erro desconhecido ao listar fotos de album: ' + #13#10 +
                               Req.LerRetornoStr());
      end;
    finally
      Req.Free;
    end;
  finally
    CoUninitialize;
  end;
end;

{ TThreadListarFotosAlbum }

procedure TThreadListarFotosAlbum.NotificarFotos;
begin
  if Assigned(FPicasa.OnListaFotos) then
    FPicasa.OnListaFotos(FPicasa, FContexto, FAlbumID, FFotos);
end;

function TThreadListarFotosAlbum.ObterURL: String;
begin
  Result := 'https://picasaweb.google.com/data/feed/api/user/default/albumid/' + FAlbumID;
end;

{ TThreadListarFotosRecentes }

procedure TThreadListarFotosRecentes.NotificarFotos;
begin
  if Assigned(FPicasa.OnListaFotos) then
    FPicasa.OnListaFotos(FPicasa, FContexto, '', FFotos);
end;

function TThreadListarFotosRecentes.ObterURL: String;
begin
  Result := 'https://picasaweb.google.com/data/feed/api/user/default?kind=photo&max-results=' + IntToStr(FQt);
end;

{ TThreadUploadFoto }

procedure TThreadUploadFoto.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnUploadFotoException) then
    FPicasa.OnUploadFotoException(FPicasa, FContexto, FException);
end;

procedure TThreadUploadFoto.NotificarProgresso;
begin
  if Assigned(FPicasa.OnUploadFotoProgress) then
    FPicasa.OnUploadFotoProgress(FPicasa, FContexto, FProgresso);
end;

procedure TThreadUploadFoto.NotificarUpload;
begin
  if Assigned(FPicasa.OnUploadFoto) then
    FPicasa.OnUploadFoto(FPicasa, FContexto, FAlbumID, FFoto);
end;

procedure TThreadUploadFoto.OnUploadProgress(Progresso: Double);
begin
  FProgresso := Progresso;

  if Self.Terminated then
    Abort; // Abortar o Upload

  Self.Sincronizar(Self.NotificarProgresso, False);
end;

procedure TThreadUploadFoto.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
  URL, Slug, Ext, Mime: String;
  Stream: TFileStream;
  XML: IXMLDocument;
  NodeEntry: IXMLNode;
begin
  inherited;
  CoInitialize(nil);

  try
    Req := TReqHTTP.Create;

    try
      Req.OnProgresso := Self.OnUploadProgress;

      URL := 'https://picasaweb.google.com/data/feed/api/user/default/albumid/';

      if FAlbumID <> '' then
        URL := URL + FAlbumID
      else
        URL := URL + 'default';

      Req.PrepareRequest(URL, 'POST');
      Req.PrepareGoogleHeader(Auth);

      if FTitulo = '' then
        Slug := ExtractFileName(FArquivo)
      else
        Slug := FTitulo;

      Ext := UpperCase(ExtractFileExt(FArquivo));

      if (Ext = '.JPG') or (Ext = '.JPEG') then
        Mime := 'image/jpg'
      else
        if Ext = '.BMP' then
          Mime := 'image/bmp'
        else
          if Ext = '.GIF' then
            Mime := 'image/gif'
          else
            if Ext = '.PNG' then
              Mime := 'image/png'
            else
              Mime := 'image';

      Req.AddHeader('Content-Type', Mime);
      Req.AddHeader('Slug', Slug);

      Stream := TFileStream.Create(FArquivo, fmOpenRead or fmShareDenyWrite);

      try
        try
          Req.Executar(Stream);
        except
          on E: EAbort do
            Exit; // O Upload foi abortado

          else
            raise;
        end;
      finally
        Stream.Free;
      end;

      case Req.StatusCode of
        201:
        begin
          XML := Req.LerRetornoXML();
          NodeEntry := XML.ChildNodes.FindNode('entry');

          if NodeEntry <> nil then
            FFoto := Self.LerFoto(NodeEntry)
          else
            Initialize(FFoto);

          Self.Sincronizar(Self.NotificarUpload);
        end;

        HTTP_STATUS_FORBIDDEN: raise EPicasaAuthFailed.Create('AuthFailed: ' + Req.LerRetornoStr);

        else
          raise EPicasa.Create('Erro desconhecido durante o upload da foto: ' + #13#10 +
                                Req.LerRetornoStr);
      end;
    finally
      Req.Free;
    end;
  finally
    CoUninitialize;
  end;
end;

{ TThreadDeleteFoto }

procedure TThreadDeleteFoto.NotificarDelete;
begin
  if Assigned(FPicasa.OnDeleteFoto) then
    FPicasa.OnDeleteFoto(FPicasa, FContexto);
end;

procedure TThreadDeleteFoto.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnDeleteFotoException) then
    FPicasa.OnDeleteFotoException(FPicasa, FContexto, FException);
end;

procedure TThreadDeleteFoto.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
begin
  inherited;
  Req := TReqHTTP.Create;

  try
    Req.PrepareRequest(FLinkToEdit, 'DELETE');
    Req.PrepareGoogleHeader(Auth);
    Req.AddHeader('If-Match', '*');
    Req.Executar();

    case Req.StatusCode of
      HTTP_STATUS_OK:
      begin  
        Self.Sincronizar(Self.NotificarDelete);
      end;

      HTTP_STATUS_FORBIDDEN: raise EPicasaAuthFailed.Create('Falha no Auth ao apagar Foto: ' +
                                                            Req.LerRetornoStr());

      else
        raise EPicasa.Create('Erro desconhecido ao apagar foto: ' + Req.LerRetornoStr);
    end;
  finally
    Req.Free;
  end;
end;

{ TThreadAlterarFoto }

procedure TThreadAlterarFoto.NotificarException;
begin
  inherited;

  if Assigned(FPicasa.OnUpdateFotoException) then
    FPicasa.OnUpdateFotoException(FPicasa, FContexto, FException);
end;

procedure TThreadAlterarFoto.NotificarUpdate;
begin
  if Assigned(FPicasa.OnUpdateFoto) then
    FPicasa.OnUpdateFoto(FPicasa, FContexto, FFoto);
end;

procedure TThreadAlterarFoto.ProcReq(const Auth: string);
var
  Req: TReqHTTP;
  XML: IXMLDocument;
  N: IXMLNode;
  Ext, MIME: String;
begin
  inherited;
  CoInitialize(nil);

  try
    Req := TReqHTTP.Create;

    try
      Req.PrepareRequest(FLinkToEdit, 'PUT');
      Req.PrepareGoogleHeader(Auth);
      Req.AddHeader('If-Match', '*');
      Req.SetContentType(CONTENT_TYPE_ATOM);

      MIME := LowerCase(FFoto.MIMEImagem);

      if (MIME = 'image/jpg') or (MIME = 'image/jpeg') then
        Ext := '.jpg'
      else
        if MIME = 'image/png' then
          Ext := '.png'
        else
          if MIME = 'image/gif' then
            Ext := '.gif'
          else
            if MIME = 'image/bmp' then
              Ext := '.bmp';

      if LowerCase(ExtractFileExt(FFoto.Titulo)) <> LowerCase(Ext) then
        FFoto.Titulo := FFoto.Titulo + Ext;

      XML := NewXMLDocument();
      XML.Encoding := 'UTF-8';

      with XML.AddChild('entry', 'http://www.w3.org/2005/Atom') do
      begin
        AddChild('title').Text := String(UTF8Encode(FFoto.Titulo));
        AddChild('summary').Text := String(UTF8Encode(FFoto.Descricao));

        with AddChild('category') do
        begin
          Attributes['scheme'] := 'http://schemas.google.com/g/2005#kind';
          Attributes['term'] := 'http://schemas.google.com/photos/2007#photo';
        end;
      end;

      Req.Executar(AnsiString(XML.XML.Text));

      case Req.StatusCode of
        HTTP_STATUS_OK:
        begin
          XML := Req.LerRetornoXML;

          N := XML.ChildNodes.FindNode('entry');

          if N <> nil then
            FFoto := Self.LerFoto(N);

          Self.Sincronizar(Self.NotificarUpdate);
        end;

        HTTP_STATUS_FORBIDDEN: raise EPicasaAuthFailed.Create('AuthFailed ao alterar foto: ' + Req.LerRetornoStr);

        else
          raise EPicasa.Create('Erro desconhecido ao alterar foto: ' + Req.LerRetornoStr);
      end;
    finally
      Req.Free;
    end;
  finally
    CoUninitialize;
  end;
end;

{ TPicasaLoginHandler }

function TPicasaLoginHandler.GetAuth: String;
begin
  Result := '';
end;

function TPicasaLoginHandler.GetUsuarioESenha(var Usuario,
  Senha: String): Boolean;
begin
  Result := False;
  Usuario := '';
  Senha := '';
end;

procedure TPicasaLoginHandler.SetAuth(const Auth: String);
begin

end;

end.

