unit RRPGRichEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics,
  ComCtrls, Forms, StdCtrls, ExtCtrls, CommCtrl, RichEdit, RichOle,
  ShellApi, Menus, ActiveX, TextObjectModel;

type
  TRichMenuItem = class(TMenuItem)
  private
    FStrSug: String;
  end;

  TOnPasteEvent = procedure (Sender: TObject; var Aceitar: Boolean) of object;

  TRCustomRichEdit = class(TRichEdit, IRichEditOleCallback)
  private
    FSpellCheck: Boolean;
    FAutoDetectURL: Boolean;
    FSingleLine: Boolean;
    FDCToDraw: HDC;
    FCanvasToDraw: TCanvas;
    FTimerPopup: TTimer;
    FTimerCheckSpell: TTimer;
    FTickUltimoSelChange: Cardinal;
    FTimerSpellChecked: Boolean;
    FIgnorarCheckSpellSelectedWord: Boolean;
    FOnPasteEvent: TOnPasteEvent;
    FNoRTF: Boolean;
    procedure SetSpellCheck(const Value: Boolean);
    procedure SetAutoDetectURL(const Value: Boolean);
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
    procedure SetSingleLine(const Value: Boolean);
  protected
    FExecutarPopup: Boolean;
    FPopupMenuRichEdit: TPopupMenu;
    FInstancieiPopupMenu: Boolean;
    FCharRangePopupMenu: TCharRange;
    FPalavraSelecionadaCorretor: String;
    FCharRangeSugestao: TCharRange;
    FSelecaoAtual: TCharRange;
    FSelecaoAtualWord: TCharRange;
    FSelecaoToCheckSpell: TCharRange;
    FIRichOle: IRichEditOle;
    FTextDocument: ITextDocument;

    FILock: ILockBytes;
    FIStorage: IStorage;
    FProcessandoPosWordChange: Boolean;

    // Flags
    FIgnorarSelChange: Boolean;
    FInSelChange: Boolean;
    FInPaint: Boolean;

    FMemRecriando: TMemoryStream;
    FRecriandoSelStart: Integer;
    FRecriandoSelLength: Integer;


    procedure CreateWnd; override;
    procedure DestroyWnd; override;

    procedure ConfigurarAutoDetectURL; virtual;
    procedure ConfigurarSingleLine; virtual;

    procedure CreateParams(var Params: TCreateParams); override;

    procedure RealizarSpellCheck(Range: TCharRange); virtual;
    procedure Change; override;
    procedure DoENSelChange(NovoRange: TCharRange; const ForceUpdateWordPosition: Boolean = False); virtual;
    procedure DoPalavraErrada(Range: TCharRange); virtual;
    function DoObjectPositions(NMHdr: PNMHdr): Integer; virtual;

    procedure OnTimerPopup(Sender: TObject); virtual;
    procedure OnTimerCheckSpell(Sender: TObject); virtual;
    procedure PreparePopupMenuRichEdit(P: TPopupMenu); virtual;
    procedure AddPalavraClick(Sender: TObject); virtual;
    procedure SugestaoClick(Sender: TObject); virtual;
    function GetClientRectForRange(Range: TCharRange): TRect; virtual;
    function DoPosWordChanged(Old, New: TCharRange): Integer; virtual;

    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure Loaded; override;

    // IRichEditOleCallback
    function GetNewStorage(out stg: IStorage): HRESULT; virtual; stdcall;
    function GetInPlaceContext(out Frame: IOleInPlaceFrame;
         out Doc: IOleInPlaceUIWindow; var FrameInfo: TOleInPlaceFrameInfo): HRESULT; virtual; stdcall;
    function ShowContainerUI(fShow: BOOL): HRESULT; virtual; stdcall;
    function QueryInsertObject(const clsid: TCLSID; stg: IStorage; cp: longint): HRESULT; virtual; stdcall;
    function DeleteObject(oleobj: IOLEObject): HRESULT; virtual; stdcall;
    function QueryAcceptData(dataobj: IDataObject; var cfFormat: TClipFormat;
         reco: DWORD; fReally: BOOL; hMetaPict: HGLOBAL): HRESULT; virtual; stdcall;
    function ContextSensitiveHelp(fEnterMode: BOOL): HRESULT; virtual; stdcall;
    function GetClipboardData(const chrg: TCharRange; reco: DWORD;
         out dataobj: IDataObject): HRESULT; virtual; stdcall;
    function GetDragDropEffect(fDrag: BOOL; grfKeyState: DWORD;
         var dwEffect: DWORD): HRESULT; virtual; stdcall;
    function GetContextMenu(seltype: Word; oleobj: IOleObject;
         const chrg: TCharRange; var menu: HMENU): HRESULT; virtual; stdcall;

    function AddRefRichCallback: Integer; stdcall;
    function ReleaseRichCallback: Integer; stdcall;

    function IRichEditOleCallback._AddRef = AddRefRichCallback;
    function IRichEditOleCallback._Release = ReleaseRichCallback;

    procedure Notification(AComponent: TComponent; Operation: TOperation);  override;
    procedure DoExit; override;
    procedure DoEnter; override;
    procedure KeyPress(var Key: Char); override;

    property AutoDetectURL: Boolean read FAutoDetectURL write SetAutoDetectURL default False;
    property SpellCheck: Boolean read FSpellCheck write SetSpellCheck default True;
    property SingleLine: Boolean read FSingleLine write SetSingleLine default False;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure InicializarComponente;

    property Storage: IStorage read FIStorage;

    property RichOle: IRichEditOle read FIRichOle;
    property TextDocument: ITextDocument read FTextDocument;
  published
    property OnPasteEvent: TOnPasteEvent read FOnPasteEvent write FOnPasteEvent;
    property NoRTF: Boolean read FNoRTF write FNoRTF;
  end;

  TRRichEdit = class(TRCustomRichEdit)
  published
    property AutoDetectURL;
    property SpellCheck;
    property SingleLine;
  end;

  TRRichEditSingleLine = class(TRCustomRichEdit)
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AutoDetectURL;
    property SpellCheck;
  end;

const
  UNDERLINE_TYPE_SPELLCHECK = $58;

  CONJUNTO_CARACTERES_SPELLCHECK: set of AnsiChar =  ['a'..'z', 'A'..'Z', '0'..'9', ''..'',
          ''..''];

  CONJUNT_CARACTERES_ESPACAMENTO: set of AnsiChar = [#32, #10, #13, #160];

implementation

uses
  RRPGSpellCheckClient;

{ TRRichEdit }

procedure TRCustomRichEdit.AddPalavraClick(Sender: TObject);
var
  Palavra: String;
begin
  inherited;

  if FPalavraSelecionadaCorretor = '' then
    Exit;

  Palavra := FPalavraSelecionadaCorretor;
  FPalavraSelecionadaCorretor := '';

  RRPGSpellCheckClient.AdicionarPalavra(String(Palavra));

  // Retirar underline das palavras erradas.
  Self.Invalidate;
end;

function TRCustomRichEdit.AddRefRichCallback: Integer;
begin
  Result := -1;
end;

procedure TRCustomRichEdit.Change;
begin
  inherited;
end;

procedure TRCustomRichEdit.CNNotify(var Message: TWMNotify);
  procedure DoLink;
  var
    EL: ^TENLink;
    Link: String;
    Range: TEXTRANGE;
  begin
    EL := Pointer(Message.NMHdr);

    case EL.msg of
      WM_LBUTTONUP:
      begin
        Range.chrg := EL^.chrg;
        SetLength(Link, 1024);
        Range.lpstrText := PChar(Link);
        SendMessage(Self.Handle, EM_GETTEXTRANGE, 0, Integer(@Range));
        Link := Range.lpstrText;

        ShellApi.ShellExecute(0, 'open', PChar(Link), nil, nil, SW_SHOWNORMAL);
      end;
    end;
  end;

  procedure DoOleOpFailed;
  begin

  end;
begin
  inherited;

  case Message.NMHdr.code of
    EN_LINK: DoLink();
    EN_SELCHANGE: Self.DoENSelChange(PSELCHANGE(Message.NMHdr).chrg);
    EN_OBJECTPOSITIONS: Message.Result := Self.DoObjectPositions(Message.NMHdr);
    EN_OLEOPFAILED : DoOleOpFailed();
  end;
end;

procedure TRCustomRichEdit.ConfigurarAutoDetectURL;
var
  EventMask: Cardinal;
begin
  EventMask := SendMessage(Self.Handle, EM_GETEVENTMASK, 0, 0);

  if FAutoDetectURL then
    EventMask := EventMask or ENM_LINK
  else
    EventMask := EventMask and (not ENM_LINK);

  SendMessage(Self.Handle, EM_SETEVENTMASK, 0, EventMask);

  if FAutoDetectURL then
    SendMessage(Self.Handle, EM_AUTOURLDETECT, 1, 0)
  else
    SendMessage(Self.Handle, EM_AUTOURLDETECT, 0, 0);
end;

procedure TRCustomRichEdit.ConfigurarSingleLine;
begin

end;

function TRCustomRichEdit.ContextSensitiveHelp(fEnterMode: BOOL): HRESULT;
begin
  Result := E_NOTIMPL;
end;

constructor TRCustomRichEdit.Create(AOwner: TComponent);
begin
  FCanvasToDraw := TCanvas.Create;
  FSpellCheck := True;
  inherited;
  FTimerPopup := TTimer.Create(nil);
  FTimerPopup.Enabled := False;
  FTimerPopup.Interval := 50;
  FTimerPopup.OnTimer := Self.OnTimerPopup;

  FTimerCheckSpell := TTimer.Create(nil);
  FTimerCheckSpell.Interval := 1000;
  FTimerCheckSpell.Enabled := True;
  FTimerCheckSpell.OnTimer := Self.OnTimerCheckSpell;
end;

procedure TRCustomRichEdit.CreateParams(var Params: TCreateParams);
begin
  inherited;

  if FSingleLine then
//    Params.Style := (Params.Style or ES_MULTILINE) and (not ES_WANTRETURN);
    Params.Style := Params.Style and (not ES_MULTILINE) or ES_AUTOHSCROLL;
end;

procedure TRCustomRichEdit.CreateWnd;
var
  EventMask: Integer;
  RichCallback: IRichEditOleCallback;
  AntigoOnChange: TNotifyEvent;
begin
  AntigoOnChange := Self.OnChange;
  Self.OnChange := nil;

  try
    inherited;
    Self.ConfigurarSingleLine();
    Self.ConfigurarAutoDetectURL();

    EventMask := SendMessage(Self.Handle, EM_GETEVENTMASK, 0, 0);
    EventMask := EventMask or ENM_CHANGE or ENM_SELCHANGE or ENM_OBJECTPOSITIONS;
    SendMessage(Self.Handle, EM_SETEVENTMASK, 0, EventMask);

    RichCallback := Self as IRichEditOleCallback;
    SendMessage(Self.Handle, EM_SETOLECALLBACK, 0, LParam(RichCallback));
    RichCallback := nil;

  //  Self.MaxLength := MaxInt - 2;
    SendMessage(Self.Handle, EM_EXLIMITTEXT, 0, MaxInt - 2);


    if FMemRecriando <> nil then
    begin
      FMemRecriando.Position := 0;
      Self.Lines.LoadFromStream(FMemRecriando);
      Self.SelStart := FRecriandoSelStart;
      Self.SelLength := FRecriandoSelLength;

      FreeAndNil(FMemRecriando);
    end;

    FIRichOle := nil;
    FTextDocument := nil;
    SendMessage(Self.Handle, EM_GETOLEINTERFACE, 0, Integer(@FIRichOle));

    if FIRichOle <> nil then
    begin
      if not Succeeded(FIRichOle.QueryInterface(IID_ITextDocument, FTextDocument)) then
        raise Exception.Create('No foi possvel obter a interface ITextDocument do RichEdit');


    end;
  finally
    Self.OnChange := AntigoOnChange;
  end;
end;

function TRCustomRichEdit.DeleteObject(oleobj: IOLEObject): HRESULT;
begin
  Result := S_OK;
end;

destructor TRCustomRichEdit.Destroy;
begin
  if not (csDesigning in Self.ComponentState) then
    if FInstancieiPopupMenu then
      FreeAndnil(FPopupMenuRichEdit);

  FreeAndNil(FMemRecriando);
  FTimerCheckSpell.Free;
  FTimerPopup.Free;
  FCanvasToDraw.Free;
  inherited;
end;

procedure TRCustomRichEdit.DestroyWnd;
begin
  if (csRecreating in Self.ControlState) and
     (not (csDesigning in Self.ComponentState)) then
  begin
    if FMemRecriando = nil then
      FMemRecriando := TMemoryStream.Create;

    FRecriandoSelStart := Self.SelStart;
    FRecriandoSelLength := Self.SelLength;
    FMemRecriando.Size := 0;
    FMemRecriando.Position := 0;
    Self.Lines.SaveToStream(FMemRecriando);

    Self.Clear;
  end;

  FTextDocument := nil;
  FIRichOle := nil;
  inherited;
end;

procedure TRCustomRichEdit.DoENSelChange(NovoRange: TCharRange;
  const ForceUpdateWordPosition: Boolean = False);
var
  SelWord: TCharRange;
  pLeft, pRight, pLeft2, pRight2, pTemp: Integer;
  Tam, Delta: Integer;
  Texto, Texto2: String;
  Palavra: AnsiString;
  DCToRelease: HDC;
  R: TRect;
  Letra: AnsiChar;

  function QuebrarEmUmaPalavra(S: AnsiString; var RangeRealTexto: TCharRange;
    const AjustarParaTras: Boolean): AnsiString;
  var
    PSel, Inicio, Fim, i: Integer;
    C: AnsiChar;
  begin
    PSel := NovoRange.cpMin - pLeft + 1;
    Inicio := 1;
    Fim := Length(S);

    if Fim = 0 then
    begin
      Result := '';
      Exit;
    end;

    if PSel > Length(S) then
      PSel := Length(S);

    if PSel < 1 then
      PSel := 1;

    //Ajustar PSel

    if AjustarParaTras then
    begin
      for i := PSel downto 1 do
      begin
        C := S[i];

        if C in CONJUNTO_CARACTERES_SPELLCHECK then
        begin
          PSel := i;
          Break
        end;
      end;
    end else
    begin
      for i := PSel to Length(S) do
      begin
        C := S[i];

        if C in CONJUNTO_CARACTERES_SPELLCHECK then
        begin
          PSel := i;
          Break
        end;
      end;
    end;

    // Achar inicio da palavra
    for i := PSel downto 1 do
    begin
      C := S[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
        Inicio := i
      else
        Break;
    end;

    RangeRealTexto.cpMin := pLeft + Inicio - 1;

    // Achar final da palavra
    for i := PSel to Length(S) do
    begin
      C := S[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
        Fim := i
      else
        Break;
    end;

    RangeRealTexto.cpMax := pLeft + Fim;
    Result := Copy(S, Inicio, Fim - Inicio + 1);
  end;

  function GetTexto(cpMin, cpMax: Integer): String;
  var
    TR: TTextRangeW;
  begin
    TR.chrg.cpMin := cpMin;
    TR.chrg.cpMax := cpMax;
    SetLength(Result, cpMax - cpMin);

    TR.lpstrText := PChar(Result);
    Tam := SendMessage(Self.Handle, EM_GETTEXTRANGE, 0, Integer(@TR));
    SetLength(Result, Tam);
  end;
begin
  if not (FSpellCheck and RRPGSpellCheckClient.IsSpellCheckAtivo()) then
    Exit;

  if FIgnorarSelChange then
    Exit;

  if FInSelChange then
    Exit;

  FInSelChange := True;

  try
    pLeft := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_LEFT, NovoRange.cpMin);
    pRight := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, NovoRange.cpMin + 1);
    Texto := GetTexto(pLeft, pRight);

    if (Trim(Texto) = '') and (NovoRange.cpMin > 0) then
    begin
      // Talvez a seleo esteja no final da palavra: "Palavra|"
      pLeft2 := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_LEFT, NovoRange.cpMin - 1);
      pRight2 := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, pLeft2 + 1);

      Texto2 := GetTexto(pLeft2, pRight2);

      if Trim(Texto2) <> '' then
      begin
        if (NovoRange.cpMax - NovoRange.cpMin) <= 1 then
        begin
          pTemp := NovoRange.cpMin - pLeft2;

          if (pTemp > 0) and (pTemp <= Length(Texto2)) then
          begin
            Letra := AnsiChar(Texto2[pTemp]);

            if not (Letra in CONJUNTO_CARACTERES_SPELLCHECK) and
               not (Letra in CONJUNT_CARACTERES_ESPACAMENTO) then
            begin
              pLeft2 := pLeft2 + pTemp - 1;
              //pRight2 := pLeft2 + 1;
              Texto2 := Char(Letra);
            end;
          end;
        end;

        pLeft := pLeft2;
        //pRight := pRight2;
        Texto := Texto2;
      end else
      begin
        // Talvez a seleo esteja no inio da palavra: "|Palavra"
      // Talvez a seleo esteja no final da palavra: "Palavra|"
        pLeft2 := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_LEFT, NovoRange.cpMin + 1);
        pRight2 := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, pLeft2 + 1);

        Texto2 := GetTexto(pLeft2, pRight2);

        if Texto2 <> '' then
        begin
          pLeft := pLeft2;
          //pRight := pRight2;
          Texto := Texto2;
        end;
      end;
    end;

    SelWord.cpMin := NovoRange.cpMin;
    SelWord.cpMax := NovoRange.cpMax;
    Palavra := QuebrarEmUmaPalavra(AnsiString(Texto), SelWord, True);

    if Palavra = '' then
      Palavra := QuebrarEmUmaPalavra(AnsiString(Texto), SelWord, False);

    if not ForceUpdateWordPosition then
      FSelecaoToCheckSpell := SelWord;

    if ((SelWord.cpMin <> FSelecaoAtualWord.cpMin)) or
       (ForceUpdateWordPosition) or (FTimerSpellChecked) then
    begin
      Delta := Self.DoPosWordChanged(FSelecaoAtualWord, SelWord);
      SelWord.cpMax := SelWord.cpMax + Delta;
      NovoRange.cpMin := NovoRange.cpMin + Delta;
      NovoRange.cpMax := NovoRange.cpMax + Delta;

      if not ForceUpdateWordPosition then
      begin
        FSelecaoToCheckSpell.cpMin := FSelecaoToCheckSpell.cpMin + Delta;
        FSelecaoToCheckSpell.cpMax := FSelecaoToCheckSpell.cpMax + Delta;
      end;

      DCToRelease := 0;

      try
        if FDCToDraw = 0 then
        begin
          DCToRelease := GetDC(Self.Handle);
          FDCToDraw := DCToRelease;
        end;

        R := Self.GetClientRectForRange(FSelecaoAtualWord);
        RedrawWindow(Self.Handle, @R, 0, RDW_INVALIDATE or RDW_ERASE);
  //      InvalidateRect(Self.Handle, @R, True);

        R := Self.GetClientRectForRange(SelWord);
        RedrawWindow(Self.Handle, @R, 0, RDW_INVALIDATE or RDW_ERASE);
  //      InvalidateRect(Self.Handle, @R, True);

  //      Self.Invalidate;
      finally
        if DCToRelease <> 0 then
        begin
          FDCToDraw := 0;
          ReleaseDC(Self.Handle, DCToRelease);
        end;
      end;
    end;

    if not ForceUpdateWordPosition then
      FSelecaoAtualWord := SelWord;

    if not ForceUpdateWordPosition then
      FSelecaoAtual := NovoRange;

    FTickUltimoSelChange := Windows.GetTickCount;
    FTimerSpellChecked := False;
  finally
    FInSelChange := False;
  end;
end;

procedure TRCustomRichEdit.DoEnter;
begin
  inherited;

end;

procedure TRCustomRichEdit.DoExit;
begin
end;

function TRCustomRichEdit.DoObjectPositions(NMHdr: PNMHdr): Integer;
begin
  Result := 0;
end;

procedure TRCustomRichEdit.DoPalavraErrada(Range: TCharRange);
var
  PtIni, PtFim, Origin: TPoint;
  TextRange: ITextRange;

  procedure DesenharBezier;
  const
    LIN_ALTURA = 2;
    LIN_LARGURA = 3;
  var
    Qt, Resto, i, XSoFar: Integer;
  begin
    PtIni.y := PtIni.y - 1;
    FCanvasToDraw.MoveTo(PtIni.x, PtIni.y - LIN_ALTURA);
    XSoFar := PtIni.x;

    Qt := (PtFim.x - PtIni.x) div LIN_LARGURA;
    Resto := (PtFim.x - PtIni.x) - (Qt * LIN_LARGURA);

    for i := 1 to Qt do
    begin
      XSoFar := XSoFar + LIN_LARGURA;

      if i mod 2 = 1 then
        FCanvasToDraw.LineTo(XSoFar, PtIni.y)
      else
        FCanvasToDraw.LineTo(XSoFar, PtIni.y - LIN_ALTURA);
    end;

    if Resto > 0 then
      FCanvasToDraw.LineTo(PtFim.x, PtFim.y);;
  end;
begin
  if FDCToDraw = 0 then
  begin
    FDCToDraw := Windows.GetDC(Self.Handle);
    FCanvasToDraw.Handle := FDCToDraw;
  end;

  TextRange := Self.TextDocument.Range(Range.cpMin, Range.cpMax);

  TextRange.GetPoint(tomStart or TA_LEFT or TA_BOTTOM, PtIni.x, PtIni.y);
  TextRange.GetPoint(tomEnd or TA_LEFT or TA_BOTTOM, PtFim.x, PtFim.y);

  Origin := Self.ClientOrigin;
  PtIni.X := PtIni.X - Origin.X;
  PtIni.Y := PtIni.Y - Origin.Y;

  PtFim.X := PtFim.X - Origin.X;
  PtFim.Y := PtFim.Y - Origin.Y;

//  PtIni := Self.ScreenToClient(PtIni);
//  PtFim := Self.ScreenToClient(PtFim);

  FCanvasToDraw.Pen.Style := psSolid;
  FCanvasToDraw.Pen.Color := clRed;

  DesenharBezier();
end;

function TRCustomRichEdit.DoPosWordChanged(Old, New: TCharRange): Integer;
var
  Palavra, SubstituirPor: String;
  TR: TTextRangeW;
  PosAtual: TCharRange;
  Delta: Integer;
begin
  Result := 0;

  if FProcessandoPosWordChange then
    Exit;

  FProcessandoPosWordChange := True;

  try
    if RRPGSpellCheckClient.IsAutoCorrAtivo then
    begin
      SubstituirPor := '';

      TR.chrg := Old;
      SetLength(Palavra, Old.cpMax - Old.cpMin);
      TR.lpstrText := PChar(Palavra);

      SendMessage(Self.Handle, EM_GETTEXTRANGE, 0, Integer(@TR));

      if RRPGSpellCheckClient.TryAutoCorr(Palavra, SubstituirPor) then
        if SubstituirPor <> '' then
        begin
          FIgnorarSelChange := True;

          try
            SendMessage(Self.Handle, EM_EXGETSEL, 0, Integer(@PosAtual));
            SendMessage(Self.Handle, EM_EXSETSEL, 0, Integer(@Old));
            Self.SelText := SubstituirPor;

            Delta := Length(SubstituirPor) - Length(Palavra);

            PosAtual.cpMin := PosAtual.cpMin + Delta;
            PosAtual.cpMax := PosAtual.cpMax + Delta;
            SendMessage(Self.Handle, EM_EXSETSEL, 0, Integer(@PosAtual));

            Result := Delta;
          finally
            FIgnorarSelChange := False;
          end;
      end;
    end;
  finally
    FProcessandoPosWordChange := False;
  end;
end;

function TRCustomRichEdit.GetClientRectForRange(Range: TCharRange): TRect;
var
  PtIni, PtFim: TPointL;
  FR: TFormatRange;
  DCToRelease: HDC;
begin
  FillChar(FR, SizeOf(FR), 0);
  DCToRelease := 0;

  try
    if FDCToDraw = 0 then
    begin
      DCToRelease := GetDC(Self.Handle);

      FR.hdc := DCToRelease;
      FR.hdcTarget := DCToRelease;
    end else
    begin
      FR.hdc := FDCToDraw;
      FR.hdcTarget := FDCToDraw;
    end;

    FR.chrg := Range;

    SendMessage(Self.Handle, EM_FORMATRANGE, 0, Integer(@FR));
    FR.rc.Bottom := FR.rc.Bottom div 15;

    SendMessage(Self.Handle, EM_POSFROMCHAR, Integer(@PtIni), Range.cpMin);
    SendMessage(Self.Handle, EM_POSFROMCHAR, Integer(@PtFim), Range.cpMax);

//    PtIni.y := PtIni.y + FR.rc.Bottom {- 1};
//    PtFim.y := PtFim.y + FR.rc.Bottom {- 1};

    Result := Rect(PtIni.x, PtIni.y, PtFim.x, PtFim.y + FR.rc.Bottom);
  finally
    if DCToRelease <> 0 then
      ReleaseDC(Self.Handle, DCToRelease);
  end;
end;

function TRCustomRichEdit.GetClipboardData(const chrg: TCharRange; reco: DWORD;
  out dataobj: IDataObject): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TRCustomRichEdit.GetContextMenu(seltype: Word; oleobj: IOleObject;
  const chrg: TCharRange; var menu: HMENU): HRESULT;
begin
  if (FCharRangePopupMenu.cpMin = 0) and (FCharRangePopupMenu.cpMax = 0) then
    FCharRangePopupMenu := chrg;

  Menu := 0;
  FExecutarPopup := True;
  FTimerPopup.Enabled := True;
  Result := S_OK;
end;

function TRCustomRichEdit.GetDragDropEffect(fDrag: BOOL; grfKeyState: DWORD;
  var dwEffect: DWORD): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TRCustomRichEdit.GetInPlaceContext(out Frame: IOleInPlaceFrame;
  out Doc: IOleInPlaceUIWindow; var FrameInfo: TOleInPlaceFrameInfo): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TRCustomRichEdit.GetNewStorage(out stg: IStorage): HRESULT;
begin
  Stg := FIStorage;
  Result := S_OK;

  {Result := StgCreateDocfileOnILockBytes(FILock,
    STGM_SHARE_EXCLUSIVE or STGM_CREATE or STGM_READWRITE,
    0, stg);}
end;

procedure TRCustomRichEdit.InicializarComponente;
begin
  if not (csDesigning in Self.ComponentState) then
  begin
    FPopupMenuRichEdit := Self.PopupMenu;
    Self.PopupMenu := nil;

    if FPopupMenuRichEdit = nil then
    begin
      FPopupMenuRichEdit := TPopupMenu.Create(nil);
      FInstancieiPopupMenu := True;
    end else
      FPopupMenuRichEdit.FreeNotification(Self);
  end;

  CreateILockBytesOnHGlobal(0, True, FILock);

  StgCreateDocfileOnILockBytes(FILock,
    STGM_SHARE_EXCLUSIVE or STGM_CREATE or STGM_READWRITE,
    0, FIStorage);
end;

procedure TRCustomRichEdit.KeyPress(var Key: Char);
begin
  if Key = #9 then  // TAB.. O maldito atalho Ctrl+I Envia TAB!
    if GetKeyState(VK_CONTROL) < 0 then
    begin
      Key := #0;
      Exit;
    end;

  inherited;
end;

procedure TRCustomRichEdit.Loaded;
begin
  inherited;
  Self.InicializarComponente();
end;

procedure TRCustomRichEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if (Operation = opRemove) and (AComponent = FPopupMenuRichEdit) then
  begin
    FPopupMenuRichEdit := nil;
    FInstancieiPopupMenu := False;
  end;
end;

procedure TRCustomRichEdit.OnTimerCheckSpell(Sender: TObject);
const
  TIMEOUT_SPELL = 2000;
var
  DCToRelease: HDC;
  R: TRect;
begin
  if FTimerSpellChecked then
    Exit;

  if not Self.HandleAllocated then
    Exit;

  if (FTickUltimoSelChange = 0) or (Windows.GetTickCount - FTickUltimoSelChange >= TIMEOUT_SPELL) then
  begin
    FTimerSpellChecked := True;

    DCToRelease := 0;

    try
      if FDCToDraw = 0 then
      begin
        DCToRelease := GetDC(Self.Handle);
        FDCToDraw := DCToRelease;
      end;

      R := Self.GetClientRectForRange(FSelecaoAtualWord);
    finally
      if DCToRelease <> 0 then
      begin
        FDCToDraw := 0;
        ReleaseDC(Self.Handle, DCToRelease);
      end;
    end;

    FIgnorarCheckSpellSelectedWord := True;

    try
      RedrawWindow(Self.Handle, @R, 0, RDW_ERASE or RDW_INVALIDATE or RDW_UPDATENOW);
    finally
      FIgnorarCheckSpellSelectedWord := False;
    end;
  end;
end;

procedure TRCustomRichEdit.OnTimerPopup(Sender: TObject);
var
  P: TPopupMenu;
  Pt: TPointL;
  Range: TCharRange;
  SP: TPoint;
begin
  FTimerPopup.Enabled := False;

  if not FExecutarPopup then
    Exit;

  FExecutarPopup := False;

  if FPopupMenuRichEdit <> nil then
  begin
    P := FPopupMenuRichEdit;

    if (FCharRangePopupMenu.cpMin <> 0) and (FCharRangePopupMenu.cpMax <> 0) then
      Range := FCharRangePopupMenu
    else
      SendMessage(Self.Handle, EM_EXGETSEL, 0, Integer(@Range));

    SendMessage(Self.Handle, EM_POSFROMCHAR, Integer(@Pt), Range.cpMin);

    SP := Self.ClientOrigin;
    SP.X := SP.X + Pt.x;
    SP.Y := SP.Y + Pt.y;

    if (FCharRangePopupMenu.cpMin <> 0) and (FCharRangePopupMenu.cpMax <> 0) then
      SendMessage(Self.Handle, EM_EXSETSEL, 0, Integer(@FCharRangePopupMenu));

    Self.PreparePopupMenuRichEdit(P);
    P.Popup(SP.X, SP.Y);
    FCharRangePopupMenu.cpMin := 0;
    FCharRangePopupMenu.cpMax := 0;
  end;
end;

procedure TRCustomRichEdit.PreparePopupMenuRichEdit(P: TPopupMenu);
var
  i: Integer;
  Range, RangeOriginal, RangeRealTexto: TCharRange;
  F: TCharFormat2;
  Texto: String;
  Palavra: AnsiString;
  pLeft, pRight, Tam: Integer;
  TR: TTextRangeW;
  Sug: TArrString;
  Mit: TMenuItem;

  function QuebrarEmUmaPalavra(S: AnsiString; var RangeRealTexto: TCharRange): AnsiString;
  var
    PSel, Inicio, Fim, i: Integer;
    C: AnsiChar;
  begin
    PSel := RangeOriginal.cpMin - pLeft + 1;
    Inicio := 1;
    Fim := Length(S);

    if Fim = 0 then
    begin
      Result := '';
      Exit;
    end;

    //Ajustar PSel
    for i := PSel downto 1 do
    begin
      C := S[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
      begin
        PSel := i;
        Break
      end;
    end;

    // Achar inicio da palavra
    for i := PSel downto 1 do
    begin
      C := S[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
        Inicio := i
      else
        Break;
    end;

    RangeRealTexto.cpMin := pLeft + Inicio - 1;

    // Achar final da palavra
    for i := PSel to Length(S) do
    begin
      C := S[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
        Fim := i
      else
        Break;
    end;

    RangeRealTexto.cpMax := pLeft + Fim;
    Result := Copy(S, Inicio, Fim - Inicio + 1);
  end;

const
  TAG_MENU_SUGESTAO = 458965;
  ADD_PALAVRA = 'Adicionar palavra ao dicionrio';
begin
  // Limpar menus antigos

  for i := P.Items.Count - 1 downto 0 do
    if P.Items[i].Tag = TAG_MENU_SUGESTAO then
    begin
      Mit := P.Items[i];
      P.Items.Delete(i);
      Mit.Free;
    end;

  FillChar(F, SizeOf(F), 0);
  F.cbSize := SizeOf(F);
  F.dwMask := CFM_UNDERLINE or CFM_UNDERLINETYPE;

  // Palavra errada

  SendMessage(Self.Handle, EM_EXGETSEL, 0, Integer(@Range));
  RangeOriginal := Range;
  pLeft := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_LEFT, Range.cpMin);
  pRight := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, Range.cpMin);

  Range.cpMin := pLeft;
  Range.cpMax := pRight;
  SetLength(Texto, pRight - pLeft);

  TR.chrg := Range;
  TR.lpstrText := PChar(Texto);
  Tam := SendMessage(Self.Handle, EM_GETTEXTRANGE, 0, Integer(@TR));
  SetLength(Texto, Tam);
  Palavra := AnsiString(Trim(Texto));
  Palavra := QuebrarEmUmaPalavra(Palavra, RangeRealTexto);
  FPalavraSelecionadaCorretor := String(Palavra);

  if not RRPGSpellCheckClient.VerificarPalavra(String(Palavra)) then
  begin
    Sug := RRPGSpellCheckClient.ObterSugestoes(String(Palavra));

    if Length(Sug) = 0 then
    begin
      Mit := TMenuItem.Create(Self);
      Mit.Caption := '(sem sugestes)';
      Mit.Enabled := False;
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(0, Mit);

      Mit := TMenuItem.Create(Self);
      Mit.Caption := '-';
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(1, Mit);

      Mit := TMenuItem.Create(Self);
      Mit.Caption := ADD_PALAVRA;
      Mit.OnClick := Self.AddPalavraClick;
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(2, Mit);

      Mit := TMenuItem.Create(Self);
      Mit.Caption := '-';
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(3, Mit);
    end else
    begin
      FCharRangeSugestao := RangeRealTexto;

      for i := 0 to Length(Sug) - 1 do
      begin
        Mit := TRichMenuItem.Create(Self);
        TRichMenuItem(Mit).FStrSug := String(Sug[i]);
        Mit.Caption := String(Sug[i]);
        Mit.Tag := TAG_MENU_SUGESTAO;
        Mit.OnClick := Self.SugestaoClick;

        P.Items.Insert(i, Mit);
      end;

      Mit := TMenuItem.Create(Self);
      Mit.Caption := '-';
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(Length(Sug), Mit);

      Mit := TMenuItem.Create(Self);
      Mit.Caption := ADD_PALAVRA;
      Mit.OnClick := Self.AddPalavraClick;
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(Length(Sug) + 1, Mit);

      Mit := TMenuItem.Create(Self);
      Mit.Caption := '-';
      Mit.Tag := TAG_MENU_SUGESTAO;
      P.Items.Insert(Length(Sug) + 2, Mit);
    end;
  end;
end;

function TRCustomRichEdit.QueryAcceptData(dataobj: IDataObject;
  var cfFormat: TClipFormat; reco: DWORD; fReally: BOOL;
  hMetaPict: HGLOBAL): HRESULT;
var
  Aceitar: Boolean;
begin
  if (reco = RECO_PASTE) and (fReally) then
  begin
    Aceitar := True;

    if Assigned(FOnPasteEvent) then
      FOnPasteEvent(Self, Aceitar);

    if FNoRTF then
      cfFormat := CF_TEXT;

    if Aceitar then
      Result := S_OK
    else
      Result := S_FALSE;
  end else
    Result := E_NOTIMPL;
end;

function TRCustomRichEdit.QueryInsertObject(const clsid: TCLSID; stg: IStorage;
  cp: Integer): HRESULT;
begin
  Result := S_OK;
end;

procedure TRCustomRichEdit.SetAutoDetectURL(const Value: Boolean);
begin
  if FAutoDetectURL <> Value then
  begin
    FAutoDetectURL := Value;

    if Self.HandleAllocated then
      Self.ConfigurarAutoDetectURL();
  end;
end;

procedure TRCustomRichEdit.SetSingleLine(const Value: Boolean);
begin
  if FSingleLine <> Value then
  begin
    FSingleLine := Value;

    if Self.HandleAllocated then
      Self.RecreateWnd();
  end;
end;

procedure TRCustomRichEdit.SetSpellCheck(const Value: Boolean);
begin
  FSpellCheck := Value;
end;

function TRCustomRichEdit.ShowContainerUI(fShow: BOOL): HRESULT;
begin
  Result := E_NOTIMPL;
end;

procedure TRCustomRichEdit.SugestaoClick(Sender: TObject);
begin
  if not (Sender is TRichMenuItem) then
    Exit;

  SendMessage(Self.Handle, EM_EXSETSEL, 0, Integer(@FCharRangeSugestao));
  Self.SelText := TRichMenuItem(Sender).FStrSug;
end;

procedure TRCustomRichEdit.WMKeyDown(var Message: TWMKeyDown);
begin
  if FSingleLine then
    if Message.CharCode = VK_RETURN then
    begin
      Message.Result := 0;
      Exit;
    end;

  inherited;
end;

procedure TRCustomRichEdit.WMPaint(var Message: TWMPaint);
var
  Range: TCharRange;
  R: TRect;
  Pt: TPointL;
  PaintS: TPaintStruct;
begin
  if FInPaint then
  begin
    Message.Result := 0;
    BeginPaint(Self.Handle, PaintS);
    EndPaint(Self.Handle, PaintS);
    Exit;
  end;

  FInPaint := True;

  try
    Windows.GetUpdateRect(Self.Handle, R, False);
    inherited;

    if (R.Right - R.Left = 0) or (R.Bottom - R.Top = 0) then
      Exit;

    Pt.x := R.Left;
    Pt.y := R.Top;
    Range.cpMin := SendMessage(Self.Handle, EM_CHARFROMPOS, 0, Integer(@Pt));

    Pt.x := R.Right;
    Pt.y := R.Bottom;
    Range.cpMax := SendMessage(Self.Handle, EM_CHARFROMPOS, 0, Integer(@Pt));

    if FSpellCheck and RRPGSpellCheckClient.IsSpellCheckAtivo() then
    begin
      FDCToDraw := 0;

      try
        Self.RealizarSpellCheck(Range);
      finally
        if FDCToDraw <> 0 then
        begin
          SendMessage(Self.Handle, EM_FORMATRANGE, 0, 0);

          FCanvasToDraw.Handle := 0;
          Windows.ReleaseDC(Self.Handle, FDCToDraw);
          FDCToDraw := 0;
        end;
      end;
    end;
  finally
    FInPaint := False;
  end;
end;

procedure TRCustomRichEdit.WMRButtonDown(var Message: TWMRButtonDown);
var
  P: TPointL;
  CR: Integer;
begin
  inherited;
  P.x := Message.XPos;
  P.y := Message.YPos;
  CR := SendMessage(Self.Handle, EM_CHARFROMPOS, 0, Integer(@P));

  FCharRangePopupMenu.cpMin := CR;
  FCharRangePopupMenu.cpMax := CR;
end;

procedure TRCustomRichEdit.RealizarSpellCheck(Range: TCharRange);
var
  PosAchada, PosAchada2, Tam, Processado: Integer;
  TR: TTextRangeW;
  Texto: AnsiString;
  TextoGet: String;

  function ProcessarTexto: Integer;
  var
    i, Ini, QtChars: Integer;
    C: AnsiChar;

    procedure ProcessarPalavra;
    var
      Palavra: AnsiString;
      Range: TCharRange;
    begin
      Palavra := Copy(Texto, Ini, QtChars);

      Range.cpMin := PosAchada + Ini - 1;
      Range.cpMax := Range.cpMin + QtChars;

      if not FIgnorarCheckSpellSelectedWord then
//        if (FSelecaoToCheckSpell.cpMin >= Range.cpMin) and (FSelecaoToCheckSpell.cpMin <= Range.cpMax) then
        if ((FSelecaoToCheckSpell.cpMin <= Range.cpMin) and (FSelecaoToCheckSpell.cpMax >= Range.cpMax)) or
           ((Range.cpMin <= FSelecaoToCheckSpell.cpMin) and (Range.cpMax >= FSelecaoToCheckSpell.cpMax)) then
          Exit;

      if not RRPGSpellCheckClient.VerificarPalavra(String(Palavra)) then
        Self.DoPalavraErrada(Range);
    end;
  begin
    //Result := 0;
    Ini := -1;
    QtChars := 0;

    for i := 1 to Length(Texto) do
    begin
      C := Texto[i];

      if C in CONJUNTO_CARACTERES_SPELLCHECK then
      begin
        if Ini = -1 then
        begin
          Ini := i;
          QtChars := 0;
        end;

        Inc(QtChars);
      end else
      begin
        if Ini <> -1 then
        begin
          ProcessarPalavra();

          Result := Ini + QtChars - 1;
          Exit;

//          Ini := -1;
//          QtChars := 0;

          Break;
        end;
      end;
    end;

    if Ini <> -1 then
    begin
      ProcessarPalavra();
      Result := Ini + QtChars - 1;
    end else
      Result := Length(Texto);
  end;
begin
  //PosAchada := Range.cpMin;
  PosAchada := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_LEFT, Range.cpMin);
  Range.cpMax := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, Range.cpMax);

  while True do
  begin
    if PosAchada < 0 then
      Break;

    PosAchada2 := SendMessage(Self.Handle, EM_FINDWORDBREAK, WB_MOVEWORDRIGHT, PosAchada + 1);

    if Range.cpMax <> -1 then
      if PosAchada2 > Range.cpMax then
        Break;

    if PosAchada2 <= 0 then
      Break;

    if PosAchada = PosAchada2 then
      Break;

    SetLength(TextoGet, PosAchada2 - PosAchada);
    TR.chrg.cpMin := PosAchada;
    TR.chrg.cpMax := PosAchada2;
    TR.lpstrText := PChar(TextoGet);
    Tam := SendMessage(Self.Handle, EM_GETTEXTRANGE, 0, Integer(@TR));
    SetLength(TextoGet, Tam);
    Texto := AnsiString(TextoGet);

    Processado := ProcessarTexto();

    if Processado <= 0 then
      PosAchada := PosAchada2
    else
      PosAchada := PosAchada + Processado;
  end;
end;

function TRCustomRichEdit.ReleaseRichCallback: Integer;
begin
  Result := -1;
end;

{ TRRichEditSingleLine }

constructor TRRichEditSingleLine.Create(AOwner: TComponent);
begin
  FSingleLine := False;
  inherited;

  if not (csLoading in Self.ComponentState) then
  begin
    if csDesigning in Self.ComponentState then
    begin
      Self.Width := 121;
      Self.Height := 21;
    end;

//    Self.SingleLine := True;
  end;
end;

procedure TRRichEditSingleLine.Loaded;
begin
  inherited;
  Self.SingleLine := True;
end;

end.
