unit RRPGSpellCheckClient;

interface

uses
  Windows;

type
  TArrString = array of String;

function VerificarPalavra(Palavra: String): Boolean;
function ObterSugestoes(const Palavra: String): TArrString;
procedure AdicionarPalavra(const Palavra: String);
function IsSpellCheckAtivo: Boolean;

function IsAutoCorrAtivo: Boolean;
function TryAutoCorr(Palavra: String; var SubstituirPor: String): Boolean;
procedure GerenciarAutoCorr;

implementation

var
  FuncVerificarPalavra: function (Palavra: PWideChar): Boolean; stdcall;
  FuncObterSugestoes: function (Palavra: PWideChar): Pointer; stdcall;
  FuncSugestoesCount: function (Sugestoes: Pointer): Integer; stdcall;
  FuncSugestaoGetLength: function (Sugestoes: Pointer; const Indice: Integer): Integer; stdcall;
  FuncSugestaoGet: function (Sugestoes: Pointer; const Indice: Integer; StrDest: PWideChar; Length: Integer): Integer; stdcall;
  FuncFreeSugestoes: procedure (Sugestoes: Pointer); stdcall;
  FuncAdicionarPalavra: procedure (Palavra: PWideChar); stdcall;
  FuncIsSpellCheckAtivo: function (): Boolean; stdcall;

  FuncIsAutoCorrectAtivo: function (): Boolean; stdcall;
  FuncGetAutoCorrecaoFor: function (Palavra: PWideChar): Pointer; stdcall;
  FuncGetAutoCorrLength: function (AutoCorr: Pointer): Integer; stdcall;
  FuncGetAutoCorrTxt: function (AutoCorr: Pointer; StrDest: PWideChar; Length: Integer): Integer; stdcall;
  FuncFreeAutoCorr: procedure (AutoCorr: Pointer); stdcall;
  FuncGerenciarAutoCorr: procedure (ParentWnd: HWND) ;stdcall;

procedure GerenciarAutoCorr;
begin
  if Assigned(FuncGerenciarAutoCorr) then
    FuncGerenciarAutoCorr(0);
end;

function TryAutoCorr(Palavra: String; var SubstituirPor: String): Boolean;
var
  AC: Pointer;
  Txt: WideString;
begin
  if Assigned(FuncGetAutoCorrecaoFor) and Assigned(FuncGetAutoCorrLength) and
     Assigned(FuncGetAutoCorrTxt) and Assigned(FuncFreeAutoCorr) then
  begin
    AC := FuncGetAutoCorrecaoFor(PWideChar(Palavra));

    if AC <> nil then
      try
        Result := True;

        SetLength(Txt, FuncGetAutoCorrLength(AC));
        FuncGetAutoCorrTxt(AC, PWideChar(Txt), Length(Txt));
        SubstituirPor := String(Txt);
      finally
        FuncFreeAutoCorr(AC)
      end
    else
    begin
      Result := False;
      SubstituirPor := '';
    end;
  end else
  begin
    SubstituirPor := '';
    Result := False;
  end;
end;

function IsSpellCheckAtivo: Boolean;
begin
  if Assigned(FuncIsSpellCheckAtivo) then
    Result := FuncIsSpellCheckAtivo()
  else
    Result := False;
end;

function IsAutoCorrAtivo: Boolean;
begin
  if Assigned(FuncIsAutoCorrectAtivo) then
    Result := FuncIsAutoCorrectAtivo()
  else
    Result := False;
end;

function VerificarPalavra(Palavra: String): Boolean;
begin
  if not Assigned(FuncVerificarPalavra) then
    Result := True
  else
    Result := FuncVerificarPalavra(PWideChar(WideString(Palavra)));
end;

function ObterSugestoes(const Palavra: String): TArrString;
var
  Sug: Pointer;
  i, Ct, Len: Integer;
  Temp: WideString;
begin
  if Assigned(FuncObterSugestoes) and Assigned(FuncSugestoesCount) and
     Assigned(FuncSugestaoGetLength) and Assigned(FuncSugestaoGet) and
     Assigned(FuncFreeSugestoes) then
  begin
    Sug := FuncObterSugestoes(PWideChar(WideString(Palavra)));

    try
      Ct := FuncSugestoesCount(Sug);
      SetLength(Result, Ct);

      for i := 0 to Ct - 1 do
      begin
        Len := FuncSugestaoGetLength(Sug, i);
        SetLength(Temp, Len);
        FuncSugestaoGet(Sug, i, PWideChar(Temp), Len);
        Result[i] := String(Temp);
      end;
    finally
      FuncFreeSugestoes(Sug);
    end;
  end else
    SetLength(Result, 0);
end;

procedure AdicionarPalavra(const Palavra: String);
begin
  if Assigned(FuncAdicionarPalavra) then
    FuncAdicionarPalavra(PWideChar(WideString(Palavra)))
end;

procedure CarregarFuncoesSpellCheck;
var
  HLib: Cardinal;
begin
  HLib := Windows.GetModuleHandle(nil);
  @FuncVerificarPalavra := GetProcAddress(HLib, 'SpellCheck_VerificarPalavra');
  @FuncObterSugestoes := GetProcAddress(HLib, 'SpellCheck_ObterSugestoes');
  @FuncSugestoesCount := GetProcAddress(HLib, 'SpellCheck_SugestoesCount');
  @FuncSugestaoGetLength := GetProcAddress(HLib, 'SpellCheck_SugestaoGetLength');
  @FuncSugestaoGet := GetProcAddress(HLib, 'SpellCheck_SugestaoGet');
  @FuncFreeSugestoes := GetProcAddress(HLib, 'SpellCheck_FreeSugestoes');
  @FuncAdicionarPalavra := GetProcAddress(HLib, 'SpellCheck_AdicionarPalavra');
  @FuncIsSpellCheckAtivo := GetProcAddress(HLib, 'SpellCheck_IsSpellCheckAtivo');

  @FuncIsAutoCorrectAtivo := GetProcAddress(HLib, 'AutoCorr_IsAutoCorrectAtivo');
  @FuncGetAutoCorrecaoFor := GetProcAddress(HLib, 'AutoCorr_GetAutorCorrecaoFor');
  @FuncGetAutoCorrLength := GetProcAddress(HLib, 'AutoCorr_GetAutorCorrLength');
  @FuncGetAutoCorrTxt := GetProcAddress(HLib, 'AutoCorr_GetAutorCorrTxt');
  @FuncFreeAutoCorr := GetProcAddress(HLib, 'AutoCorr_FreeAutoCorr');
  @FuncGerenciarAutoCorr := GetProcAddress(HLib, 'AutoCorr_GerenciarAutoCorr');
end;

initialization
  CarregarFuncoesSpellCheck();

finalization
  @FuncVerificarPalavra := nil;
  @FuncObterSugestoes := nil;
  @FuncSugestoesCount := nil;
  @FuncSugestaoGetLength := nil;
  @FuncSugestaoGet := nil;
  @FuncFreeSugestoes := nil;
  @FuncAdicionarPalavra := nil;

  @FuncIsSpellCheckAtivo := nil;
  @FuncIsAutoCorrectAtivo := nil;
  @FuncGetAutoCorrecaoFor := nil;
  @FuncGetAutoCorrLength := nil;
  @FuncGetAutoCorrTxt := nil;
  @FuncFreeAutoCorr := nil;
  @FuncGerenciarAutoCorr := nil;
end.
