unit RRPGStreamUtils;

interface

uses
  RRPGModulo, DB, RRPGDataSetUtil, Variants;

procedure RStWriteString(const S: IRRPGStream; Texto: AnsiString);
function RStReadString(const S: IRRPGStream): AnsiString;

procedure RStWriteField(const S: IRRPGStream; F: TField);
procedure RStReadVar(const S: IRRPGStream; TipoVar: TVarType; var V: Variant);

implementation

procedure RStWriteString(const S: IRRPGStream; Texto: AnsiString);
var
  Len: Integer;
begin
  Len := Length(Texto);
  S.Write(@Len, SizeOf(Len));
  S.Write(@Texto[1], Length(Texto));
end;

function RStReadString(const S: IRRPGStream): AnsiString;
var
  Len: Integer;
begin
  S.Read(@Len, SizeOf(Len));
  SetLength(Result, Len);
  S.Read(@Result[1], Len);
end;

procedure RStWriteField(const S: IRRPGStream; F: TField);
var
  TempByte: Byte;
  TipoVariant: Word;
  Valor: Variant;
  VData: TVarData;

  procedure Write(const V; Tamanho: Integer);
  begin
    S.Write(@V, Tamanho);
  end;
begin
  if F.IsNull then
  begin
    TempByte := 0;
    S.Write(@TempByte, 1);
    Exit;
  end;

  TempByte := 1;
  S.Write(@TempByte, 1);

  if F is TBlobField then
  begin
    RStWriteString(S, AnsiString(TBlobField(F).AsString));
    Exit;
  end;

  TipoVariant := FieldTypeToVariantType(F.DataType);
  Valor := F.AsVariant;
  VData := TVarData(Valor);

  case TipoVariant of
    varSmallint, varWord: Write(VData.VSmallInt, SizeOf(VData.VSmallInt));
    varInteger: Write(VData.VInteger, SizeOf(VData.VInteger));
    varSingle: Write(VData.VSingle, SizeOf(VData.VSingle));
    varDouble: Write(VData.VDouble, SizeOf(VData.VDouble));
    varCurrency: Write(VData.VCurrency, SizeOf(VData.VCurrency));
    varDate: Write(VData.VDate, SizeOf(VData.VDate));
    varBoolean: Write(VData.VBoolean, SizeOf(VData.VBoolean));
    varByte, varShortInt: Write(VData.VByte, SizeOf(VData.VByte));
    varInt64: Write(VData.VInt64, SizeOf(VData.VInt64));
    varString: RStWriteString(S, AnsiString(Valor));
  end;
end;

procedure RStReadVar(const S: IRRPGStream; TipoVar: TVarType; var V: Variant);
var
  TempByte: Byte;
  VData: PVarData;
  TempStr: AnsiString;

  procedure Ler(var B; Tamanho: Integer);
  begin
    S.Read(@B, Tamanho);
  end;
begin
  S.Read(@TempByte, 1);

  if TempByte = 0 then
  begin
    V := Null;
    Exit;
  end;

  VarClear(V);
  VData := PVarData(@V);

  case TipoVar of
    varSmallint, varWord: Ler(VData^.VSmallInt, SizeOf(VData^.VSmallInt));
    varInteger: Ler(VData^.VInteger, SizeOf(VData^.VInteger));
    varSingle: Ler(VData^.VSingle, SizeOf(VData^.VSingle));
    varDouble: Ler(VData^.VDouble, SizeOf(VData^.VDouble));
    varCurrency: Ler(VData^.VCurrency, SizeOf(VData^.VCurrency));
    varDate: Ler(VData^.VDate, SizeOf(VData^.VDate));
    varBoolean: Ler(VData^.VBoolean, SizeOf(VData^.VBoolean));
    varByte, varShortInt: Ler(VData^.VByte, SizeOf(VData^.VByte));
    varInt64: Ler(VData^.VInt64, SizeOf(VData^.VInt64));
    varString:
    begin
      TempStr := RStReadString(S);
      V := TempStr;
    end;
  end;

  if TipoVar <> varString then
    VData^.VType := TipoVar;
end;

end.
