unit RRPGURLGetter;

interface

uses
  Windows, SysUtils, Classes, SyncObjs, URLMon, ActiveX, Messages;

const
  WM_UIG_ERRO = WM_USER + 5000;
  WM_UIG_INICIO = WM_USER + 5001;
  WM_UIG_FIM = WM_USER + 5002;
  WM_UIG_PROGRESSO = WM_USER + 5003;
  WM_UIG_DADOS = WM_USER + 5004;

type
  TURLGetterProgressoEvento = procedure (const iPosicao, iMaximo: Integer) of object;
  TURLGetterDadosEvento = procedure (const Arquivo: String) of object;

  TRRPGDownloadJob = class;

  TThreadURLGetter = class(TThread, IBindStatusCallback, IAuthenticate)
  private
    FDownloadJob: TRRPGDownloadJob;
    FAbortar: Boolean;
    FSecaoCritica: TCriticalSection;
    FTempFileLocalizacao: String;
  protected
    // Interface IAuthenticate
    function Authenticate(var hwnd: HWnd; var szUserName, szPassWord: LPWSTR): HResult; stdcall;    

    // Interface IBindStatusCallback
    function OnStartBinding(dwReserved: DWORD; pib: IBinding): HResult; stdcall;
    function GetPriority(out nPriority): HResult; stdcall;
    function OnLowResource(reserved: DWORD): HResult; stdcall;
    function OnProgress(ulProgress, ulProgressMax, ulStatusCode: ULONG;
      szStatusText: LPCWSTR): HResult; stdcall;
    function OnStopBinding(hresult: HResult; szError: LPCWSTR): HResult; stdcall;
    function GetBindInfo(out grfBINDF: DWORD; var bindinfo: TBindInfo): HResult; stdcall;
    function OnDataAvailable(grfBSCF: DWORD; dwSize: DWORD; formatetc: PFormatEtc;
      stgmed: PStgMedium): HResult; stdcall;
    function OnObjectAvailable(const iid: TGUID; punk: IUnknown): HResult; stdcall;

    function CapturarImagem: Boolean; virtual;

    procedure ProcessarJob; virtual;
    procedure Execute; override;
    procedure NotificarErro; virtual;
    procedure NotificarDados; virtual;
    procedure SafeNotify(Msg: Cardinal; lParam, wParam: Integer);
  public
    constructor Create(ASuspended: Boolean; Job: TRRPGDownloadJob); reintroduce;
    destructor Destroy; override;

    function QueryInterface(const IID: TGUID; out Obj): HRESULT; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  end;

  TRRPGDownloadJob = class(TObject)
  private
    FURL: String;
    FUsuario: String;
    FSenha: String;
    FHandleNotificar: HWND;
    FGetterAtual: TThreadURLGetter;
  protected
    procedure LiberarThreadGetterURL; virtual;
  public
    constructor Create(const URL, Usuario, Senha: String; HandleNotificar: HWND);
    destructor Destroy; override;

    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
  end;

  TRRPGURLGetter = class(TObject)
  private
    FOnInicio: TNotifyEvent;
    FOnErro: TNotifyEvent;
    FOnFim: TNotifyEvent;
    FOnProgresso: TURLGetterProgressoEvento;
    FOnDados: TURLGetterDadosEvento;
    FAbortado: Boolean;

    FDownloadJob: TRRPGDownloadJob;
    FHandleComunicar: HWND;
  protected
    procedure WindowMethodInterno(var Message: TMessage); virtual;
    procedure LiberarDownloadJob; virtual;

    procedure WMUIGErro(var Message: TMessage); message WM_UIG_ERRO;
    procedure WMUIGDados(var Message: TMessage); message WM_UIG_DADOS;
    procedure WMUIGInicio(var Message: TMessage); message WM_UIG_INICIO;
    procedure WMUIGProgresso(var Message: TMessage); message WM_UIG_PROGRESSO;
    procedure WMUIGFim(var Message: TMessage); message WM_UIG_FIM;    
  public
    constructor Create;
    destructor Destroy; override;

    procedure CarregarURL(const URL: String; const Usuario: String = '';
                                             const Senha: String = '');
    procedure PararDownload;

    property Abortado: Boolean read FAbortado;
    property OnInicio: TNotifyEvent read FOnInicio write FOnInicio;
    property OnErro: TNotifyEvent read FOnErro write FOnErro;
    property OnFim: TNotifyEvent read FOnFim write FOnFim;
    property OnProgresso: TURLGetterProgressoEvento read FOnProgresso write FOnProgresso;
    property OnDados: TURLGetterDadosEvento read FOnDados write FOnDados;
  end;

const
  MAX_JOBS = 5;

implementation

var
  LJobs: TList; // TList de TRRPGDownloadJob
  LThreads: TList; // TList de TThreadURLGetter
  ScJobs: TCriticalSection;

procedure RemoverDownloadJob(J: TRRPGDownloadJob);
begin
  ScJobs.Enter;

  try
    LJobs.Remove(J);
  finally
    ScJobs.Leave;
  end;
end;

procedure AddDownloadJob(J: TRRPGDownloadJob);
var
  CriarThread: Boolean;
begin
  CriarThread := False;
  ScJobs.Enter;

  try
    if LThreads.Count < MAX_JOBS then
    begin
      CriarThread := True;
      J.FGetterAtual := TThreadURLGetter.Create(False, J);
    end;

    if not CriarThread then
      LJobs.Add(J);
  finally
    ScJobs.Leave;
  end;
end;

{ TURLImageGetter }

constructor TThreadURLGetter.Create(ASuspended: Boolean; Job: TRRPGDownloadJob);
begin
  FSecaoCritica := TCriticalSection.Create;
  FDownloadJob := Job;
  Self.FreeOnTerminate := True;
  inherited Create(ASuspended);
end;

destructor TThreadURLGetter.Destroy;
begin
  FreeAndNil(FSecaoCritica);
  inherited;
end;

procedure TThreadURLGetter.Execute;
begin
  inherited;

  try
    ScJobs.Enter;

    try
      LThreads.Add(Self);
    finally
      ScJobs.Leave;
    end;

    try
      while FDownloadJob <> nil do
      begin
        Self.ProcessarJob();
        Sleep(1);

        ScJobs.Enter;

        try
          if LJobs.Count > 0 then
          begin
            FDownloadJob := LJobs[0];
            FDownloadJob.FGetterAtual := Self;
            RemoverDownloadJob(FDownloadJob);
          end else
            FDownloadJob := nil;
        finally
          ScJobs.Leave;
        end;
      end;
    finally
      ScJobs.Enter;

      try
        LThreads.Remove(Self);
      finally
        ScJobs.Leave;
      end;
    end;
  except
    // Ignorar qualquer exceo
  end;
end;

function TThreadURLGetter.Authenticate(var hwnd: HWnd; var szUserName,
  szPassWord: LPWSTR): HResult;
begin
  if FDownloadJob <> nil then
  begin
    if (FDownloadJob.FUsuario <> '') then
    begin
      hwnd := 0;
      szUserName := PWideChar(WideString(FDownloadJob.FUsuario));
      szPassWord := PWideChar(WideString(FDownloadJob.FSenha));
    end;
  end;

  Result := S_OK;
end;

function TThreadURLGetter.GetBindInfo(out grfBINDF: DWORD;
  var bindinfo: TBindInfo): HResult;
begin
  Result := E_NOTIMPL;
end;

function TThreadURLGetter.GetPriority(out nPriority): HResult;
begin
  Result := E_NOTIMPL;
end;

function TThreadURLGetter.OnDataAvailable(grfBSCF, dwSize: DWORD;
  formatetc: PFormatEtc; stgmed: PStgMedium): HResult;
begin
  Result := S_OK;
end;

function TThreadURLGetter.OnLowResource(reserved: DWORD): HResult;
begin
  Result := E_NOTIMPL;
end;

function TThreadURLGetter.OnObjectAvailable(const iid: TGUID;
  punk: IInterface): HResult;
begin
  Result := E_NOTIMPL;
end;

function TThreadURLGetter.OnProgress(ulProgress, ulProgressMax,
  ulStatusCode: ULONG; szStatusText: LPCWSTR): HResult;
begin
  if FAbortar then
  begin
    Result := E_ABORT;
    Exit;
  end;

  case ulStatusCode of
    BINDSTATUS_DOWNLOADINGDATA: Self.SafeNotify(WM_UIG_PROGRESSO, ulProgress, ulProgressMax);
    BINDSTATUS_CACHEFILENAMEAVAILABLE: FTempFileLocalizacao := szStatusText;
  end;

  Result := S_OK;
end;

function TThreadURLGetter.OnStartBinding(dwReserved: DWORD;
  pib: IBinding): HResult;
begin
  Result := E_NOTIMPL;
  Self.SafeNotify(WM_UIG_INICIO, 0, 0);
end;

function TThreadURLGetter.OnStopBinding(hresult: HResult;
  szError: LPCWSTR): HResult;
begin
  Result := E_NOTIMPL;

  if FTempFileLocalizacao <> '' then
    Self.NotificarDados();

  Self.SafeNotify(WM_UIG_FIM, 0, 0);
end;

function TThreadURLGetter._AddRef: Integer;
begin
  Result := -1; // No importa o AddRef e Release nesta interface
end;

function TThreadURLGetter._Release: Integer;
begin
  Result := -1; // No importa o AddRef e Release nesta interface
end;

function TThreadURLGetter.QueryInterface(const IID: TGUID;
  out Obj): HRESULT;
begin
  if GetInterface(IID, Obj) then
    Result := 0
  else
    Result := E_NOINTERFACE;
end;

procedure TThreadURLGetter.NotificarErro;
begin
  FSecaoCritica.Enter;

  try
    if FDownloadJob <> nil then
      Windows.PostMessage(FDownloadJob.FHandleNotificar, WM_UIG_ERRO, 0, 0);
  finally
    FSecaoCritica.Leave;
  end;
end;

procedure TThreadURLGetter.SafeNotify(Msg: Cardinal; lParam,
  wParam: Integer);
begin
  FSecaoCritica.Enter;

  try
    if FDownloadJob <> nil then
      Windows.PostMessage(FDownloadJob.FHandleNotificar, Msg, lParam, wParam);
  finally
    FSecaoCritica.Leave;
  end;
end;

procedure TThreadURLGetter.NotificarDados;
var
  H: Cardinal;
begin
  FSecaoCritica.Enter;

  try
    if FDownloadJob = nil then
      H := 0
    else
      H := FDownloadJob.FHandleNotificar;
  finally
    FSecaoCritica.Leave;
  end;

  if H <> 0 then
    Windows.SendMessage(H, WM_UIG_DADOS, 0, Integer(PChar(FTempFileLocalizacao)));
end;

function TThreadURLGetter.CapturarImagem: Boolean;
var
  AnsiURL: AnsiString;
begin
  FAbortar := False;
  FTempFileLocalizacao := '';

  FSecaoCritica.Enter;

  try
    AnsiURL := AnsiString(FDownloadJob.FURL);
  finally
    FSecaoCritica.Leave;
  end;

  Result := URLMon.URLOpenStreamA(nil, PAnsiChar(AnsiURL), 0, Self) = S_OK;
end;

{ TRRPGURLGetter }

procedure TRRPGURLGetter.CarregarURL(const URL: String; const Usuario: String = '';
                                                        const Senha: String = '');
begin
  Self.LiberarDownloadJob();
  FAbortado := False;

  if Trim(URL) <> '' then
  begin
    FDownloadJob := TRRPGDownloadJob.Create(URL, Usuario, Senha, FHandleComunicar);

    if Assigned(FOnInicio) then
      FOnInicio(Self);    
  end;
end;

constructor TRRPGURLGetter.Create;
begin
  FHandleComunicar := AllocateHWnd(Self.WindowMethodInterno);
end;

destructor TRRPGURLGetter.Destroy;
begin
  Self.LiberarDownloadJob();
  DeallocateHWnd(FHandleComunicar);
  inherited;
end;

procedure TRRPGURLGetter.LiberarDownloadJob;
begin
  if FDownloadJob <> nil then
    FreeAndNil(FDownloadJob);
end;

procedure TRRPGURLGetter.PararDownload;
begin
  Self.LiberarDownloadJob();

  if Assigned(FOnFim) then
    FOnFim(Self);  
end;

procedure TRRPGURLGetter.WindowMethodInterno(var Message: TMessage);
begin
  case Message.Msg of
    WM_UIG_ERRO, WM_UIG_INICIO, WM_UIG_FIM, WM_UIG_PROGRESSO, WM_UIG_DADOS:
      Self.Dispatch(Message);

    else
      Message.Result := DefWindowProc(FHandleComunicar, Message.Msg,
                                      Message.WParam, Message.LParam);
  end;
end;

procedure TRRPGURLGetter.WMUIGDados(var Message: TMessage);
var
  Arq: String;
begin
  if Assigned(FOnDados) then
  begin
    Arq := PChar(Message.LParam);  
    FOnDados(Arq);
  end;
end;

procedure TRRPGURLGetter.WMUIGErro(var Message: TMessage);
begin
  if Assigned(FOnErro) then
    FOnErro(Self);
end;

procedure TRRPGURLGetter.WMUIGFim(var Message: TMessage);
begin
  if Assigned(FOnFim) then
    FOnFim(Self);
end;

procedure TRRPGURLGetter.WMUIGInicio(var Message: TMessage);
begin
  FAbortado := False;
end;

procedure TRRPGURLGetter.WMUIGProgresso(var Message: TMessage);
begin
  if Assigned(FOnProgresso) then
    try
      FOnProgresso(Message.WParam, Message.LParam);
    except
      on E: EAbort do
      begin
        FAbortado := True;
        Self.PararDownload();

        {if Assigned(FOnErro) then
          FOnErro(Self);}
      end;

      else
        raise;
    end;
end;

{ TRRPGDownloadJob }

procedure TRRPGDownloadJob.AfterConstruction;
begin
  inherited;
  AddDownloadJob(Self);
end;

procedure TRRPGDownloadJob.BeforeDestruction;
begin
  Self.LiberarThreadGetterURL();
  inherited;
end;

constructor TRRPGDownloadJob.Create(const URL, Usuario, Senha: String;
  HandleNotificar: HWND);
begin
  FURL := URL;
  FUsuario := Usuario;
  FSenha := Senha;
  FHandleNotificar := HandleNotificar;
end;

destructor TRRPGDownloadJob.Destroy;
begin
  RemoverDownloadJob(Self);
  Self.LiberarThreadGetterURL();
  inherited;
end;

procedure TRRPGDownloadJob.LiberarThreadGetterURL;
begin
  if FGetterAtual <> nil then
  begin
    FGetterAtual.FSecaoCritica.Enter;

    try
      FGetterAtual.FAbortar := True;
      FGetterAtual.FDownloadJob := nil;
      FGetterAtual.FreeOnTerminate := True;
      FGetterAtual.Terminate;
    finally
      FGetterAtual.FSecaoCritica.Leave;
    end;

    FGetterAtual := nil;
  end;
end;

procedure TThreadURLGetter.ProcessarJob;
begin
  try
    try
      try
        if not Self.CapturarImagem() then
          Self.NotificarErro();
      except
        Self.NotificarErro();
      end;
    finally
      FSecaoCritica.Enter;

      try
        if FDownloadJob <> nil then
        begin
          FDownloadJob.FGetterAtual := nil;
          FDownloadJob := nil;
        end;
      finally
        FSecaoCritica.Leave;
      end;
    end;
  except
    // Ingnorar Exceptions
  end;
end;

procedure EsperarThreads;
var
  Arr: array of Cardinal;
  i: Integer;
  T: TThreadURLGetter;
begin
  ScJobs.Enter;

  try
    SetLength(Arr, LThreads.Count);

    for i := LThreads.Count - 1 downto 0 do
    begin
      T := LThreads[i];
      Arr[i] := T.Handle;
      T.FAbortar := True;
    end;
  finally
    ScJobs.Leave;
  end;

  if Length(Arr) > 0 then
    Windows.WaitForMultipleObjects(Length(Arr), @Arr[0], True, 2000);
end;

initialization
  LJobs := TList.Create;
  LThreads := TList.Create;
  ScJobs := TCriticalSection.Create;

finalization
  EsperarThreads();

  ScJobs.Free;
  LThreads.Free;
  LJobs.Free;

end.
