unit RRPGUtil;

interface

uses
  Windows, SysUtils, Classes, Variants;

type
  TRRPGUtil = class(TObject)
  public
    class procedure FreeTList(var Lista: TList);
    class procedure LimparTList(Lista: TList);
    class procedure InterpretarParametros(const S: String; Em: TStringList);
    class procedure ProcessarMsgs;
    class procedure ForcarJanelaApp(Handle: Cardinal; const Forcar: Boolean = True);
    class function LerDoArquivo(const Arq: String): String;
    class function UsuarioDoWindows: String;
  end;

function BoolToStrSN(const Valor: Boolean): String;
function IsNull(V: Variant): Boolean;
function AparentaSerEmail(const Email: String): Boolean;

var
  IsMaiorIgualWindows7: Boolean = False;
  IsMaiorIgualWinVista: Boolean = False;

function TamanhoToStr(const Tam: Int64): String;

implementation

function AparentaSerEmail(const Email: String): Boolean;
var
  Temp, Temp2: String;
  PArroba, PPonto: Integer;
begin
  Temp := Trim(Email);

  if Temp <> '' then
  begin
    PArroba := Pos('@', Temp);

    if (PArroba > 1) and (PArroba < Length(Email)) then
    begin
      Temp2 := Copy(Temp, PArroba + 1, Length(Email));
      PPonto := Pos('.', Temp2);
      Result := (PPonto > 1) and (PPonto < Length(Temp2));

      if Result then
        Result := (Pos(';', Temp) = 0) and (Pos(',', Temp) = 0);
    end else
      Result := False;
  end else
    Result := False;
end;

function TamanhoToStr(const Tam: Int64): String;
begin
  if Tam < 1024 then
    Result := Format('%d bytes', [Tam])
  else
    if Tam < 1024 * 1024  then
      Result := Format('%2.2f kb', [Tam / 1024])
  else
    if Tam < 1024 * 1024 * 1024  then
      Result := Format('%2.2f mb', [Tam / (1024 * 1024)])
  else
    Result := Format('%2.2f gb', [Tam / (1024 * 1024 * 1024)]);
end;

procedure InicializarVersaoWindows;
var
  V: TOSVersionInfoEx;
begin
  FillChar(V, SizeOf(V), 0);
  V.dwOSVersionInfoSize := SizeOf(V);
  Windows.GetVersionEx(V);

  if V.dwMajorVersion >= 6 then
  begin
    IsMaiorIgualWinVista := True;

    if V.dwMajorVersion = 6 then
    begin
      if V.dwMinorVersion >= 1 then
        IsMaiorIgualWindows7 := True;
    end else
      IsMaiorIgualWindows7 := True;
  end;
end;

function BoolToStrSN(const Valor: Boolean): String;
begin
  if Valor then
    Result := 'S'
  else
    Result := 'N';
end;

function IsNull(V: Variant): Boolean;
begin
  Result := (V = Null) or (VarIsEmpty(V));
end;

{ TRRPGUtil }

class procedure TRRPGUtil.FreeTList(var Lista: TList);
begin
  if Lista = nil then
    Exit;

  try
    TRRPGUtil.LimparTList(Lista);
  finally
    Lista.Free;
    Lista := nil;
  end;
end;

class procedure TRRPGUtil.InterpretarParametros(const S: String;
  Em: TStringList);
var
  i: Integer;
  C: Char;
  Temp: String;
  InString, AntAspas: Boolean;

  procedure AddTemp;
  begin
    if Temp = '' then
      Exit;

    Em.Add(Temp);
    Temp := '';
  end;
begin
  Temp := '';
  InString := False;
  AntAspas := False;

  for i := 1 to Length(S) do
  begin
    C := S[i];

    if AntAspas then
    begin
      if C = '"' then
      begin
        if InString then
          Temp := Temp + '"';

        Continue;
      end else
        InString := not InString;

      AntAspas := False;
    end;

    case C of
      ';':
        begin
          if InString then
            Temp := Temp + C
          else
            AddTemp();
        end;

      '"': AntAspas := True;

      else
        Temp := Temp + C;
    end;
  end;

  AddTemp();
end;

class procedure TRRPGUtil.ForcarJanelaApp(Handle: Cardinal; const Forcar: Boolean = True);
var
  Estilo: Cardinal;
begin
  Estilo := GetWindowLong(Handle, GWL_EXSTYLE);

  if Forcar then
    Estilo := Estilo or WS_EX_APPWINDOW
  else
    Estilo := Estilo and (not WS_EX_APPWINDOW);

  SetWindowLong(Handle, GWL_EXSTYLE, Estilo);
end;

class procedure TRRPGUtil.ProcessarMsgs;
var
  Msg: tagMSG;
begin
  while Windows.PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
end;

class function TRRPGUtil.LerDoArquivo(const Arq: String): String;
var
  F: TFileStream;
begin
  if (Arq = '') or (not FileExists(Arq)) then
  begin
    Result := '';
    Exit;
  end;

  F := TFileStream.Create(Arq, fmOpenRead or fmShareDenyNone);

  try
    F.Position := 0;

    SetLength(Result, F.Size);
    F.Read(Result[1], F.Size);
  finally
    F.Free;
  end;
end;

class procedure TRRPGUtil.LimparTList(Lista: TList);
var
  i: Integer;
begin
  for i := 0 to Lista.Count - 1 do
    TObject(Lista[i]).Free;

  Lista.Clear;
end;

class function TRRPGUtil.UsuarioDoWindows: String;
var
  Tam: Cardinal;
begin
  SetLength(Result, 512);
  Tam := Length(Result);

  if Windows.GetUserName(PChar(Result), Tam) then
    SetLength(Result, Tam)
  else
    Result := '';

  Result := Trim(Result);
end;

initialization
  InicializarVersaoWindows();

end.
