unit RRPGWinUrlImage;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, RRPGURLGetter,
  RRPGUrlPicture, StdCtrls, ExtCtrls, RRPGDrawUtil, RRPGColorScheme; // DBCtrls

type
  TRWinURLImage = class(TCustomControl)
  private
    FPicture: TURLPicture;
    FBorderStyle: TBorderStyle;
    FStretch: Boolean;
    FCenter: Boolean;
    FProporcional: Boolean;
    FIgnorarNeedRedraw: Boolean;
    FMaxBytes: Integer;
    FExibirProgresso: Boolean;

    FOnNoImageDraw: TNotifyEvent;
    FOtimizarDesenho: Boolean;
    FBoaQualidade: Boolean;
    FURLImagem: String;

    function GetPicture: TPicture;
    procedure SetMaxBytes(const Value: Integer);
    procedure SetExibirProgresso(const Value: Boolean);
    procedure SetBoaQualidade(const Value: Boolean);
    procedure SetOtimizarDesenho(const Value: Boolean);
  protected
    procedure SetURLImagem(const Value: String); virtual;

    procedure NeedRedraw; virtual;
    procedure SetBorderStyle(const Value: TBorderStyle); virtual;
    procedure SetProporcional(const Value: Boolean); virtual;

    procedure CreateParams(var Params: TCreateParams); override;

    procedure Paint; override;
    function DestRect: TRect; virtual;

    procedure SetStretch(Value: Boolean); virtual;
    procedure SetCenter(Value: Boolean); virtual;

    // Eventos do TPicture
    procedure PictureChanged(Sender: TObject); virtual;
    procedure PictureLoadedChanged(Sender: TObject); virtual;

    // Eventos Messages
    procedure Resize; override;
  protected
    procedure CarregarImagemDeURL(const URL: String); virtual;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Picture: TPicture read GetPicture;

  published
    property OnResize;
    property Color;
    property ParentColor;
    property ParentBackground;
    property URLImagem: String read FURLImagem write SetURLImagem;

    // Propertys
    property BorderStyle
      : TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Center: Boolean read FCenter write SetCenter default True;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property Proporcional
      : Boolean read FProporcional write SetProporcional default False;
    property MaxBytes: Integer read FMaxBytes write SetMaxBytes default 0;
    property OnNoImageDraw
      : TNotifyEvent read FOnNoImageDraw write FOnNoImageDraw;
    property ExibirProgresso: Boolean read FExibirProgresso write
      SetExibirProgresso default True;
    property OtimizarDesenho: Boolean read FOtimizarDesenho write
      SetOtimizarDesenho default True;
    property BoaQualidade
      : Boolean read FBoaQualidade write SetBoaQualidade default True;
  end;

implementation

uses
  GDIPlus;

procedure TRWinURLImage.CarregarImagemDeURL(const URL: String);
begin
  FPicture.URL := URL;
end;

constructor TRWinURLImage.Create(AOwner: TComponent);
begin
  if not GDIPlus.IsGDIPCarregado then
    GDIPlus.CarregarGDIPlus();

  inherited;
  ControlStyle := ControlStyle + [csOpaque, csReplicatable, csNeedsBorderPaint];
  Width := 105;
  Height := 105;
  // TabStop := True;

  ParentColor := False;
  FPicture := TURLPicture.Create;
  FPicture.OnChange := Self.PictureChanged;
  FPicture.OnPictureLoadedChange := Self.PictureLoadedChanged;
  FBorderStyle := bsSingle;
  FCenter := True;
  FExibirProgresso := True;
  FOtimizarDesenho := True;
  FBoaQualidade := True;

  Self.ControlStyle := Self.ControlStyle - [csOpaque];
end;

procedure TRWinURLImage.CreateParams(var Params: TCreateParams);
begin
  inherited;

  { with Params do
    begin
    if FBorderStyle = bsSingle then
    if NewStyleControls and Ctl3D then
    ExStyle := ExStyle or WS_EX_CLIENTEDGE
    else
    Style := Style or WS_BORDER;

    WindowClass.Style := WindowClass.Style and not(CS_HREDRAW or CS_VREDRAW);
    end; }
end;

function TRWinURLImage.DestRect: TRect;
var
  w, h, cw, ch: Integer;
  xyaspect: Double;
begin
  w := Picture.Width;
  h := Picture.Height;
  cw := ClientWidth;
  ch := ClientHeight;

  if Stretch or (FProporcional and ((w > cw) or (h > ch))) then
  begin
    if FProporcional and (w > 0) and (h > 0) then
    begin
      xyaspect := w / h;
      if w > h then
      begin
        w := cw;
        h := Trunc(cw / xyaspect);
        if h > ch then // woops, too big
        begin
          h := ch;
          w := Trunc(ch * xyaspect);
        end;
      end
      else
      begin
        h := ch;
        w := Trunc(ch * xyaspect);

        if w > cw then // woops, too big
        begin
          w := cw;
          h := Trunc(cw / xyaspect);
        end;
      end;
    end
    else
    begin
      w := cw;
      h := ch;
    end;
  end;

  with Result do
  begin
    Left := 0;
    Top := 0;
    Right := w;
    Bottom := h;
  end;

  if Center then
    OffsetRect(Result, (cw - w) div 2, (ch - h) div 2);
end;

destructor TRWinURLImage.Destroy;
begin
  FreeAndNil(FPicture);
  inherited;
end;

function TRWinURLImage.GetPicture: TPicture;
begin
  Result := FPicture.Picture;
end;

procedure TRWinURLImage.Loaded;
begin
  inherited;

  if FOtimizarDesenho then
    FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
      FProporcional);
end;

procedure TRWinURLImage.NeedRedraw;
begin
  if (not FIgnorarNeedRedraw) and not(csLoading in Self.ComponentState) then
    Self.Invalidate;
end;

procedure TRWinURLImage.Paint;
begin
  with Canvas do
  begin
    if (csDesigning in Self.ComponentState) and (not FPicture.PictureLoaded) then
    begin
      Brush.Style := bsSolid;
      Brush.Color := Color;
      FillRect(ClientRect);
    end;

    if ((not FExibirProgresso) and (not FPicture.PictureLoaded)) or
      (FPicture.URL = '') then
    begin
      if Assigned(FOnNoImageDraw) then
        FOnNoImageDraw(Self);
    end
    else
      FPicture.DesenharEm(Canvas, ClientRect, FStretch, FCenter, FProporcional);
  end;
end;

procedure TRWinURLImage.PictureChanged(Sender: TObject);
begin
  if FExibirProgresso then
    Self.NeedRedraw();
end;

procedure TRWinURLImage.PictureLoadedChanged(Sender: TObject);
begin
  if not FExibirProgresso then
    Self.NeedRedraw();
end;

procedure TRWinURLImage.Resize;
begin
  inherited;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;

procedure TRWinURLImage.SetBoaQualidade(const Value: Boolean);
begin
  if FBoaQualidade <> Value then
  begin
    FBoaQualidade := Value;
    Self.NeedRedraw;
  end;
end;

procedure TRWinURLImage.SetBorderStyle(const Value: TBorderStyle);
begin
  FBorderStyle := Value;
  Self.NeedRedraw();
end;

procedure TRWinURLImage.SetCenter(Value: Boolean);
begin
  FCenter := Value;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;

procedure TRWinURLImage.SetExibirProgresso(const Value: Boolean);
begin
  if FExibirProgresso <> Value then
  begin
    FExibirProgresso := Value;
  end;
end;

procedure TRWinURLImage.SetMaxBytes(const Value: Integer);
begin
  if FMaxBytes <> Value then
  begin
    FMaxBytes := Value;
    FPicture.MaxBytes := FMaxBytes;
  end;
end;

procedure TRWinURLImage.SetOtimizarDesenho(const Value: Boolean);
begin
  if FOtimizarDesenho <> Value then
  begin
    FOtimizarDesenho := Value;

    if not(csLoading in Self.ComponentState) then
      if FOtimizarDesenho then
        FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter,
          FStretch, FProporcional)
      else
        FPicture.OtimizarDesenhoPara(0, 0, FCenter, FStretch, FProporcional);
    // No otimizar
  end;
end;

procedure TRWinURLImage.SetProporcional(const Value: Boolean);
begin
  if FProporcional <> Value then
  begin
    FProporcional := Value;

    if not(csLoading in Self.ComponentState) then
      if FOtimizarDesenho then
        FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter,
          FStretch, FProporcional);

    Self.NeedRedraw();
  end;
end;

procedure TRWinURLImage.SetStretch(Value: Boolean);
begin
  FStretch := Value;

  if not(csLoading in Self.ComponentState) then
    if FOtimizarDesenho then
      FPicture.OtimizarDesenhoPara(Self.Width, Self.Height, FCenter, FStretch,
        FProporcional);

  Self.NeedRedraw();
end;


procedure TRWinURLImage.SetURLImagem(const Value: String);
begin
  if FURLImagem <> Value then
  begin
    FURLImagem := Value;
    Self.CarregarImagemDeURL(Value);
  end;
end;

end.
